! ===========================================================================================================
! extract_traj_mc.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180920 VH     Fix assignment of launch_secs from isc = launch_sc to ksc = launch_sc
! 20170927 JN     Change 'status of' from OTH to MSG on two calls to fill_juld_status
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
! n/a      20161002 JN  Fix to synchronize values between N_CYCLE arrays and N_MEASUREMENTS arrays.
!                       ... Improvements/Fixes with reference to the issues reported by FileChecker at GDAC.
!
!==================================================================================================	
! Notes
!
! Function to extract traj parameters for netcdf files from qc file header
!
!==================================================================================================
! NOTE: !For N_CYCLE variables, param_types (for Status and Juld) are seperated by a comma in the TABLE - TRAJ_TABLE_MC

! 1 = JULD
! 2 = LAT
! 3 = LON
! 4 = PRES
! 5 = TEMP
! 6 = PSAL
! 7 = SAT_NAME
! 8 = POS_ACY
! 9 = MCCODE
! 10= TIME DESCENT START STATUS : MC100  
! 11= TIME DESCENT END STATUS   : MC200
! 12= TIME DRIFT START STATUS   : MC250
! 13= TIME DRIFT END STATUS     : MC300
! 14= TIME ASCENT START STATUS  : MC500 
! 15= TIME ASCENT END STATUS    : MC600
! 16= TIME DEEP DESCENT END STATUS:  MC400
! 17= JULD_DEEP_ASCENT_START    : MC550
! 18= JULD_FIRST_STABILIZATION  : MC150

integer function extract_traj_mc(jcyc)
use communication                       ! module
use data_arrays                         ! module
use nctraj_arrays			! module

implicit none

!--------------------------------- Variable definitions -----------------------------------------
character(len=14), parameter :: FUNCT_NAME = 'extract_traj_mc';
integer :: iH, iT, iTS, i_look, i_found, jcyc,count_posn_blk = 0;
character(len=SZC) :: str_pqc_tmp1;
character(len=40) :: str_pqc_tmp2;   
character(len=128) :: str_pqc_tmp3;
character(len=5) :: str_pqc_tmp4;
character(len=128) :: str_nc_tmp,str_nc_tmp6 ;
integer :: i1o, i1c, i2o, i2c, i3o, i3c         ! for TECH_TABLE lines with "P,T,S" (-> 3 names & 3 values)
integer :: i1s, i2s, i2p; ! for the lines in qc files with up to 3 values
integer :: i1sp, i2sp, i3sp ,i4sp , i3p              ! for the lines in qc files with up to 3 values
integer :: measurement_param                    ! variable for p,t,s measurements
integer :: param_type !           ! Pressure = 4 , Temperature = 5 , PSaL = 6
character(len=40)  :: str_pqc_tmp2_prev
!character(len=15) :: mc_code,mc_code_prev
integer           :: mc_code,mc_code_prev
character(len=10) :: variable_type, variable_type_prev
character(len=10) :: val_tmp1,val_tmp2,val_tmp3 ! for the lines in qc files with up to 3 values
integer :: kyr,kmo,kdy,khh,kmn,ksc, idate(6),count_trajectory_params, idx
equivalence (idate(1),kyr), (idate(2),kmo),(idate(3),kdy),(idate(4),khh),(idate(5),kmn),(idate(6),ksc)
character(len=12), parameter :: sub = 'EXTRACT_TRAJ_MC'
logical :: new_pqc		        ! To reset n_measurement value , to identify processing of a new qc file,gooddate
logical :: line_found_in_table   ! indicates the MC line not matched in the look up table	
integer, parameter :: f_date(6) = (/3500,01,01,00,00,00/) ! very far future date as standard for comparison
character(len=1) :: result_juld_stat
double precision :: one_min_corrctd_juld
character(len=30) :: interim_str
extract_traj_mc=SUCCESS  ! Assuming all traj variables extracted
cnvrt_cndc_traj=.false. ;

! count_enter_traj = 3rd array dimension
! When count_enter_traj = 1 , it can read the content of the meta file and 
! profile "zero"
pos_added_flag = .false. ! Resetting position processed flag array before every extraction
MC703_count=0; !Resets the count of MC703 at the beginning of every cycle.
if(count_enter_traj .eq. 1)then
   !Get launch time and position  from meta file
   ! 1 = JULD
   ! 2 = LAT
   ! 3 = LON
   kyr = launch_year
   kmo = launch_mon
   kdy = launch_day
   khh = launch_hh
   kmn = launch_mm
   ksc = launch_sc 
   if(kyr /= 9999)double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
   call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement) !use the JULD_QC from read_pqc
   int_trajectory_parameter_measurement(MCCODE,n_measurement)=0
   !NOTE: To prevent getting double values of very long positions after decimal
   write(interim_str,*)launch_lat
   read(interim_str,*)double_trajectory_parameter_measurement(LAT_TRAJ,n_measurement)
   write(interim_str,*)launch_lon
   read(interim_str,*)double_trajectory_parameter_measurement(LONG_TRAJ,n_measurement)
   CYCLE_NUMBER_TRAJ(n_measurement)=-1; ! values from meta file
   call fill_juld_status('MET',result_juld_stat);
   call fill_posn_qc(sub,n_measurement,0)
   JULD_STATUS_TRAJ(n_measurement) = result_juld_stat
   
!   if(jcyc .eq. 0)then
!     ! Fill Start of Transmission if it exists
!     if(PREV_TRANS_START_FOUND)then
!          START_OF_TRANS(cyc_count)= prev_start_of_trans_juld
!     endif
!      CYCLE_NUMBER_INDEX(cyc_count)=jcyc;
!      DATA_MODE_TRAJ(cyc_count)='R'
!      GROUNDED_TRAJ(cyc_count)='U'
!   endif
!  first_profile_no = jcyc
endif

!Best Case scenario when we have a profile zero 
!if(jcyc .gt. 0)then
   new_pqc = .true.
   CYCLE_NUMBER_INDEX(cyc_count)=jcyc;
   !DATA_MODE_TRAJ(cyc_count)='R'
   !GROUNDED_TRAJ(cyc_count)='U'
   !write(*,*) "DBG:>>> Cycle is -" , jcyc 
   do iH=1,npqc
   
      i_look=1; i_found=0; line_found_in_table = .false.;hasTIME_reg=.false.      
      str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
      str_pqc_tmp3=trim(str_pqc_tmp1(41:));str_pqc_tmp3 = adjustl(str_pqc_tmp3);         
      str_pqc_tmp4=str_pqc_tmp1(1:5);
      
      if (trim(str_pqc_tmp2).eq.'NUMBER OF COLUMNS') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (trim(str_pqc_tmp2(3:40)).eq.'COLUMN') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (trim(str_pqc_tmp2(4:40)).eq.'COLUMN') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (trim(str_pqc_tmp2(5:40)).eq.'COLUMN') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (trim(str_pqc_tmp2(6:40)).eq.'COLUMN') then
         i_look=0;
         exit;						! exit the loop, not the program
	  elseif (trim(str_pqc_tmp2).eq.'CONFIG FLOAT MISSION' .OR. trim(str_pqc_tmp2).eq.'CONFIG_MISSION_NUMBER' ) then
         if(trim(str_pqc_tmp1(41:)) /= '-999')read(str_pqc_tmp1(41:),*)CONFIG_MISSION_NUMBER_TRAJ(cyc_count); 
      elseif (str_pqc_tmp2(1:7).eq.'CONFIG ') then  
         i_look=0;
      elseif (str_pqc_tmp2(1:20).eq.'====================') then
         i_look=0;count_posn_blk = count_posn_blk + 1;
   	     if( count_posn_blk .eq. 2 )then 
   	        exit;  
         endif
      endif                                    
                               
      if(trim(str_pqc_tmp2(1:2)) =='MC')then
         i_look=1; !This is a measurement code line
      else
      	 i_look=0; !Not a measurement code line
      endif 
      
      if(pre_deploy_last_idx > 0 .and. iH <= pre_deploy_last_idx)then
          i_look=0; ! pre-deployment measurement code line can be skipped.    
          if(trim(str_pqc_tmp2(1:2)) =='MC '.and. &
          (trim(str_pqc_tmp2(4:5))=='00' .or. trim(str_pqc_tmp2(4:5))=='50'))then ! if it is fill value lines with mandatory MC.
              if(index(str_pqc_tmp2,'(YYYYMMDDHHMMSS)')>0 .and. trim(str_pqc_tmp3)=='99999999999999')then;i_look=1;
              elseif(trim(str_pqc_tmp3) == '-999.0')then;i_look=1;endif
          endif
      endif
      if(i_look>0) then
         !str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
         !str_pqc_tmp3=trim(str_pqc_tmp1(41:));str_pqc_tmp3 = adjustl(str_pqc_tmp3);
         !str_pqc_tmp4=str_pqc_tmp1(1:5);
         do iT=1,cnt_traj_mc_table_param  ! Reading the look up table
            str_nc_tmp=trim(nc_traj_mc_params_lst(iT));
            !Get the Param Type from the table
            read(str_nc_tmp(1:10),*)param_type;    
	    read(str_nc_tmp(11:14),*)mc_code;
	    read(str_nc_tmp(27:),*)variable_type;
            call to_upper(str_pqc_tmp2); ! In case the qc file has the header line(1:40) in lower-case.
	     if(len_trim(str_pqc_tmp2)> 0 .AND. INDEX(str_pqc_tmp2, trim(qc_traj_mc_params_lst(iT)))>0) then; ! If it's a match
	          line_found_in_table = .true.       ! Line is a match with the table
	          if (variable_type == "PRES" .and. .not. hasPRES_traj)then
                      nparam_traj=nparam_traj+1
                      hasPRES_traj=.true. ; hasNCPAR_traj(1) = .true.; hasQCPAR_traj(1) = .true.; 
                  elseif (variable_type == "TEMP".and. .not. hasTEMP_traj)then
                      nparam_traj=nparam_traj+1
                      hasTEMP_traj=.true. ; hasNCPAR_traj(2) = .true.; hasQCPAR_traj(2) = .true.;
                  elseif (variable_type == "CNDC".and. .not. hasCNDC_traj)then
                      nparam_traj=nparam_traj+1
                      hasCNDC_traj=.true. ; hasNCPAR_traj(3) = .true.; hasQCPAR_traj(3) = .true.;   
                      ! This unit is from QC file CONDUCTIVITY (MS/CM) 
                      ! it has to be coverted to MHOS/M, while adding it in the NetCDF file.
                      if(INDEX(str_pqc_tmp2, 'CONDUCTIVITY (MS/CM)')>0) cnvrt_cndc_traj=.true. ; 
                  elseif (variable_type == "PSAL".and. .not. hasPSAL_traj)then
                      nparam_traj=nparam_traj+1
                      hasPSAL_traj=.true. ; hasNCPAR_traj(4) = .true.; hasQCPAR_traj(4) = .true.;
                  elseif (variable_type == "JULD")then   !If the input line contains a Time Value
                     hasTIME_reg=.true.
                  else
                     hasTIME_reg=.false.
		  endif
		  
		  !Note: This is a special case from SIO. if "fast surface mode" is activated, last rise pressue (MC690) happens after the first GPS fix.
		  !So, MC690 has to be filled only after the filling the first GPS position in Traj NC file. 
		  !Ref: Email from John Glison on 11/21/2016. Subj: "Minor change to SIO Phy files and implications to traj netCDF".
		  !Eg: float 6807, cycle 3-9.
		  if(mc_code==LAST_RISE_PRES_MC .and. fst_srf_actvtd)then
		      if(param_type==PRES_TRAJ)then
		          read(str_pqc_tmp3,*)last_rise_pres;    
		      elseif(param_type==JULD_TRAJ)then
		      	  read(str_pqc_tmp3,'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc;
		          last_rise_pres_juld=juldays1950(idate);
		          lr_yr=kyr;lr_mo=kmo;lr_dy=kdy;lr_hh=khh;lr_mn=kmn;lr_sc=ksc;
		      endif;
		      cycle; 
		  endif
		  
    	          if(index(str_pqc_tmp3,' ')>0)then  !If the input line has multiple values seperated by " "
    	             i1sp=index(trim(str_pqc_tmp3),' ');
	             if(i1sp>0) i2sp=index(trim(str_pqc_tmp3(i1sp+1:)),' ');
	             if(i2sp>0) i3sp=index(trim(str_pqc_tmp3(i1sp+i2sp+1:)),' ');
	             !if(i3sp>0) write(*,*)"i1sp,i2sp,i3sp,i4sp =",i1sp,i2sp,i3sp,i4sp
    	          endif
                  if(trim(str_pqc_tmp2_prev) .ne. trim(str_pqc_tmp2))then
		        ! If its not a new file and the previous parameter no equal to the current incoming
			! reset the n_measurement to previous value
                       
                        if( .not. new_pqc )then
			   if(mc_code_prev == mc_code .AND. variable_type_prev /= 'JULD')then
                                !write(*,*)"SUBTRACT N_MEASUREMENT_PARAM"
		                n_measurement = n_measurement - measurement_param
                           endif
                        endif
                        new_pqc = .false.
                        !write(*,*)"RESET N_MEASUREMENT"
		         measurement_param = 0; !reset measurement_param 
		  endif		   
		 
                  if(i1sp >= 0)then  ! Line has multiple values seperated by spaces 
    	             measurement_param = measurement_param + 1;
		     n_measurement = n_measurement + 1 	
		     if(hasTIME_reg)then
		        if(i1sp /= 0)then
			   if(i1sp == 5)then
			      str_nc_tmp6 = trim(str_pqc_tmp3)
		              read(str_nc_tmp6,'(i4,x,i2,x,i2,x,i2,x,i2,x,i2)')kyr,kmo,kdy,khh,kmn,ksc
			   else
		              read(str_pqc_tmp3(1:i1sp),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
			   endif
                        else !JN :Added 05/25/2016
                             if(DEBUG)then;write(msg,*) "DBG:>>> str1 is - ", trim(str_pqc_tmp1),"  ", hasTIME_reg; 
                             call wrtdbg(sub,msg);endif
                             read(str_pqc_tmp3,'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc 
		        endif
                        if(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC100 TIME STATUS")>0)then
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
                           if(kyr /= 9999)then  ! In case the date is a fill value
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate); 			   
                              call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 
			      JULD_DESCENT_START(cyc_count)=juldays1950(idate)
			      if(len(str_pqc_tmp2)>19)then
		                 JULD_DESCENT_START_STATUS(cyc_count) = str_pqc_tmp2(19:20)
				 JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			   else
			      JULD_DESCENT_START_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN  ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement)= JULD_FLAG_FOR_UNKNOWN  
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC150 TIME STATUS")>0)then
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then  ! In case the date is a fill value
		              !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 
			      !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_FIRST_STABILIZATION(cyc_count)
			      JULD_FIRST_STABILIZATION(cyc_count)=juldays1950(idate);
			      if(len(str_pqc_tmp2)>19)then
		                 JULD_FIRST_STABILIZATION_STATUS(cyc_count) = str_pqc_tmp2(19:20)
				 JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			    else
			      JULD_FIRST_STABILIZATION_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement)= JULD_FLAG_FOR_UNKNOWN
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
		        elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC200 TIME STATUS")>0)then  
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
			      !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
                              !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_DESCENT_END(cyc_count)
			      JULD_DESCENT_END(cyc_count)=juldays1950(idate);
			      if(len(str_pqc_tmp2)>19)then
		                 JULD_DESCENT_END_STATUS(cyc_count) = str_pqc_tmp2(19:20)
				 JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else
			      JULD_DESCENT_END_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement)= JULD_FLAG_FOR_UNKNOWN    
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;	
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC250 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
		              !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			   			
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_PARK_START(cyc_count)
			      JULD_PARK_START(cyc_count)=juldays1950(idate);
			      if(len(str_pqc_tmp2)>19)then
			         JULD_PARK_START_STATUS(cyc_count) = str_pqc_tmp2(19:20)
				 JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else
			      JULD_PARK_START_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN          ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement)= JULD_FLAG_FOR_UNKNOWN  
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC300 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999) then
		              !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 			   			
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_PARK_END(cyc_count)
			      JULD_PARK_END(cyc_count)=juldays1950(idate);
			      if(len(str_pqc_tmp2)>19)then
			         JULD_PARK_END_STATUS(cyc_count) = str_pqc_tmp2(19:20)
				 JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			    else
			      JULD_PARK_END_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN         ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN 
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			   ! NOTE: According to John Gilson's email, NC File should have 
			   !       MC "301" even tohugh it is not present in Phy file
			   n_measurement = n_measurement + 1 	
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)'301';
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=301
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC400 TIME STATUS")>0)then  
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
		              !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			   			
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_DEEP_DESCENT_END(cyc_count)
			      JULD_DEEP_DESCENT_END(cyc_count)=juldays1950(idate);
			      if(len(str_pqc_tmp2)>19)then
			         JULD_DEEP_DESCENT_END_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			         JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else
			      JULD_DEEP_DESCENT_END_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN    ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN 
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif			   
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC500 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
		               !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			       double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			       call fill_juld_status('OTH',result_juld_stat)
			       JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 
                               call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			       !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_ASCENT_START(cyc_count)
			       JULD_ASCENT_START(cyc_count)=juldays1950(idate);
			       if(len(str_pqc_tmp2)>19)then
			          JULD_ASCENT_START_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			          JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			       endif
			   else
			      JULD_ASCENT_START_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN        ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN  
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;	
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC550 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
			       double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			       call fill_juld_status('OTH',result_juld_stat)
			       JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 
                               call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			       JULD_DEEP_ASCENT_START(cyc_count)=juldays1950(idate);
			       if(len(str_pqc_tmp2)>19)then
			          JULD_DEEP_ASCENT_START_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			          JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			       endif
			   else
			      JULD_DEEP_ASCENT_START_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN        ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN  
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif	
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code  
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC600 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			    if(kyr /= 9999)then
		               !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			       double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			       call fill_juld_status('OTH',result_juld_stat)
			       JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			   		
                               call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			       !read(trajectory_parameter_measurement(param_type,n_measurement),*)JULD_ASCENT_END(cyc_count)
			       JULD_ASCENT_END(cyc_count)=juldays1950(idate);
			       call fill_juld_status('OTH',result_juld_stat)
			       JULD_STATUS_TRAJ(n_measurement) = result_juld_stat 
			       if(len(str_pqc_tmp2)>19)then
		                   JULD_ASCENT_END_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			           JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			       endif
			    else
			      JULD_ASCENT_END_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN    ! Meausrement happened, but unknown   
			      JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN        
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			    !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			    int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC700 TIME STATUS")>0)then  
			   if(kyr /= 9999)then
		              START_OF_TRANS(cyc_count) = juldays1950(idate)
			      ! Call the pos processing line to fill 703 values correctly
			      call add_position_to_trajmc(sub, jcyc, juldays1950(idate))
			      CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			      !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('SOT',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			   			
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      if(len(str_pqc_tmp2)>19)then
			          JULD_TRANSMISSION_START_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			          JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else
			         CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			         JULD_TRANSMISSION_START_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN ! Meausrement happened, but unknown   
			         JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN 
			         call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC702 TIME STATUS")>0)then 
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
                           if(kyr /= 9999)then
		              JULD_FIRST_MESSAGE(cyc_count) = juldays1950(idate)
			      !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('MSG',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      if(len(str_pqc_tmp2)>19)then
			         JULD_FIRST_MESSAGE_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			         JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else
			         JULD_FIRST_MESSAGE_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN  ! Meausrement happened, but unknown   
			         JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN   
			         call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif 
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC704 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
		              JULD_LAST_MESSAGE(cyc_count) = juldays1950(idate)
			      !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('MSG',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			   			
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      if(len(str_pqc_tmp2)>19)then
			         JULD_LAST_MESSAGE_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			         JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else  
			       JULD_LAST_MESSAGE_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN    ! Meausrement happened, but unknown   
			       JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN
			       call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif			   
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			 elseif(len_trim(str_pqc_tmp2)>0 .AND. INDEX(str_pqc_tmp2,"MC800 TIME STATUS")>0)then  
                           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
			   if(kyr /= 9999)then
		              END_OF_TRANS(cyc_count) = juldays1950(idate)  
			      ! Call the pos processing line to fill 703 values correctly
			      one_min_corrctd_juld = juldays1950(idate) - ONE_MIN_AS_DAY
			      ! write(msg,*) "DBG:>>> one_min_corrctd_juld =", one_min_corrctd_juld; call wrtlog(sub,msg)
			      call add_position_to_trajmc(sub, jcyc, one_min_corrctd_juld)
			      !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			      call fill_juld_status('OTH',result_juld_stat)
			      JULD_STATUS_TRAJ(n_measurement) = result_juld_stat			      			
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			      if(len(str_pqc_tmp2)>19)then
		                 JULD_TRANSMISSION_END_STATUS(cyc_count) = str_pqc_tmp2(19:20)
			         JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			      endif
			   else  
			       JULD_TRANSMISSION_END_STATUS(cyc_count) = JULD_FLAG_FOR_UNKNOWN     ! Meausrement happened, but unknown   
			       JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN   
			       call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
			   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc; !by this time, "n_measurement" could be an incremented value.
			  ! NOTE: If there are still position julds to be processed and are greater than 800's JULD
			  ! add them after 800's juld in the traj array. So call subroutine with a far future date. 
			  ! That's the intention of the following block
			   do idx = 1,ppos2
			     if(.NOT. pos_added_flag(idx))then
				n_measurement = n_measurement + 1
			        call add_position_to_trajmc(sub, jcyc, juldays1950(f_date))
			        n_measurement = n_measurement - 1
				exit
			     endif
			   enddo
			 else
		           CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
		           if(kyr /= 9999)then
			      !write(trajectory_parameter_measurement(param_type,n_measurement),*)juldays1950(idate);
			      double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
			       if(len(str_pqc_tmp2)>19)then
			          JULD_STATUS_TRAJ(n_measurement) = str_pqc_tmp2(19:20)
			       else
				  call fill_juld_status('OTH',result_juld_stat)
			          JULD_STATUS_TRAJ(n_measurement) = result_juld_stat
			       endif
                              call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
			   else
			      JULD_STATUS_TRAJ(n_measurement) = JULD_FLAG_FOR_UNKNOWN ! Meausrement happened, but unknown 
			      call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
			   endif  
			   !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;	
			   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
                        endif  
		     else !If it is not TIME
		        CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
		        if(i1sp /= 0)then
                          
			  if(param_type >=PRES_TRAJ .and. param_type <=CNDC_TRAJ )then
                 read(str_pqc_tmp3(1:i1sp),*)double_trajectory_parameter_measurement(param_type,n_measurement);               
			     if(param_type == CNDC_TRAJ .and. cnvrt_cndc_traj) then  !Check4CNDC & if required, convert the unit
                    double_trajectory_parameter_measurement(param_type,n_measurement)= &
                    double_trajectory_parameter_measurement(param_type,n_measurement)*multplr_CM2M;
                 endif   
			  endif
		       	  !write(trajectory_parameter_measurement(param_type,n_measurement),*)str_pqc_tmp3(1:i1sp);
		          !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			  int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
		      else
			  if(param_type >= PRES_TRAJ .and. param_type <=CNDC_TRAJ)then
                 read(str_pqc_tmp3,*)double_trajectory_parameter_measurement(param_type,n_measurement)
			     if(param_type == CNDC_TRAJ .and. cnvrt_cndc_traj) then  !Check4CNDC & if required, convert the unit
                   double_trajectory_parameter_measurement(param_type,n_measurement)= &
                   double_trajectory_parameter_measurement(param_type,n_measurement)*multplr_CM2M;
                 endif   
			  endif
		       	  !write(trajectory_parameter_measurement(param_type,n_measurement),*)trim(str_pqc_tmp3);
		          !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)mc_code;
			  int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
		        endif
			if(param_type == PRES_TRAJ .OR. param_type == TEMP_TRAJ .OR. param_type == PSAL_TRAJ .OR. param_type == CNDC_TRAJ )then
			   if(INDEX(str_pqc_tmp3, '-999.0') >0)double_trajectory_parameter_measurement(param_type,n_measurement)&  
			   =FILLNC
			endif
			if(double_trajectory_parameter_measurement(param_type,n_measurement) /= FILLNC)then
			   if(param_type == PRES_TRAJ)pres_qc_traj(n_measurement) = '0' ! Indicates "Not QCed" yet
			   if(param_type == TEMP_TRAJ)temp_qc_traj(n_measurement) = '0' ! Indicates "Not QCed" yet
			   if(param_type == PSAL_TRAJ)psal_qc_traj(n_measurement) = '0' ! Indicates "Not QCed" yet
			   if(param_type == CNDC_TRAJ)cndc_qc_traj(n_measurement) = '0' ! Indicates "Not QCed" yet
			endif
		     endif
    	          endif 
                  i1sp = 0;			!Needs to be reset to encounter the next DRIFT PRESSURE
                  i2sp = 0;			!Needs to be reset to encounter the next DRIFT PRESSURE
                  i3sp = 0;			!Needs to be reset to encounter the next DRIFT PRESSURE
                  str_pqc_tmp2_prev = str_pqc_tmp2
		  mc_code_prev = mc_code
                  variable_type_prev = variable_type
		  !exit  ! Once a match is found in the table, no need to continue the loop with this line from file
	        endif
           end do !loop for lookup table 
           if((.not. line_found_in_table) .and. trim(str_pqc_tmp2(1:2)).eq. 'MC')then
                if(DEBUG)then
                    write(msg,'(a20,": ",a20, "Measurement Code line not in TRAJ_TABLE_MC")')qcfile, trim(str_pqc_tmp2); 
		            write(MSM,'(a)')trim(msg);	                	
                    write(errmsg(1),'("FATAL ERROR 493 DBG=T NCTRAJ in extract_traj_mc")')
                    write(errmsg(2),'("FATAL ERROR 493 DBG=T NCTRAJ Measurement Code line not in TRAJ_TABLE_MC")');
                    write(errmsg(3),'("FATAL ERROR 493 Lind: ",a)')trim(str_pqc_tmp2);
                    call error_exit(sub,493,3,errmsg); ! DBG=T extract_traj_mc DBG=T MC line not in TRAJ_TABLE_MC
                else
                    write(msg,'(a20,": ",a20, "Measurement Code line not in TRAJ_TABLE_MC")')qcfile, trim(str_pqc_tmp2); 
		            write(MSM,'(a)')trim(msg);	                	
                    write(errmsg(1),'("NON-FATAL ERROR 393 DBG=F NCTRAJ in extract_traj_mc")')
                    write(errmsg(2),'("NON-FATAL ERROR 393 DBG=F NCTRAJ Measurement Code line not in TRAJ_TABLE_MC")');
                    write(errmsg(3),'("NON-FATAL ERROR 393 Line: ",a)')trim(str_pqc_tmp2);
                    call error_report(sub,393,3,errmsg); ! DBG=F extract_traj_mc DBG=F MC line not in TRAJ_TABLE_MC
                endif
           endif 
       endif ! ending if(i_look>0)
    end do   ! loop for qc file
 
  go to 999; ! Skip to THE_END
  
960 continue
    extract_traj_mc=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 294 NCTRAJ in extract_traj_mc")')
    write(errmsg(2),'("NON-FATAL ERROR 294 internal read")');
    write(errmsg(3),'("NON-FATAL ERROR 294 found premature end")')
    call error_report(sub,294,3,errmsg); ! extract_traj_mc premature end
    go to 999; ! Skip to THE_END
961 continue
    extract_traj_mc=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 295 internal read")');
    write(errmsg(2),'("NON-FATAL ERROR 295 error occurred (e.g. format inconsistent)")')
    call error_report(sub,295,2,errmsg); ! extract_traj_mcinconsistent format

999 continue ! This is THE_END

return;

! ===========================================================================================================
end function extract_traj_mc
! ===========================================================================================================
