! ===========================================================================================================
! integer function extract_tech.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!==================================================================================================
! Function to extract tech parameters for netcdf files from qc file header
!
! uses: qc_tech_params_lst, nc_tech_params_lst, cnt_tech_table_param    (from extract_tech.f90)
!       npqc, pqcline                                                   (from read_pqc.f90)
!==================================================================================================

integer function extract_tech(icyc)

use communication                       ! module
use data_arrays                         ! module
use nctech_arrays                       ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=14),parameter :: FUNCT_NAME = 'extract_tech';
integer :: iH, iT, iTS, i_look, i_found, icyc;
character(len=SZC) :: str_pqc_tmp1;
character(len=40) :: str_pqc_tmp2;
character(len=128) :: str_pqc_tmp3;
character(len=5) :: str_pqc_tmp4;
character(len=128) :: str_nc_tmp;
integer :: i1o, i1c, i2o, i2c, i3o, i3c         ! for TECH_TABLE lines with "P,T,S" (-> 3 names & 3 values)
integer :: i1s, i2s, i1p, i2p, i3p              ! for the lines in qc files with up to 3 values
character(len=10) :: val_tmp1,val_tmp2,val_tmp3 ! for the lines in qc files with up to 3 values

integer :: result = 0; ! number of table values read
character(len=16),parameter :: sub = 'extract_tech    '
character(len=15),parameter :: typ = 'NCTECH EXTTECH'

! ===========================================================================================================
! executable statements

if(DEBUG)then; write(msg,'("==== ",a,"BEG ==== icyc: ",i3," ===============")')typ,icyc; call wrtdbg(sub,msg); endif

extract_tech=SUCCESS  ! Assuming all tech variables extracted

do iH=1,npqc
   i_look=1; i_found=0;
   if(DEBUG) write(DBG,'("DBG NCTECH ext icyc: ", i4," iH: ",i4," line: ",a)')icyc,iH,trim(str_pqc_tmp2)
   if (str_pqc_tmp2(1:40).eq.'=======================================') then
	go to 999;  ! If it's the starting of position block, no need to extract anymore
   endif
   do iTS=1,cnt_tech_table_skip_param
      str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
      call to_upper(qc_tech_skip_lst(iTS));call to_upper(str_pqc_tmp2); 
      if (trim(str_pqc_tmp2).eq.'NUMBER OF COLUMNS') then
         i_look=0;
         exit;						! exit the loop, not the program
      elseif (INDEX(trim(str_pqc_tmp2),'. COLUMN')> 0) then 
      	 i_look=0;
         exit;
      elseif (str_pqc_tmp2(1:6).eq.'CONFIG') then
         i_look=0;
         exit;  	
      elseif (str_pqc_tmp2(1:5).eq.'CALIB') then
         i_look=0;
         exit; 
      elseif (str_pqc_tmp2(1:20).eq.'====================') then
         i_look=0;
         exit   					! exit the loop, not the program
      elseif (str_pqc_tmp2(1:2).eq.'MC') then
         i_look=0;
         exit   					! exit the loop, not the program
      elseif (trim(str_pqc_tmp2).eq.trim(qc_tech_skip_lst(iTS))) then
         i_look=0;
         exit;						! exit the loop, not the program
      endif
   enddo
   if(i_look>0) then
      str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
      str_pqc_tmp3=str_pqc_tmp1(41:168); str_pqc_tmp4=str_pqc_tmp1(1:5);
      do iT=1,cnt_tech_table_param
         str_nc_tmp=nc_tech_params_lst(iT);
	 call to_upper(qc_tech_params_lst(iT));call to_upper(str_pqc_tmp2); 
         if (trim(str_pqc_tmp2).eq.trim(qc_tech_params_lst(iT))) then;
            i_found=1;
            if (index(str_nc_tmp,'(')>0) then
               i1o=index(str_nc_tmp,'('); i1c=index(str_nc_tmp,')');
               i2o=index(str_nc_tmp(i1c:128),'(')+i1c-1; i2c=index(str_nc_tmp(i1c+1:128),')')+i1c;
               i3o=index(str_nc_tmp(i2c:128),'(')+i2c-1; i3c=index(str_nc_tmp(i2c+1:128),')')+i2c;
               i1s=index(str_pqc_tmp3,' '); i1p=index(str_pqc_tmp3,'.');
               i2p=index(str_pqc_tmp3(i1p+1:128),'.')+i1p; i2s=index(str_pqc_tmp3(i2p:128),' '); 
               i3p=index(str_pqc_tmp3(i2p:128),'.');
               if(i2s==0)then
                  i2s=128;
               else
                  i2s=i2s+i2p-1;
               endif
               if(i1o>0 .and. i1c>i1o)then
                  read(str_pqc_tmp3(1:i1s),'(a)',end=960,err=961) val_tmp1
                  do while(index(val_tmp1,' ') .eq. 1)
                     val_tmp1=val_tmp1(2:10)
                  enddo
                  n_tech_param=n_tech_param+1;
                  technical_parameter_value(n_tech_param)=val_tmp1;
                  technical_parameter_name(n_tech_param)=str_nc_tmp(i1o+1:i1c-1);
                  technical_parameter_cycles(n_tech_param)=icyc;
               endif
               if(i2o>i1c .and. i2c>i2o)then;
                  read(str_pqc_tmp3(i1s:i2s),'(a)',end=960,err=961) val_tmp2
                  do while(index(val_tmp2,' ') .eq. 1)
                     val_tmp2=val_tmp2(2:10)
                  enddo
                  n_tech_param=n_tech_param+1;
                  technical_parameter_value(n_tech_param)=val_tmp2;
                  technical_parameter_name(n_tech_param)=str_nc_tmp(i2o+1:i2c-1);
                  technical_parameter_cycles(n_tech_param)=icyc;
               endif
               if(i3o>i2c .and. i3c>i3o)then;
                  read(str_pqc_tmp3(i2s:128),'(a)',end=960,err=961) val_tmp3
                  do while(index(val_tmp3,' ') .eq. 1)
                     val_tmp3=val_tmp3(2:10)
                  enddo
                  n_tech_param=n_tech_param+1;
                  technical_parameter_value(n_tech_param)=val_tmp3;
                  technical_parameter_name(n_tech_param)=str_nc_tmp(i3o+1:i3c-1);
                  technical_parameter_cycles(n_tech_param)=icyc;
               endif
            else
               n_tech_param=n_tech_param+1;
               technical_parameter_value(n_tech_param)=str_pqc_tmp3;
               technical_parameter_name(n_tech_param)=nc_tech_params_lst(iT);
               technical_parameter_cycles(n_tech_param)=icyc;
            endif
         endif
      enddo
      if(i_found==0) then
	 write(msg,'(a20,": ",a)')qcfile,trim(str_pqc_tmp2); write(MST,'(a)')trim(msg);		! NCTECH MISS_TECH
	 if(DEBUG) write(DBG,'("DBG NCTECH MISS_TECH add line: ",a)')trim(msg);			! DBG
      endif
   endif
enddo
go to 999; ! Skip to THE_END

960 continue
    extract_tech=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 280 in extract_tech")');
    write(errmsg(2),'("NON-FATAL ERROR 280 internal read")');
    write(errmsg(3),'("NON-FATAL ERROR 280 found premature end")')
    call error_report(sub,280,3,errmsg); ! extract_tech premature end
    go to 999; ! Skip to THE_END
961 continue
    extract_tech=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 281 in extract_tech")');
    write(errmsg(2),'("NON-FATAL ERROR 281 internal read")');
    write(errmsg(3),'("NON-FATAL ERROR 281 error occurred (e.g. format inconsistent)")')
    call error_report(sub,281,3,errmsg); ! extract_tech inconsistent format

999 continue ! This is THE_END

if(DEBUG)then; write(msg,'("==== ",a,"<<<<< Leave ",a," ret= ",i3)')typ,sub,extract_tech;call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== ",a,"END ====================")')typ;call wrtdbg(sub,msg); endif

return;
end function extract_tech
! ===========================================================================================================
