! ===========================================================================================================
! integer function get_all_qc.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180422 VH     No tech params: eliminate extraneous warning, write message to forlog instead of error
! 20180417 VH     General WARNING revisions.
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! load all qc files of a float from cycle 0 to maximum cycle number
! extract data as needed for meta, tech and/or traj file
! ===========================================================================================================

integer function get_all_qc(pathname)

use communication                       ! module
use data_arrays                         ! module
use ncmeta_arrays                       ! module
use nctech_arrays                       ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=*), intent(in) :: pathname

character(len=256) :: prevqc, stem
character(len=4) :: testch
integer, parameter :: maxback = 3; ! maximum number of profiles to go back
integer :: ios, ic,  endcyc, icyc, prvcyc
logical :: file_exist
logical :: first_read = .false.		! Controls reading profile data - get_all_qc NEVER reads the profile data
!logical :: is_descending = .false.	! Is a profile descending - most are not, deep_solo & a few others are
character(len=10), parameter :: sub = 'get_all_qc'
integer :: read_pqc 			! declare the function actually loads the data
integer :: prepare_launch_config;	! declare the funcdtion that preps meta launch config information
integer :: extract_tech,extract_config,extract_traj,extract_traj_mc 	! declare the functions for data extraction
FOUND_P_POS = .false.;
FOUND_P_PROF = .false.;

! ===========================================================================================================
! executable statements

get_all_qc = FAILURE; 					! Assume failure
cyc_count = 0    ! Resetting cycle count for the new float - JN
prev_good_lat = FILL; prev_good_lon = FILL; prev_good_posn_date = FILL14CH; prev_posn_found=.false.
has_irid_positioning = .false.  !Resetting the positioing indicator flag
hasMC_qc=.false.; prev_hasMC_qc=.false.;

if (ncmeta_noprof)then ! case to extract sensor and param info from meta file when there is no profiles at all.
    call extract_sensor_meta_data(sub); ! Call to extract ascii meta sensor details
    call extract_parameter_meta_data(sub); ! Call to extract ascii meta param details
    get_all_qc = SUCCESS;				! Signal success
    if(DBGIO) call dbg_leave(sub,get_all_qc);
    return;
endif

if(DBGIO) call dbg_enter(sub);
if(DEBUG)then; write(msg,'("NC get_all_qc at 1 pathname: ",a)')trim(pathname); call wrtlog(sub,msg); endif
ic=len(trim(pathname))
stem=trim(pathname);

testch=pathname(ic-5:ic-3);
if(index(testch,"D") > 0)then
    testch=pathname(ic-6:ic-4);  ! it is a Descending profile.
elseif(pathname(ic-6:ic-6) /= '_' .and. pathname(ic-7:ic-7)== '_')then;
    testch=pathname(ic-6:ic-3); ! cycle > 999
endif
read(testch,*) endcyc;

if(DEBUG)then; write(msg,'("NC get_all_qc thisqc: ",a)')trim(pathname); call wrtlog(sub,msg); endif

! Set the defaults now, in case no qc file is found

found_pres_mrvsp = .false.; pres_mrvsp = FILL;

if (make_ncmeta)then
    ret = prepare_launch_config(-1)  ! Call to extract ascii meta config details !JN
    call extract_sensor_meta_data(sub); ! Call to extract ascii meta sensor details
    call extract_parameter_meta_data(sub); ! Call to extract ascii meta param details
endif
do icyc=0,endcyc
   write(tcyc,'(i3.3)')icyc;
   qc_descending = .false. 
   prevqc = trim(pathname_aomlid_qcdir) // compose_profname(sub,float_name,icyc,".qc");      ! Looks for ascending file
   inquire(file=prevqc,exist=file_exist)
   if(.not. file_exist)then
       if(DEBUG)then;
           write(msg,'("NC get_all_qc, icyc: ",i4," file prevqc: ",a,"DOES NOT EXIST, looking for Desecending file")')&
           icyc,trim(prevqc);
           call wrtlog(sub,msg); ! DBG
       endif
       prevqc = trim(pathname_aomlid_qcdir) // compose_profname(sub,float_name,icyc,"D.qc");  ! Looks for descending file
       inquire(file=prevqc,exist=file_exist)
       if(file_exist) qc_descending = .true.   ! Sets the qc -file desecnding attribute
   endif
   
   if(DEBUG)then; write(msg,'("NC get_all_qc loop icyc: ",i4," prevqc: ",a)')icyc,trim(prevqc); call wrtlog(sub,msg); endif
   inquire(file=prevqc,exist=file_exist)
   if(file_exist)then
      if(DEBUG)then; write(msg,'("NC prevqc: ",a," EXISTS, call read_pqc")')trim(prevqc);   call wrtdbg(sub,msg); endif

      ret = read_pqc(sub,prevqc,first_read);  ! pqcline has header data above,positions $are in: prev_year, prev_lon, ...

      if(DEBUG)then; write(msg,'("NC get_all_qc, ret=",i2," is returned from read_pqc")')ret; call wrtlog(sub,msg); endif
      if(ret == FAILURE)then
         write(msg,'("previsous qcfile: ",a," POS NOT RETRIEVED")')trim(prevqc); call wrttrc(sub,msg);
      endif
      if(DEBUG) write(DBG,'("=====================================================================================")');
      if(DEBUG) write(DBG,'("=== cycle ",i3.3," hasMC_qc: ",l1)')icyc, hasMC_qc;
      if(DEBUG) write(DBG,'("=== cycle ",i3.3," pref/_qc: ",l1)')icyc, prev_hasMC_qc;
      if(make_nctraj .and. cyc_count > 0 .and. (hasMC_qc .neqv. prev_hasMC_qc))then !MC mix-up, Traj NC file won't be right.
	 if(DEBUG) write(DBG,'("=====================================================================================")');
	 if(DEBUG) write(DBG,'("-------------------------------------------------------------------------------------")');
	 write(msg,'("previsous qcfile: ",a," MC Non-MC mix up in cycle in ")')trim(prevqc); call wrttrc(sub,msg);
	 write(errmsg(1),'("NON-FATAL ERROR 231 in get_all_qc for float: ",a)')trim(profile);
	 write(errmsg(2),'("NON-FATAL ERROR 231 MC/Non-MC mix up found in prevqc: ",a)')trim(prevqc);
	 write(errmsg(3),'("NON-FATAL ERROR 231 Traj NC file is not created. ")');
	 call error_report(sub,231,2,errmsg);
	 make_nctraj = .false.; ! we do not need to keep extracting the traj part
	 ! The error_report call weill set the filmsg to 'f' filmsg(5:5) = 'N'; ! traj file requested, but could not be produced
      endif
      if(DEBUG) write(DBG,'("=====================================================================================")');

      ! need to decode header here ...
      cyc_count = cyc_count + 1  ! incrementing the cyc_count for every existing cycle
      if(make_ncmeta)then
         ! try to get the launch config from cycle  1 for iridium and 0 for non-iridium floats
         if(trans_type /= 'IRIDIUM' .and. icyc == 0) then
            ret = prepare_launch_config(0);
         elseif(trans_type == 'IRIDIUM' .and. icyc == 1) then
            ret = prepare_launch_config(0);
         endif
         if(DEBUG) write(DBG,'("NCMETA call extract_config for cyc: ",i3," qcfile: ",a)')icyc,trim(prevqc);
         ret=extract_config(icyc);
         if(DEBUG) write(DBG,'("NCMETA rtrn extract_config for cyc: ",i3," qcfile: ",a)')icyc,trim(prevqc);
      endif
      if (make_nctech)then;
	    if(DEBUG) write(DBG,'("NCTECH EXTTECH read tech header for cyc: ",i3," qcfile: ",a)')icyc,trim(prevqc);
	    ret=extract_tech(icyc)
	    if(DEBUG) write(DBG,'("NCTECH done with header for cyc: ",i3," ret: ",i3," n_tech_par: ",i5)')icyc,ret,n_tech_param;
      endif
      count_enter_traj=count_enter_traj+1
      if (make_nctraj)then
	if(hasMC)then
	    if(DEBUG)then; write(msg,'("NC get_all_qc TRAJ calls extract_traj_mc for icyc",i4)')icyc; call wrtlog(sub,msg); endif
	    ret=extract_traj_mc(icyc)
	else
	    if(DEBUG)then; write(msg,'("NC get_all_qc TRAJ calls extract_traj for icyc",i4)')icyc; call wrtlog(sub,msg); endif
	    ret=extract_traj(icyc)
	endif
      endif
   else
	if(DEBUG)then;
	    write(msg,'("NC get_all_qc, icyc: ",i4," file prevqc: ",a,"DOES NOT EXIST")')icyc,trim(prevqc);
	    call wrtlog(sub,msg); ! DBG
	endif
   endif
enddo

if(make_nctech .and. n_tech_param == 0)then
    make_nctech = .false.;
    write(HEDF,'("FILMSG ",a," wrt l: ",i1," var/filmsg: ")')'NCTECH',5
    filmsg(LTECH:LTECH) = 'N'; write(msg,1000)HEDF,'N',filmsg,'Cannot make requested nctech file'; call wrtlog(sub,msg); ! nctech=0
    1000 format(a,a1,a9,x,a);
endif

get_all_qc = SUCCESS;				! Signal success

if(DBGIO) call dbg_leave(sub,get_all_qc);

return;
end function get_all_qc
! ===========================================================================================================
