! ===========================================================================================================
! subroutine initialize_arrays.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180307 VH     Initialize new biop arrays
! 20171112 VH     Initialize the bioARGO arrays
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! ===========================================================================================================

subroutine initialize_arrays(caller)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in )     :: caller

character(len=14), parameter ::  sub = 'init_arrays'
integer :: ifl

! ===========================================================================================================
! executable statements

if(DEBUG)then; write(msg,'(a," is called by ",a)')sub,caller; call wrtdbg(sub,msg); endif

! initilize a few stray variables to initial values (just to be sure)

do ilev=1,MAXLEV
    FILLNCD(ilev) = FILLNC;         ! FILL data array for the nc file 
    FILLNCF(ilev) = ' ';            ! FILL flag array for the nc file - use 9=MISSING
    FILLDATA1(1,ilev) = FILLNC;		! FILL array for the nc file
    FILLCHAR1(1,ilev) = fillchar;	! for the nc file
    FILLDATA2(1,ilev) = FILLNC;         ! FILL array for the nc file 
    FILLCHAR2(1,ilev) = fillchar;       ! for the nc file
    FILLDATA2(2,ilev) = FILLNC;         ! FILL array for the nc file 
    FILLCHAR2(2,ilev) = fillchar;       ! for the nc file
enddo

do idat=1,MAXDAT
    do ilev=1,MAXLEV
	profdata(idat,ilev) = FILL		! initialize to fill value
	profflag(idat,ilev) = MISSING		! initialize to missing value flag
	do ifl=1,MAXFLAGS
	    testflag(idat,ilev,ifl) = NOTF
	enddo
    enddo
enddo

if(hasCNDC(0))then
    do iprf=1,MAXPRF
	if(hasCNDC(iprf))then
	    do ilev=1,MAXLEV
		profcndc(iprf,ilev) = FILL
	    enddo
	endif
    enddo
endif

do i=1,MAXPRF
    if(hasCKSM(i))then
	do ilev=1,nlev
	    cksm(i,ilev) = 0;
	enddo
    endif
enddo

STUCK_T = 0; STUCK_S = 0; STUCK_O = 0;	! reported as FLAG, so 0,1,4
JUMPC_T = 0; JUMPC_S = 0;		! reported as FLAG, so 0,1,4
FROZN_T = IND_NOT; FROZN_S = IND_NOT;	! reported as INDICATOR, IND_NOT=-1, could not apply

do iprf=1,MAXPRF
    do i = 2,NTESTS
	QCTEST_FLAG(iprf,i) = 0; ! So far, not performed, no flag determined
    enddo
    QCTEST_FLAG(iprf, 1) = 1;    ! float id already done, so this flag will be PERFORMED/GOOD
    QCTEST_FLAG(iprf,10) = 0;    ! bottom spike test was REMOVED, so the flage willl be NOT PERFORMED
    QCTEST_FLAG(iprf,12) = 1;    ! digit rollover done in decoder, so this flag also will be PERFORMED/GOOD
enddo

!initialize the prof_<param>_qc as 'n'
do i=1,MAXPAR
    DO iprf=1, MAXPRF
	prof_qc(i,iprf) = 'n';
    enddo
enddo

do i=1,MAXPOS; pos_onland = '0'; enddo

! initialize the results arrays
qcdataresult = -1; ! DBG TEST 0;
qcprofresult = -1; ! DBG TEST 0;
qcresult =  -1; ! DBG TEST 0;

! Initialize the array that controls STS near-surface profiles
near_surface = .false.
STS_prof = .false.;
NST_prof = .false.;
nsflag = 0;		! Near surface flag - only one flag per level, applied to T (NST) ro T & S (STS)
do_psal_prof = .true.;		! Adjust all columns IF do_psal_adj is true

if(DOXY_float)then
    ! The profbdata array holds the intermidiate/extra DOXY/BioARGO parameters, and we do not QC them
    do ilev=1,MAXLEV
	do iprf=1,MAXPRF
	    do i=1,MAXBPAR
		profbdata(i,iprf,ilev) = FILLNC         ! initialize to fill value
	    enddo
	enddo
    enddo
endif

return;
end subroutine initialize_arrays
! ===========================================================================================================
