! ===========================================================================================================
! module nctraj_arrays
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who   what
! 20181218 JN     Exclude interpolated positions in the Traj.nc file
! 20180920 CA    Change start(1)_,count(1) to start,cound
! 20180619 JN    Add new SOLODIR & ALTOIR, adjust ALAMO
! 20170927 JN    Many changes to accomodate requests from Megan in July 2017
! 20170801 JN&VH Added to check the instrument type - "APEXDIR" while filling JULD STATUS in 
!                "fill_juld_status" inside nctraj_arrays.f90
! 20170712 all   Many changes to all, some documented/tracked, some not
! 20170228 JN    Fix to synchronize values between N_CYCLE arrays and N_MEASUREMENTS arrays.
!                Improvements/Fixes with reference to the issues reported by FileChecker at GDAC.
! 20160511 all   Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:

! This module contains variable/array definitions used by write_ncbiot to create the Core-ARGO trajectory nc file.
!========================================================================================

module nctraj_arrays

  use communication					! module
  use data_arrays					! module
  use netcdf_arrays					! module

  implicit none

! ===========================================================================================================
!  include 'netcdf.inc' ! FIX is this needed - it it already included in netcdf_arrays
! ===========================================================================================================
! moedule varialbe/array declarations
! ===========================================================================================================

logical :: hasTIME_reg,hasSTATUS;

! character(len=128), dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: trajectory_parameter_measurement =FILLCHAR 
double precision, dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) ::  double_trajectory_parameter_measurement =FILLDBLE 
integer , dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: int_trajectory_parameter_measurement =FILLINT 
!character(len=1), dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: status_trajectory_parameter_measurement =FILLCHAR
character(len=1), dimension(MAX_MEASUREMENT) :: SATELLITE_NAME =FILLCHAR
character(len=1), dimension(MAX_MEASUREMENT) :: POSITION_ACCURACY=FILLCHAR

character(len=40), dimension(MAX_PARM_TABLE) :: qc_traj_mc_params_lst
character(len=128),dimension(MAX_PARM_TABLE):: nc_traj_mc_params_lst  !NC Traj MC Parameters
integer :: cnt_traj_mc_table_param = 0                                !Count of params from Traj MC table
character(len=1), parameter      ::  JULD_FLAG_FOR_UNKNOWN = '9'
double precision, parameter :: ONE_MIN_AS_DAY = 0.000694454  ! value of 1/1440 which is  1/(24X60)

character(len=40), dimension(MAX_PARM_TABLE) ::  qc_traj_skip_lst    !non-Traj parameters
character(len=40), dimension(MAX_PARM_TABLE) ::  qc_traj_params_lst  !Traj parameters
character(len=128), dimension(MAX_PARM_TABLE) :: nc_traj_params_lst  !new NC Traj parameters
! character(len=128), dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: trajectory_parameter_measurement =FILLCHAR
logical :: pos_added_flag(MAXPOS) = .false.
character(len=128), dimension(MAXTRAJ) :: trajectory_parameter_name
integer :: trajectory_parameter_cycles(MAXTRAJ),cyc_no(MAXTRAJ)


character(len=1) :: JULD_QC_TRAJ(MAXTRAJ)=JULD_FLAG_FOR_UNKNOWN;
character(len=1) :: JULD_STATUS_TRAJ(MAXTRAJ)=JULD_FLAG_FOR_UNKNOWN;
character(len=1) :: JULD_TRANSMISSION_START_STATUS(MAXTRAJ)=JULD_FLAG_FOR_UNKNOWN
character(len=1) :: JULD_TRANSMISSION_END_STATUS(MAXTRAJ)=JULD_FLAG_FOR_UNKNOWN

! mvd to data_arrays integer :: nparam_traj;
! mvd to data_arrays logical :: hasPRES_traj,hasTEMP_traj,hasPSAL_traj;
! mvd to data_arrays logical :: hasNCPAR_traj(MAXNCPAR), hasQCPAR_traj(MAXPAR);

integer, parameter :: JULD_TRAJ=1  ! double_trajectory_parameter_measurement
integer, parameter :: LAT_TRAJ=2;  ! double_trajectory_parameter_measurement
integer, parameter :: LONG_TRAJ=3  ! double_trajectory_parameter_measurement
integer, parameter :: PRES_TRAJ=4; ! double_trajectory_parameter_measurement
integer, parameter :: TEMP_TRAJ=5; ! double_trajectory_parameter_measurement
integer, parameter :: PSAL_TRAJ=6; ! double_trajectory_parameter_measurement
integer, parameter :: SAT_NAME=1;  ! char_trajectory_parameter_measurement
!integer, parameter :: POS_ACY=2;   ! status_trajectory_parameter_measurement
integer, parameter :: MCCODE=1;    ! int_trajectory_parameter_measurement
integer, parameter :: LAST_RISE_PRES_MC=690; !MCode corresponding to the last rise pressure measurement.

integer                 ::  pres_gain_phy, temp_gain_phy, psal_gain_phy
character(len=1)        ::  pres_qc_traj (MAX_MEASUREMENT) = FILLCHAR
character(len=1)        ::  temp_qc_traj (MAX_MEASUREMENT) = FILLCHAR
character(len=1)        ::  psal_qc_traj (MAX_MEASUREMENT) = FILLCHAR
real    :: pres_traj_1d(MAX_MEASUREMENT) = FILLNC;
real    :: temp_traj_1d(MAX_MEASUREMENT) = FILLNC;
real    :: psal_traj_1d(MAX_MEASUREMENT) = FILLNC;
integer ::   CYCLE_NUMBER_TRAJ(MAXTRAJ)=FILLINT
integer ::   CYCLE_NUMBER_INDEX(MAXCYC)=FILLINT
double precision :: last_rise_pres=FILLDBLE, last_rise_pres_juld=FILLDBLE;
double precision :: first_msg_juld=FILLDBLE, last_msg_juld=FILLDBLE;
	
integer :: lr_yr,lr_mo,lr_dy,lr_hh,lr_mn,lr_sc; !intgers to store the datetime numbers of last rise measurement MC690. 
integer :: fm_yr,fm_mo,fm_dy,fm_hh,fm_mn,fm_sc; !intgers to store the datetime numbers of first message MC702
integer :: lm_yr,lm_mo,lm_dy,lm_hh,lm_mn,lm_sc; !intgers to store the datetime numbers of last message MC704 

  ! Define the dimensions in netcdf_arrays
  integer :: PROJECT_NAME_id,               PROJECT_NAME_dims;		        character(len=64) :: PROJECT_NAME;
  integer :: PI_NAME_TRAJ_id,               PI_NAME_TRAJ_dims;		        character(len=64) :: PI_NAME;   
  integer :: TRAJECTORY_PARAMETERS_id,      TRAJECTORY_PARAMETERS_dims(NDIMS2); character(len=16) :: TRAJECTORY_PARAMETERS(MAXPAR);
  integer :: DATA_STATE_INDICATOR_id,       DATA_STATE_INDICATOR_dims(NDIMS1);  character(len=4)  :: DATA_STATE_INDICATOR;
  integer :: PLATFORM_TYPE_id,              PLATFORM_TYPE_dims(NDIMS1);         character(len=32) :: PLATFORM_TYPE;
  integer :: FLOAT_SERIAL_NO_id,            FLOAT_SERIAL_NO_dims(NDIMS1);       character(len=32) :: FLOAT_SERIAL_NO;
  integer :: FIRMWARE_VERSION_id,           FIRMWARE_VERSION_dims(NDIMS1);      character(len=32) :: FIRMWARE_VERSION;
  integer :: WMO_INST_TYPE_id,              WMO_INST_TYPE_dims(NDIMS1);         character(len=4)  :: WMO_INST_TYPE;
  integer :: POSITIONING_SYSTEM_id,         POSITIONING_SYSTEM_dims(NDIMS1);    character(len=8)  :: POSITIONING_SYSTEM;

  integer :: JULD_TRAJ_id,                  JULD_TRAJ_dims(NDIMS1);     double precision :: JULD_TRAJ_ARR(MAX_MEASUREMENT)=FILLDBLE;
  integer :: JULD_STATUS_id,                JULD_STATUS_dims(NDIMS1);           
  integer :: JULD_QC_id,                    JULD_QC_dims(NDIMS1);               
  integer :: JULD_ADJUSTED_id,              JULD_ADJUSTED_dims(NDIMS1);      double precision :: JULD_ADJUSTED(MAXTRAJ)=FILLDBLE;
  integer :: JULD_ADJUSTED_STATUS_id,       JULD_ADJUSTED_STATUS_dims(NDIMS1);  
  character(len=1) :: JULD_ADJUSTED_STATUS(MAXTRAJ)=FILLCHAR;
  integer :: JULD_ADJUSTED_QC_id,           JULD_ADJUSTED_QC_dims(NDIMS1);   character(len=1) :: JULD_ADJUSTED_QC(MAXTRAJ)=FILLCHAR;
  integer :: LATITUDE_id,                   LATITUDE_dims(NDIMS1);           double precision :: LATITUDE_TRAJ(MAXTRAJ)=FILLPOSN;
  integer :: LONGITUDE_id,                  LONGITUDE_dims(NDIMS1);          double precision :: LONGITUDE_TRAJ(MAXTRAJ)=FILLPOSN;
  integer :: POSITION_ACCURACY_id,          POSITION_ACCURACY_dims(NDIMS1);      
  !character(len=1) :: POSITION_ACCURACY(MAXTRAJ)=FILLCHAR;
  integer :: POSITION_QC_id,                POSITION_QC_dims(NDIMS1);             
  integer :: CYCLE_NUMBER_id,               CYCLE_NUMBER_dims(NDIMS1);            
  integer :: CYCLE_NUMBER_ADJUSTED_id,      CYCLE_NUMBER_ADJUSTED_dims(NDIMS1);   integer :: CYCLE_NUMBER_ADJUSTED(MAXTRAJ);
  integer :: MEASUREMENT_CODE_id,           MEASUREMENT_CODE_dims(NDIMS1);        integer :: MEASUREMENT_CODE(MAXTRAJ)=FILLINT;
  integer :: AXES_ERROR_ELLIPSE_MAJOR_id,   AXES_ERROR_ELLIPSE_MAJOR_dims(NDIMS1);       
  integer :: AXES_ERROR_ELLIPSE_MINOR_id,   AXES_ERROR_ELLIPSE_MINOR_dims(NDIMS1);       
  integer :: AXES_ERROR_ELLIPSE_ANGLE_id,   AXES_ERROR_ELLIPSE_ANGLE_dims(NDIMS1);       
  integer :: SATELLITE_NAME_id,             SATELLITE_NAME_dims(NDIMS1);     !character(len=1) :: SATELLITE_NAME(MAXTRAJ)=FILLCHAR;

   integer   count,start; ! ... used to have start(1)
 
  !N_CYCLE Variables 
  double precision        ::  JULD_DESCENT_START(MAXCYC)=FILLDBLE;
  character(len=1)          ::  JULD_DESCENT_START_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_DESCENT_END(MAXCYC)=FILLDBLE;
  character(len=1)          ::  JULD_DESCENT_END_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_PARK_START(MAXCYC)=FILLDBLE;
  character(len=1)          ::  JULD_PARK_START_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_PARK_END(MAXCYC)=FILLDBLE;
  character(len=1)          ::  JULD_PARK_END_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_ASCENT_START(MAXCYC)=FILLDBLE;
  character(len=1)          ::  JULD_ASCENT_START_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision          ::  JULD_ASCENT_END(MAXCYC)=FILLDBLE;
  character(len=1)        ::  JULD_ASCENT_END_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  START_OF_TRANS(MAXCYC)=FILLDBLE;
  double precision        ::  END_OF_TRANS(MAXCYC)=FILLDBLE;
  double precision        ::  JULD_FIRST_MESSAGE(MAXCYC) = FILLDBLE
  character(len=1)        ::  JULD_FIRST_MESSAGE_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_LAST_MESSAGE(MAXCYC) = FILLDBLE
  character(len=1)        ::  JULD_LAST_MESSAGE_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  integer                 ::  CONFIG_MISSION_NUMBER_TRAJ(MAXCYC)=FILLINT

  double precision        ::  JULD_DEEP_DESCENT_END(MAXCYC) = FILLDBLE
  character(len=1)        ::  JULD_DEEP_DESCENT_END_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_FIRST_LOCATION(MAXCYC) = FILLDBLE
  character(len=1)        ::  JULD_FIRST_LOCATION_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_LAST_LOCATION(MAXCYC) = FILLDBLE
  character(len=1)        ::  JULD_LAST_LOCATION_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_FIRST_STABILIZATION(MAXCYC) = FILLDBLE ;
  character(len=1)        ::  JULD_FIRST_STABILIZATION_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  double precision        ::  JULD_DEEP_ASCENT_START(MAXCYC) = FILLDBLE
  character(len=1)        ::  JULD_DEEP_ASCENT_START_STATUS(MAXCYC)=JULD_FLAG_FOR_UNKNOWN;
  
  integer :: JULD_DESCENT_START_id,          JULD_DESCENT_START_dims (NDIMS1);   
  integer :: JULD_DESCENT_START_STATUS_id,   JULD_DESCENT_START_STATUS_dims(NDIMS1);
  integer :: JULD_FIRST_STABILIZATION_id,    JULD_FIRST_STABILIZATION_dims(NDIMS1);
  integer :: JULD_FIRST_STABILIZATION_STATUS_id,JULD_FIRST_STABILIZATION_STATUS_dims(NDIMS1);
  integer :: JULD_DESCENT_END_id,            JULD_DESCENT_END_dims(NDIMS1);
  integer :: JULD_DESCENT_END_STATUS_id,     JULD_DESCENT_END_STATUS_dims(NDIMS1);
  integer :: JULD_PARK_START_id,             JULD_PARK_START_dims(NDIMS1);
  integer :: JULD_PARK_START_STATUS_id,      JULD_PARK_START_STATUS_dims(NDIMS1);
  integer :: JULD_PARK_END_id,               JULD_PARK_END_dims(NDIMS1);
  integer :: JULD_PARK_END_STATUS_id,        JULD_PARK_END_STATUS_dims(NDIMS1);
  integer :: JULD_DEEP_DESCENT_END_id,       JULD_DEEP_DESCENT_END_dims(NDIMS1);
  integer :: JULD_DEEP_DESCENT_END_STATUS_id,JULD_DEEP_DESCENT_END_STATUS_dims(NDIMS1);
  integer :: JULD_DEEP_PARK_START_id,        JULD_DEEP_PARK_START_dims(NDIMS1);
  integer :: JULD_DEEP_PARK_START_STATUS_id, JULD_DEEP_PARK_START_STATUS_dims(NDIMS1);
  integer :: JULD_ASCENT_START_id,           JULD_ASCENT_START_dims(NDIMS1);
  integer :: JULD_ASCENT_START_STATUS_id,    JULD_ASCENT_START_STATUS_dims(NDIMS1);
  integer :: JULD_DEEP_ASCENT_START_id,      JULD_DEEP_ASCENT_START_dims(NDIMS1);
  integer :: JULD_DEEP_ASCENT_START_STATUS_id,JULD_DEEP_ASCENT_START_STATUS_dims(NDIMS1);
  integer :: JULD_ASCENT_END_id,             JULD_ASCENT_END_dims(NDIMS1);
  integer :: JULD_ASCENT_END_STATUS_id,      JULD_ASCENT_END_STATUS_dims(NDIMS1);
  integer :: JULD_TRANSMISSION_START_id,     JULD_TRANSMISSION_START_dims(NDIMS1);
  integer :: JULD_TRANSMISSION_START_STATUS_id,JULD_TRANSMISSION_START_STATUS_dims(NDIMS1);
  integer :: JULD_FIRST_MESSAGE_id,          JULD_FIRST_MESSAGE_dims(NDIMS1);
  integer :: JULD_FIRST_MESSAGE_STATUS_id,   JULD_FIRST_MESSAGE_STATUS_dims(NDIMS1);
  integer :: JULD_FIRST_LOCATION_id,         JULD_FIRST_LOCATION_dims(NDIMS1);
  integer :: JULD_FIRST_LOCATION_STATUS_id,  JULD_FIRST_LOCATION_STATUS_dims(NDIMS1);
  integer :: JULD_LAST_LOCATION_id,          JULD_LAST_LOCATION_dims(NDIMS1);
  integer :: JULD_LAST_LOCATION_STATUS_id,   JULD_LAST_LOCATION_STATUS_dims(NDIMS1);
  integer :: JULD_LAST_MESSAGE_id,           JULD_LAST_MESSAGE_dims(NDIMS1);
  integer :: JULD_LAST_MESSAGE_STATUS_id,    JULD_LAST_MESSAGE_STATUS_dims(NDIMS1);
  integer :: JULD_TRANSMISSION_END_id,       JULD_TRANSMISSION_END_dims(NDIMS1);
  integer :: JULD_TRANSMISSION_END_STATUS_id,JULD_TRANSMISSION_END_STATUS_dims(NDIMS1);
  integer :: CLOCK_OFFSET_id,                CLOCK_OFFSET_dims(NDIMS1);
  integer :: GROUNDED_id,                    GROUNDED_dims(NDIMS1);
  integer :: REPRESENTATIVE_PARK_PRESSURE_id,REPRESENTATIVE_PARK_PRESSURE_dims(NDIMS1);
  integer :: REPRESENTATIVE_PARK_PRESSURE_STATUS_id,REPRESENTATIVE_PARK_PRESSURE_STATUS_dims(NDIMS1);
  integer :: CONFIG_MISSION_NUMBER_id,       CONFIG_MISSION_NUMBER_dims(NDIMS1);
  integer :: CYCLE_NUMBER_INDEX_id,          CYCLE_NUMBER_INDEX_dims(NDIMS1);
  integer :: CYCLE_NUMBER_INDEX_ADJUSTED_id, CYCLE_NUMBER_INDEX_ADJUSTED_dims(NDIMS1);
  integer :: DATA_MODE_id,                   DATA_MODE_dims(NDIMS1);
  character(len=1) :: GROUNDED_TRAJ(MAXTRAJ)='U' !FILLCHAR;
  character(len=1) :: DATA_MODE_TRAJ(MAXTRAJ)='R'!FILLCHAR;

  !History Variables 

  integer :: HISTORY_INSTITUTION_id,                  HISTORY_INSTITUTION_dims(NDIMS2);  character(len=4)  :: HISTORY_INSTITUTION
  integer :: HISTORY_STEP_id,                         HISTORY_STEP_dims(NDIMS2);  character(len=4)  :: HISTORY_STEP
  integer :: HISTORY_SOFTWARE_id,                     HISTORY_SOFTWARE_dims(NDIMS2); character(len=4)  :: HISTORY_SOFTWARE
  integer :: HISTORY_SOFTWARE_RELEASE_id,             HISTORY_SOFTWARE_RELEASE_dims(NDIMS2);  
   character(len=4)  :: HISTORY_SOFTWARE_RELEASE ;
  integer :: HISTORY_REFERENCE_id,                    HISTORY_REFERENCE_dims(NDIMS2); 
  character(len=64) :: HISTORY_REFERENCE;
  integer :: HISTORY_DATE_id,                         HISTORY_DATE_dims(NDIMS2); 
  character(len=14) :: HISTORY_DATE ;
  integer :: HISTORY_ACTION_id,                       HISTORY_ACTION_dims(NDIMS2);  character(len=4)  :: HISTORY_ACTION
  integer :: HISTORY_PARAMETER_id,                    HISTORY_PARAMETER_dims(NDIMS2);
  character(len=16) :: HISTORY_PARAMETER;
  integer :: HISTORY_PREVIOUS_VALUE_id,               HISTORY_PREVIOUS_VALUE_dims(NDIMS2);
  integer :: HISTORY_INDEX_DIMENSION_id,              HISTORY_INDEX_DIMENSION_dims(NDIMS2);
  integer :: HISTORY_START_INDEX_id,                  HISTORY_START_INDEX_dims(NDIMS2); 
  real              :: HISTORY_START_INDEX;
  integer :: HISTORY_STOP_INDEX_id,                   HISTORY_STOP_INDEX_dims(NDIMS2); 
  real              :: HISTORY_STOP_INDEX ;
  integer :: HISTORY_QCTEST_id,                       HISTORY_QCTEST_dims(NDIMS2); character(len=16)  :: HISTORY_QCTEST


  character(len=30) dummy_var
  double precision :: resol_var
  character(len=30) :: fmt_str
  integer :: MC703_count
! ====================================================================================================
! <PARAM> id definitions
! 
! 20Jun2016 - separate ncprof & nctraj defs for valic_min,valid_max and long_name ncprof=v3.03, nctraj=v3.1
   real,parameter :: PAR_vmin(MAXPAR) = (/    0.0,    -2.5,     0.0,     2.0,     0.0    /);	! User Man 3.1
   real,parameter :: PAR_vmax(MAXPAR) = (/12000.0,    40.0,     8.5,    41.0,   600.0  /);	! User Man 3.1
    character(len=41) :: PAR_lnam(MAXPAR) =  (/ &						! User Man 3.1
	'Sea water pressure, equals 0 at sea-level', &          ! long_name PRES
	'Sea temperature in-situ ITS-90 scale     ', &          ! long_name TEMP
	'Electrical conductivity                  ', &          ! long_name CNDC
	'Practical salinity                       ', &          ! long_name PSAL
	'Dissolved oxygen                         ' /)          ! long_name DOXY

integer :: cnt_traj_table_param,cnt_traj_table_skip_param
integer :: n_measurement=1

! ====================================================================================================

  contains

! ====================================================================================================
! module functions/subroutines
! ===========================================================================================================

  subroutine def_nctraj_vars(caller,nctype,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/tech
    integer,      intent( in ) :: ncid          ! file id

    integer :: iret
    integer :: ret = 0;
    integer :: loc = 1;

    character(len=15), parameter :: sub = 'def_nctraj_vars'
    double precision, parameter :: minlat =  -90.0;
    double precision, parameter :: maxlat =   90.0;
    double precision, parameter :: minlon = -180.0;
    double precision, parameter :: maxlon =  180.0;
    double precision :: doublefill(1);			! Set differently for different variables

    write(dummy_var,'(e10.5)')1.0/24.0/3600.0
    read(dummy_var,'(e10.5)')resol_var


    write(msg,*)'NCTRAJ def TRAJECTORY_PARAMETERS for traj'; call wrtlog(sub,msg);
    !================================================================================
    PROJECT_NAME_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'PROJECT_NAME', NF_CHAR,1, PROJECT_NAME_dims,PROJECT_NAME_id);
    loc=01; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PROJECT_NAME_id, 'long_name',19, 'Name of the project');
    loc=02; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PROJECT_NAME_id, '_FillValue', 1, ' ');
    loc=04; call cke(sub,ret,loc);
    !================================================================================
    PI_NAME_TRAJ_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'PI_NAME', NF_CHAR,1, PI_NAME_TRAJ_dims,PI_NAME_TRAJ_id);
    loc=01; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PI_NAME_TRAJ_id, 'long_name',34, 'Name of the principal investigator');
    loc=02; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PI_NAME_TRAJ_id, '_FillValue', 1, ' ');
    loc=04; call cke(sub,ret,loc);
    !================================================================================
    TRAJECTORY_PARAMETERS_dims(2) = N_PARAM_dimid;
    TRAJECTORY_PARAMETERS_dims(1) = STRING16_dimid;
     ret=nf_def_var(ncid, 'TRAJECTORY_PARAMETERS', NF_CHAR,NDIMS2, TRAJECTORY_PARAMETERS_dims,TRAJECTORY_PARAMETERS_id);
    loc=34; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,TRAJECTORY_PARAMETERS_id, 'long_name', 44, 'List of available parameters for the station');
    loc=35; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRAJECTORY_PARAMETERS_id, 'conventions', 22, 'Argo reference table 3');
    loc=32; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRAJECTORY_PARAMETERS_id, '_FillValue', 1, ' ');
    loc=37; call cke(sub,ret,loc);
    !================================================================================
    DATA_CENTRE_dims = STRING2_dimid;
     ret=nf_def_var(ncid, 'DATA_CENTRE', NF_CHAR,NDIMS1, DATA_CENTRE_dims,DATA_CENTRE_id);
    loc=19; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,DATA_CENTRE_id, 'long_name', 46, 'Data centre in charge of float data processing');
    loc=20; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_CENTRE_id, 'conventions', 22, 'Argo reference table 4');
    loc=21; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_CENTRE_id, '_FillValue', 1, ' ');
    loc=22; call cke(sub,ret,loc);
    !================================================================================
    DATA_STATE_INDICATOR_dims(1) = STRING4_dimid;
     ret=nf_def_var(ncid, 'DATA_STATE_INDICATOR', NF_CHAR,NDIMS1, &
    DATA_STATE_INDICATOR_dims,DATA_STATE_INDICATOR_id);
    loc=24; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_STATE_INDICATOR_id, 'long_name', 49, 'Degree of processing the data have passed through');
    loc=25; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_STATE_INDICATOR_id, 'conventions', 22, 'Argo reference table 6');
    loc=26; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_STATE_INDICATOR_id, '_FillValue', 1, ' ');
    loc=27; call cke(sub,ret,loc);
    !================================================================================
    PLATFORM_TYPE_dims(1) = STRING32_dimid;
     ret=nf_def_var(ncid, 'PLATFORM_TYPE', NF_CHAR,NDIMS1, PLATFORM_TYPE_dims,PLATFORM_TYPE_id);
    loc=42; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_TYPE_id, 'long_name', 13, 'Type of float');
    loc=43; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_TYPE_id, 'conventions',23,'Argo reference table 23');
    loc=43; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_TYPE_id, '_FillValue', 1, ' ');
    loc=44; call cke(sub,ret,loc);
    !==================================================================
    FLOAT_SERIAL_NO_dims(1) = STRING32_dimid;
     ret=nf_def_var(ncid, 'FLOAT_SERIAL_NO', NF_CHAR,NDIMS1, FLOAT_SERIAL_NO_dims,FLOAT_SERIAL_NO_id);
    loc=23; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FLOAT_SERIAL_NO_id, 'long_name', 26, 'Serial number of the float');
    loc=24; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FLOAT_SERIAL_NO_id, '_FillValue', 1, ' ');
    loc=25; call cke(sub,ret,loc);
    !==================================================================
    FIRMWARE_VERSION_dims(1) = STRING32_dimid;
     ret=nf_def_var(ncid, 'FIRMWARE_VERSION', NF_CHAR,NDIMS1, FIRMWARE_VERSION_dims,FIRMWARE_VERSION_id);
    loc=54; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FIRMWARE_VERSION_id, 'long_name', 27, 'Instrument firmware version');
    loc=51; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FIRMWARE_VERSION_id, '_FillValue', 1, ' ');
    loc=52; call cke(sub,ret,loc);
    !================================================================================
    WMO_INST_TYPE_dims(1) = STRING4_dimid;
     ret=nf_def_var(ncid, 'WMO_INST_TYPE', NF_CHAR,NDIMS1, WMO_INST_TYPE_dims,WMO_INST_TYPE_id);
    loc=32; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, WMO_INST_TYPE_id, 'long_name', 21, 'Coded instrument type');
    loc=33; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, WMO_INST_TYPE_id, 'conventions', 22, 'Argo reference table 8');
    loc=34; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, WMO_INST_TYPE_id, '_FillValue', 1, ' ');
    loc=35; call cke(sub,ret,loc);
    !================================================================================
    POSITIONING_SYSTEM_dims(1) = STRING8_dimid;
     ret=nf_def_var(ncid, 'POSITIONING_SYSTEM', NF_CHAR,NDIMS1, POSITIONING_SYSTEM_dims,POSITIONING_SYSTEM_id);
    loc=70; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITIONING_SYSTEM_id, 'long_name', 18, 'Positioning system');
    loc=71; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITIONING_SYSTEM_id, '_FillValue', 1, ' ');
    loc=72; call cke(sub,ret,loc);
    !==================================================================
    JULD_TRAJ_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'JULD', NF_DOUBLE,NDIMS1, JULD_TRAJ_dims,JULD_TRAJ_id);
    loc=36; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRAJ_id, 'long_name', 68, &
                'Julian day (UTC) of each measurement relative to REFERENCE_DATE_TIME')
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRAJ_id, 'standard_name', 4, 'time');
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRAJ_id, 'units', 34,'days since 1950-01-01 00:00:00 UTC');
    loc=38; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRAJ_id, 'conventions', 56,&
          'Relative julian days with decimal part (as parts of day)')
    loc=39; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_TRAJ_id, 'resolution',NF_DOUBLE, 1, JULD_RESOLUTION);
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_TRAJ_id, '_FillValue', NF_DOUBLE,1,doublefill(1));
    loc=40; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRAJ_id, 'axis', 1, 'T')
    loc=40; call cke(sub,ret,loc);
    !================================================================================
    JULD_STATUS_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'JULD_STATUS', NF_CHAR,NDIMS1, JULD_STATUS_dims,JULD_STATUS_id);
    loc=41; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_STATUS_id, 'long_name', 27, 'Status of the date and time');
    loc=42; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_STATUS_id, 'conventions', 23, 'Argo reference table 19');
    loc=43; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_STATUS_id, '_FillValue', 1, FILLCHAR);
    loc=44; call cke(sub,ret,loc);
    !================================================================================
    JULD_QC_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'JULD_QC', NF_CHAR,NDIMS1, JULD_QC_dims,JULD_QC_id);
    loc=41; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_QC_id, 'long_name', 24, 'Quality on date and time');
    loc=42; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_QC_id, 'conventions', 22, 'Argo reference table 2');
    loc=43; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_QC_id, '_FillValue', 1, FILLCHAR);
    loc=44; call cke(sub,ret,loc);
    !================================================================================
    JULD_ADJUSTED_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'JULD_ADJUSTED', NF_DOUBLE,NDIMS1, JULD_ADJUSTED_dims,JULD_ADJUSTED_id);
    loc=36; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_id, 'long_name', 77, &
                'Adjusted julian day (UTC) of each measurement relative to REFERENCE_DATE_TIME')
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_id, 'standard_name', 4, 'time');
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_id, 'units', 34,'days since 1950-01-01 00:00:00 UTC');
    loc=38; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_id, 'conventions', 56,&
                 'Relative julian days with decimal part (as parts of day)')
    loc=39; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_ADJUSTED_id, 'resolution',NF_DOUBLE, 1, resol_var); 
    loc=40; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_ADJUSTED_id, '_FillValue', NF_DOUBLE,1,doublefill(1));
    loc=40; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_id, 'axis', 1, 'T')
    loc=40; call cke(sub,ret,loc);
    !================================================================================
    JULD_ADJUSTED_STATUS_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'JULD_ADJUSTED_STATUS', NF_CHAR,NDIMS1, JULD_ADJUSTED_STATUS_dims,JULD_ADJUSTED_STATUS_id);
    loc=36; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_STATUS_id, 'long_name', 32, &
                'Status of the JULD_ADJUSTED date')
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_STATUS_id, 'conventions', 23,'Argo reference table 19')
    loc=39; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_STATUS_id, '_FillValue', 1, FILLCHAR);
    loc=40; call cke(sub,ret,loc);
    !================================================================================
    JULD_ADJUSTED_QC_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'JULD_ADJUSTED_QC', NF_CHAR,NDIMS1, JULD_ADJUSTED_QC_dims,JULD_ADJUSTED_QC_id);
    loc=36; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_QC_id, 'long_name', 33, &
                'Quality on adjusted date and time')
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_QC_id, 'conventions', 22,'Argo reference table 2')
    loc=39; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ADJUSTED_QC_id, '_FillValue', 1, FILLCHAR);
    loc=40; call cke(sub,ret,loc);
    !================================================================================
    JULD_STATUS_dims(1) = N_MEASUREMENT_dimid;
    JULD_STATUS_dims(1) = N_MEASUREMENT_dimid;
    JULD_STATUS_dims(1) = N_MEASUREMENT_dimid;
    LATITUDE_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'LATITUDE', NF_DOUBLE,NDIMS1, LATITUDE_dims,LATITUDE_id);
    loc=50; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LATITUDE_id, 'long_name', 25,'Latitude of each location')
    loc=51; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LATITUDE_id, 'standard_name', 8,'latitude');
    loc=52; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LATITUDE_id, 'units', 12, 'degree_north');
    loc=53; call cke(sub,ret,loc);
    doublefill(1)=99999. ;  ret=nf_put_att_double(ncid, LATITUDE_id, '_FillValue', NF_DOUBLE,1,doublefill(1));
    loc=54; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, LATITUDE_id, 'valid_min', NF_DOUBLE, 1, minlat)
    loc=55; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, LATITUDE_id, 'valid_max', NF_DOUBLE, 1, maxlat)
    loc=56; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LATITUDE_id, 'axis', 1, 'Y')
    loc=57; call cke(sub,ret,loc);
    !================================================================================
    LONGITUDE_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'LONGITUDE', NF_DOUBLE,NDIMS1, LONGITUDE_dims,LONGITUDE_id);
    loc=58; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LONGITUDE_id, 'long_name', 26,'Longitude of each location')
    loc=59; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LONGITUDE_id, 'standard_name', 9,'longitude');
    loc=60; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LONGITUDE_id, 'units', 11, 'degree_east');
    loc=61; call cke(sub,ret,loc);
    doublefill(1)=99999. ;  ret=nf_put_att_double(ncid, LONGITUDE_id, '_FillValue', NF_DOUBLE,1,doublefill(1));
    loc=62; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, LONGITUDE_id, 'valid_min', NF_DOUBLE, 1, minlon)
    loc=63; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, LONGITUDE_id, 'valid_max', NF_DOUBLE, 1, maxlon)
    loc=64; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LONGITUDE_id, 'axis', 1, 'X')
    loc=65; call cke(sub,ret,loc);
    !================================================================================
    POSITION_ACCURACY_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'POSITION_ACCURACY', NF_CHAR,NDIMS1, POSITION_ACCURACY_dims,POSITION_ACCURACY_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITION_ACCURACY_id, 'long_name', 44, 'Estimated accuracy in latitude and longitude');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITION_ACCURACY_id, 'conventions', 22, 'Argo reference table 5');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITION_ACCURACY_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    POSITION_QC_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'POSITION_QC', NF_CHAR,NDIMS1, POSITION_QC_dims,POSITION_QC_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITION_QC_id, 'long_name', 19, 'Quality on position');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITION_QC_id, 'conventions', 22, 'Argo reference table 2');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITION_QC_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    CYCLE_NUMBER_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'CYCLE_NUMBER', NF_INT,NDIMS1, CYCLE_NUMBER_dims,CYCLE_NUMBER_id);
    loc=11; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,CYCLE_NUMBER_id, 'long_name', 37, 'Float cycle number of the measurement')
    cycnum_conventions = traj_cycnum_conventions; nlt = len(trim(cycnum_conventions));
     ret=nf_put_att_text(ncid,CYCLE_NUMBER_id, 'conventions', nlt, cycnum_conventions);
     ret=nf_put_att_int(ncid,CYCLE_NUMBER_id, '_FillValue', NF_INT, 1, FILLINT) ! FILLINT int fill value in netcdf_arrays
    !==================================================================
    CYCLE_NUMBER_ADJUSTED_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'CYCLE_NUMBER_ADJUSTED', NF_INT,NDIMS1, CYCLE_NUMBER_ADJUSTED_dims,CYCLE_NUMBER_ADJUSTED_id);
    loc=11; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,CYCLE_NUMBER_ADJUSTED_id, 'long_name', 46, 'Adjusted float cycle number of the measurement')
    cycnum_conventions = traj_cycnum_conventions; nlt = len(trim(cycnum_conventions));
     ret=nf_put_att_text(ncid,CYCLE_NUMBER_ADJUSTED_id, 'conventions', len(trim(cycnum_conventions)), cycnum_conventions);
     ret=nf_put_att_int(ncid,CYCLE_NUMBER_ADJUSTED_id, '_FillValue', NF_INT, 1, FILLINT) ! FILLINT int fill value in netcdf_arrays
    !==================================================================
    MEASUREMENT_CODE_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'MEASUREMENT_CODE', NF_INT,NDIMS1, MEASUREMENT_CODE_dims,MEASUREMENT_CODE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, MEASUREMENT_CODE_id, 'long_name', 50, 'Flag referring to a measurement event in the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, MEASUREMENT_CODE_id, 'conventions', 23, 'Argo reference table 15');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_int(ncid, MEASUREMENT_CODE_id, '_FillValue',NF_INT, 1, FILLINT);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    ! Calling sub-routines which will define <PARAM> variables.
    ! PARAM PARAM_QC PARAM_ADJUSTED PARAM_ADJUSTED_QC PARAM_ADJUSTED_ERROR
    call def_nctraj_params(sub,ncid); 
    !================================================================================
    AXES_ERROR_ELLIPSE_MAJOR_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'AXES_ERROR_ELLIPSE_MAJOR', NF_REAL,NDIMS1, AXES_ERROR_ELLIPSE_MAJOR_dims,AXES_ERROR_ELLIPSE_MAJOR_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, AXES_ERROR_ELLIPSE_MAJOR_id, 'long_name', 51,&
        'Major axis of error ellipse from positioning system');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, AXES_ERROR_ELLIPSE_MAJOR_id, 'units', 6, 'meters');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_int(ncid, AXES_ERROR_ELLIPSE_MAJOR_id, '_FillValue',NF_REAL, 1, FILLINT);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    AXES_ERROR_ELLIPSE_MINOR_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'AXES_ERROR_ELLIPSE_MINOR', NF_REAL,NDIMS1, AXES_ERROR_ELLIPSE_MINOR_dims,AXES_ERROR_ELLIPSE_MINOR_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, AXES_ERROR_ELLIPSE_MINOR_id, 'long_name', 51,&
        'Minor axis of error ellipse from positioning system');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, AXES_ERROR_ELLIPSE_MINOR_id, 'units', 6, 'meters');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_int(ncid, AXES_ERROR_ELLIPSE_MINOR_id, '_FillValue',NF_REAL, 1, FILLINT);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    AXES_ERROR_ELLIPSE_ANGLE_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'AXES_ERROR_ELLIPSE_ANGLE', NF_REAL,NDIMS1, AXES_ERROR_ELLIPSE_ANGLE_dims,AXES_ERROR_ELLIPSE_ANGLE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, AXES_ERROR_ELLIPSE_ANGLE_id, 'long_name', 46,&
        'Angle of error ellipse from positioning system');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, AXES_ERROR_ELLIPSE_ANGLE_id, 'units', 38, 'Degrees (from North when heading East)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_int(ncid, AXES_ERROR_ELLIPSE_ANGLE_id, '_FillValue',NF_REAL, 1, FILLINT);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    SATELLITE_NAME_dims(1) = N_MEASUREMENT_dimid;
     ret=nf_def_var(ncid, 'SATELLITE_NAME', NF_CHAR,NDIMS1, SATELLITE_NAME_dims,SATELLITE_NAME_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SATELLITE_NAME_id, 'long_name', 38, 'Satellite name from positioning system');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SATELLITE_NAME_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    ! JN
    !-----------------  Starting to define N_CYCLE Variables ---------------
    JULD_DESCENT_START_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DESCENT_START', NF_DOUBLE, NDIMS1,JULD_DESCENT_START_dims,JULD_DESCENT_START_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_id, 'long_name', 31, 'Descent start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_DESCENT_START_id, 'resolution',NF_DOUBLE, 1, resol_var); 
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid,JULD_DESCENT_START_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DESCENT_START_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DESCENT_START_STATUS', NF_CHAR, NDIMS1,JULD_DESCENT_START_STATUS_dims,&
           JULD_DESCENT_START_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_STATUS_id, 'long_name', 41,'Status of descent start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_START_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_FIRST_STABILIZATION_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid,'JULD_FIRST_STABILIZATION', NF_DOUBLE, NDIMS1, &
                     JULD_FIRST_STABILIZATION_dims,JULD_FIRST_STABILIZATION_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_FIRST_STABILIZATION_id, 'long_name', 45,&
                    'Time when a float first becomes water-neutral');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_FIRST_STABILIZATION_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_FIRST_STABILIZATION_id, 'units', 34, &
                   'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_FIRST_STABILIZATION_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_FIRST_STABILIZATION_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.; ret=nf_put_att_double(ncid,JULD_FIRST_STABILIZATION_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_FIRST_STABILIZATION_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_FIRST_STABILIZATION_STATUS', NF_CHAR, NDIMS1,JULD_FIRST_STABILIZATION_STATUS_dims,&
           JULD_FIRST_STABILIZATION_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_STABILIZATION_STATUS_id, 'long_name', 55,& 
          'Status of time when a float first becomes water-neutral');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_STABILIZATION_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_STABILIZATION_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DESCENT_END_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DESCENT_END', NF_DOUBLE, NDIMS1,JULD_DESCENT_END_dims,JULD_DESCENT_END_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_id, 'long_name', 29, 'Descent end date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_DESCENT_END_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_DESCENT_END_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DESCENT_END_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DESCENT_END_STATUS', NF_CHAR, NDIMS1,JULD_DESCENT_END_STATUS_dims,&
           JULD_DESCENT_END_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_STATUS_id, 'long_name', 39,'Status of descent end date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DESCENT_END_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_PARK_START_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_PARK_START', NF_DOUBLE, NDIMS1,JULD_PARK_START_dims,JULD_PARK_START_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_id, 'long_name', 29, 'Drift start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_PARK_START_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_PARK_START_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_PARK_START_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_PARK_START_STATUS', NF_CHAR, NDIMS1,JULD_PARK_START_STATUS_dims, &
           JULD_PARK_START_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_STATUS_id, 'long_name', 39,&
            'Status of drift start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_START_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_PARK_END_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_PARK_END', NF_DOUBLE, NDIMS1,JULD_PARK_END_dims,JULD_PARK_END_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_id, 'long_name', 27, 'Drift end date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_PARK_END_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_PARK_END_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_PARK_END_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_PARK_END_STATUS', NF_CHAR, NDIMS1,JULD_PARK_END_STATUS_dims,&
           JULD_PARK_END_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_STATUS_id, 'long_name', 37, &
            'Status of drift end date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_PARK_END_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DEEP_DESCENT_END_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DEEP_DESCENT_END', NF_DOUBLE, NDIMS1,JULD_DEEP_DESCENT_END_dims,&
               JULD_DEEP_DESCENT_END_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_id, 'long_name', 34, 'Deep descent end date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_DEEP_DESCENT_END_id, 'resolution',NF_DOUBLE, 1, resol_var); 
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_DEEP_DESCENT_END_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DEEP_DESCENT_END_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DEEP_DESCENT_END_STATUS', NF_CHAR, NDIMS1,JULD_DEEP_DESCENT_END_STATUS_dims,&
           JULD_DEEP_DESCENT_END_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_STATUS_id, 'long_name', 44, &
            'Status of deep descent end date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_DESCENT_END_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DEEP_PARK_START_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DEEP_PARK_START', NF_DOUBLE, NDIMS1,JULD_DEEP_PARK_START_dims,JULD_DEEP_PARK_START_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_id, 'long_name', 33, 'Deep park start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_DEEP_PARK_START_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, JULD_DEEP_PARK_START_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DEEP_PARK_START_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DEEP_PARK_START_STATUS', NF_CHAR, NDIMS1,JULD_DEEP_PARK_START_STATUS_dims,&
           JULD_DEEP_PARK_START_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_STATUS_id, 'long_name', 43, &
            'Status of deep park start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_PARK_START_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_ASCENT_START_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_ASCENT_START', NF_DOUBLE, NDIMS1,JULD_ASCENT_START_dims,JULD_ASCENT_START_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_id, 'long_name', 39, 'Start date of the ascent to the surface');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_ASCENT_START_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_ASCENT_START_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_ASCENT_START_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_ASCENT_START_STATUS', NF_CHAR, NDIMS1,JULD_ASCENT_START_STATUS_dims,&
           JULD_ASCENT_START_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_STATUS_id, 'long_name', 49, &
            'Status of start date of the ascent to the surface');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_START_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_DEEP_ASCENT_START_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DEEP_ASCENT_START', NF_DOUBLE, NDIMS1,JULD_DEEP_ASCENT_START_dims,JULD_DEEP_ASCENT_START_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_id, 'long_name', 35, 'Deep ascent start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_DEEP_ASCENT_START_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_DEEP_ASCENT_START_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================    
    JULD_DEEP_ASCENT_START_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_DEEP_ASCENT_START_STATUS', NF_CHAR, NDIMS1,JULD_DEEP_ASCENT_START_STATUS_dims,&
           JULD_DEEP_ASCENT_START_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_STATUS_id, 'long_name', 45, &
            'Status of deep ascent start date of the cycle');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_DEEP_ASCENT_START_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_ASCENT_END_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_ASCENT_END', NF_DOUBLE, NDIMS1,JULD_ASCENT_END_dims,JULD_ASCENT_END_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_id, 'long_name', 33, 'End date of ascent to the surface');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_id, 'conventions', 56, &
           'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_ASCENT_END_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_ASCENT_END_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_ASCENT_END_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_ASCENT_END_STATUS', NF_CHAR, NDIMS1,JULD_ASCENT_END_STATUS_dims,&
           JULD_ASCENT_END_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_STATUS_id, 'long_name', 43, &
            'Status of end date of ascent to the surface');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_ASCENT_END_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_TRANSMISSION_START_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_TRANSMISSION_START', NF_DOUBLE, NDIMS1, &
            JULD_TRANSMISSION_START_dims,JULD_TRANSMISSION_START_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_START_id, 'long_name', 26, 'Start date of transmission');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_START_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_START_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_START_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_TRANSMISSION_START_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_TRANSMISSION_START_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_TRANSMISSION_START_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_TRANSMISSION_START_STATUS', NF_CHAR, NDIMS1,JULD_TRANSMISSION_START_STATUS_dims,&
           JULD_TRANSMISSION_START_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_START_STATUS_id, 'long_name', 36, &
            'Status of start date of transmission');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_START_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_TRANSMISSION_START_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_FIRST_MESSAGE_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_FIRST_MESSAGE', NF_DOUBLE, NDIMS1, &
            JULD_FIRST_MESSAGE_dims,JULD_FIRST_MESSAGE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_MESSAGE_id, 'long_name', 39, &
            'Date of earliest float message received');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_MESSAGE_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_MESSAGE_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_MESSAGE_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_FIRST_MESSAGE_id, 'resolution',NF_DOUBLE, 1, resol_var);   
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_FIRST_MESSAGE_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_FIRST_MESSAGE_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_FIRST_MESSAGE_STATUS', NF_CHAR, NDIMS1,JULD_FIRST_MESSAGE_STATUS_dims,&
           JULD_FIRST_MESSAGE_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_MESSAGE_STATUS_id, 'long_name', 49, &
            'Status of date of earliest float message received');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_MESSAGE_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_FIRST_MESSAGE_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_FIRST_LOCATION_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_FIRST_LOCATION', NF_DOUBLE, NDIMS1, &
            JULD_FIRST_LOCATION_dims,JULD_FIRST_LOCATION_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_LOCATION_id, 'long_name', 25, &
            'Date of earliest location');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_LOCATION_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_LOCATION_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_LOCATION_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_FIRST_LOCATION_id, 'resolution',NF_DOUBLE, 1, resol_var);
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_FIRST_LOCATION_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_FIRST_LOCATION_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_FIRST_LOCATION_STATUS', NF_CHAR, NDIMS1,JULD_FIRST_LOCATION_STATUS_dims,&
           JULD_FIRST_LOCATION_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_LOCATION_STATUS_id, 'long_name', 35, &
            'Status of date of earliest location');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_FIRST_LOCATION_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_FIRST_LOCATION_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_LAST_LOCATION_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_LAST_LOCATION', NF_DOUBLE, NDIMS1, &
            JULD_LAST_LOCATION_dims,JULD_LAST_LOCATION_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_LOCATION_id, 'long_name', 23, &
            'Date of latest location');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_LOCATION_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_LOCATION_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_LOCATION_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_LAST_LOCATION_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_LAST_LOCATION_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_LAST_LOCATION_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_LAST_LOCATION_STATUS', NF_CHAR, NDIMS1,JULD_LAST_LOCATION_STATUS_dims,&
           JULD_LAST_LOCATION_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_LOCATION_STATUS_id, 'long_name', 33, &
            'Status of date of latest location');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_LOCATION_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_LAST_LOCATION_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_LAST_MESSAGE_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_LAST_MESSAGE', NF_DOUBLE, NDIMS1, &
            JULD_LAST_MESSAGE_dims,JULD_LAST_MESSAGE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_MESSAGE_id, 'long_name', 37, &
            'Date of latest float message received');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_MESSAGE_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_MESSAGE_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_MESSAGE_id, 'conventions', 56, &
                 'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_LAST_MESSAGE_id, 'resolution',NF_DOUBLE, 1, resol_var);  
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_LAST_MESSAGE_id, '_FillValue',NF_DOUBLE, 1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_LAST_MESSAGE_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_LAST_MESSAGE_STATUS', NF_CHAR, NDIMS1,JULD_LAST_MESSAGE_STATUS_dims,&
           JULD_LAST_MESSAGE_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_MESSAGE_STATUS_id, 'long_name', 47, &
            'Status of date of latest float message received');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_LAST_MESSAGE_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_LAST_MESSAGE_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_TRANSMISSION_END_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_TRANSMISSION_END', NF_DOUBLE, NDIMS1, &
            JULD_TRANSMISSION_END_dims,JULD_TRANSMISSION_END_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_END_id, 'long_name', 21, &
            'Transmission end date');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_END_id, 'standard_name', 4, 'time');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_END_id, 'units', 34, 'days since 1950-01-01 00:00:00 UTC');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_END_id, 'conventions', 56, &
             'Relative julian days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_TRANSMISSION_END_id, 'resolution',NF_DOUBLE, 1, resol_var); 
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, JULD_TRANSMISSION_END_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    JULD_TRANSMISSION_END_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'JULD_TRANSMISSION_END_STATUS', NF_CHAR, NDIMS1,JULD_TRANSMISSION_END_STATUS_dims,&
           JULD_TRANSMISSION_END_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_END_STATUS_id, 'long_name', 31, &
            'Status of transmission end date');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, JULD_TRANSMISSION_END_STATUS_id, 'conventions', 23,'Argo reference table 19');      
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,JULD_TRANSMISSION_END_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    CLOCK_OFFSET_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'CLOCK_OFFSET', NF_DOUBLE, NDIMS1, &
            CLOCK_OFFSET_dims,CLOCK_OFFSET_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CLOCK_OFFSET_id, 'long_name', 25, &
            'Time of float clock drift');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CLOCK_OFFSET_id, 'units', 4, 'days');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CLOCK_OFFSET_id, 'conventions', 40, &
             'Days with decimal part (as parts of day)');
    loc=68; call cke(sub,ret,loc);
    doublefill(1)=999999.;  ret=nf_put_att_double(ncid, CLOCK_OFFSET_id, '_FillValue', NF_DOUBLE,1, doublefill(1));
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    GROUNDED_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'GROUNDED', NF_CHAR, NDIMS1, &
            GROUNDED_dims,GROUNDED_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, GROUNDED_id, 'long_name', 49, &
            'Did the profiler touch the ground for that cycle?');
    !loc=67; call cke(sub,ret,loc);
    ! ret=nf_put_att_text(ncid, GROUNDED_id, 'units', 4, 'days');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, GROUNDED_id, 'conventions', 23, &
             'Argo reference table 20');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, GROUNDED_id, '_FillValue', 1, FILLCHAR);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    REPRESENTATIVE_PARK_PRESSURE_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'REPRESENTATIVE_PARK_PRESSURE', NF_REAL, NDIMS1, &
            REPRESENTATIVE_PARK_PRESSURE_dims,REPRESENTATIVE_PARK_PRESSURE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, REPRESENTATIVE_PARK_PRESSURE_id, 'long_name', 37, &
            'Best pressure value during park phase');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, REPRESENTATIVE_PARK_PRESSURE_id, 'units', 7, 'decibar');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_real(ncid, REPRESENTATIVE_PARK_PRESSURE_id, '_FillValue',    NF_REAL, 1,FILLNC );
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    REPRESENTATIVE_PARK_PRESSURE_STATUS_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'REPRESENTATIVE_PARK_PRESSURE_STATUS', NF_CHAR, NDIMS1, &
            REPRESENTATIVE_PARK_PRESSURE_STATUS_dims,REPRESENTATIVE_PARK_PRESSURE_STATUS_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, REPRESENTATIVE_PARK_PRESSURE_STATUS_id, 'long_name', 47, &
    'Status of best pressure value during park phase');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, REPRESENTATIVE_PARK_PRESSURE_STATUS_id, 'conventions', 23, &
         'Argo reference table 21');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, REPRESENTATIVE_PARK_PRESSURE_STATUS_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    CONFIG_MISSION_NUMBER_dims(1) = N_CYCLE_dimid
     ret=nf_def_var(ncid, 'CONFIG_MISSION_NUMBER', NF_INT, ND1, CONFIG_MISSION_NUMBER_dims, CONFIG_MISSION_NUMBER_id)
    loc = 153; call cke(sub, ret, loc);
     ret=nf_put_att_text(ncid, CONFIG_MISSION_NUMBER_id, 'long_name', 58, &
    		 'Unique number denoting the missions performed by the float')
    loc = 154; call cke(sub, ret, loc);
     ret=nf_put_att_text(ncid, CONFIG_MISSION_NUMBER_id, 'conventions',33,'1...N, 1 : first complete mission')
    loc = 155; call cke(sub, ret, loc);
     ret=nf_put_att_int(ncid,CONFIG_MISSION_NUMBER_id, '_FillValue', NF_INT, 1, FILLINT) ! FILLINT int fill value in netcdf_arrays
    loc = 156; call cke(sub, ret, loc);
    !================================================================================
    CYCLE_NUMBER_INDEX_dims(1) = N_CYCLE_dimid
     ret=nf_def_var(ncid, 'CYCLE_NUMBER_INDEX', NF_INT, ND1, CYCLE_NUMBER_INDEX_dims, CYCLE_NUMBER_INDEX_id)
    loc = 153; call cke(sub, ret, loc);
     ret=nf_put_att_text(ncid, CYCLE_NUMBER_INDEX_id, 'long_name', 50, &
    		 'Cycle number that corresponds to the current index')
    loc = 154; call cke(sub, ret, loc);
    cycnum_conventions = traj_cycnum_conventions; nlt = len(trim(cycnum_conventions));
     ret=nf_put_att_text(ncid,CYCLE_NUMBER_INDEX_id, 'conventions', len(trim(cycnum_conventions)), cycnum_conventions);
     ret=nf_put_att_int(ncid,CYCLE_NUMBER_INDEX_id, '_FillValue', NF_INT, 1, FILLINT) ! FILLINT int fill value in netcdf_arrays
    loc = 156; call cke(sub, ret, loc);
    !================================================================================
    CYCLE_NUMBER_INDEX_ADJUSTED_dims(1) = N_CYCLE_dimid
     ret=nf_def_var(ncid, 'CYCLE_NUMBER_INDEX_ADJUSTED', NF_INT, ND1, CYCLE_NUMBER_INDEX_ADJUSTED_dims,&
	   CYCLE_NUMBER_INDEX_ADJUSTED_id)
    loc = 153; call cke(sub, ret, loc);
     ret=nf_put_att_text(ncid, CYCLE_NUMBER_INDEX_ADJUSTED_id, 'long_name', 59, &
    		 'Adjusted cycle number that corresponds to the current index')
    loc = 154; call cke(sub, ret, loc);
    cycnum_conventions = traj_cycnum_conventions; nlt = len(trim(cycnum_conventions));
     ret=nf_put_att_text(ncid, CYCLE_NUMBER_INDEX_ADJUSTED_id, 'conventions',  len(trim(cycnum_conventions)), cycnum_conventions);
     ret=nf_put_att_int(ncid,CYCLE_NUMBER_INDEX_ADJUSTED_id, '_FillValue', NF_INT, 1, FILLINT) ! FILLINT int fill value in netcdf_arrays
    loc = 156; call cke(sub, ret, loc);
    !================================================================================
    DATA_MODE_dims(1) = N_CYCLE_dimid;
     ret=nf_def_var(ncid, 'DATA_MODE', NF_CHAR, NDIMS1, &
        DATA_MODE_dims,DATA_MODE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_MODE_id, 'long_name', 30, &
        'Delayed mode or real time data');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_MODE_id, 'conventions', 62, &
         'R : real time; D : delayed mode; A : real time with adjustment');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DATA_MODE_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    !               HISTORY SECTION
    !================================================================================
    HISTORY_INSTITUTION_dims(1) = STRING4_dimid;
    HISTORY_INSTITUTION_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_INSTITUTION', NF_CHAR, NDIMS2, &
        HISTORY_INSTITUTION_dims,HISTORY_INSTITUTION_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_INSTITUTION_id, 'long_name', 34, &
        'Institution which performed action');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_INSTITUTION_id, 'conventions', 22, &
         'Argo reference table 4');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_INSTITUTION_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_STEP_dims(1) = STRING4_dimid;
    HISTORY_STEP_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_STEP', NF_CHAR, NDIMS2, &
        HISTORY_STEP_dims,HISTORY_STEP_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_STEP_id, 'long_name', 23, &
        'Step in data processing');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_STEP_id, 'conventions', 23, &
         'Argo reference table 12');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_STEP_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_SOFTWARE_dims(1) = STRING4_dimid;
    HISTORY_SOFTWARE_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_SOFTWARE', NF_CHAR, NDIMS2, &
        HISTORY_SOFTWARE_dims,HISTORY_SOFTWARE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_SOFTWARE_id, 'long_name', 39, &
        'Name of software which performed action');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_SOFTWARE_id, 'conventions', 21, &
         'Institution dependent');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_SOFTWARE_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_SOFTWARE_RELEASE_dims(1) = STRING4_dimid;
    HISTORY_SOFTWARE_RELEASE_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_SOFTWARE_RELEASE', NF_CHAR, NDIMS2, &
        HISTORY_SOFTWARE_RELEASE_dims,HISTORY_SOFTWARE_RELEASE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_SOFTWARE_RELEASE_id, 'long_name', 50, &
        'Version/release of software which performed action');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_SOFTWARE_RELEASE_id, 'conventions', 21, &
         'Institution dependent');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_SOFTWARE_RELEASE_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_REFERENCE_dims(1) = STRING64_dimid;
    HISTORY_REFERENCE_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_REFERENCE', NF_CHAR, NDIMS2, &
        HISTORY_REFERENCE_dims,HISTORY_REFERENCE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_REFERENCE_id, 'long_name', 21, &
        'Reference of database');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_REFERENCE_id, 'conventions', 21, &
         'Institution dependent');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_REFERENCE_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_DATE_dims(1) = DATE_TIME_dimid;
    HISTORY_DATE_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_DATE', NF_CHAR, NDIMS2, &
        HISTORY_DATE_dims,HISTORY_DATE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_DATE_id, 'long_name', 35, &
        'Date the history record was created');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_DATE_id, 'conventions', 14, &
         'YYYYMMDDHHMISS');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_DATE_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_ACTION_dims(1) = STRING4_dimid;
    HISTORY_ACTION_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_ACTION', NF_CHAR, NDIMS2, &
        HISTORY_ACTION_dims,HISTORY_ACTION_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_ACTION_id, 'long_name', 24, &
        'Action performed on data');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_ACTION_id, 'conventions', 22, &
        'Argo reference table 7');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_ACTION_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_PARAMETER_dims(1) = STRING16_dimid;
    HISTORY_PARAMETER_dims(2) = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_PARAMETER', NF_CHAR, NDIMS2, &
        HISTORY_PARAMETER_dims,HISTORY_PARAMETER_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_PARAMETER_id, 'long_name', 40, &
        'Station parameter action is performed on');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_PARAMETER_id, 'conventions', 22, &
        'Argo reference table 3');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_PARAMETER_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_PREVIOUS_VALUE_dims = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_PREVIOUS_VALUE', NF_REAL, NDIMS1, &
        HISTORY_PREVIOUS_VALUE_dims,HISTORY_PREVIOUS_VALUE_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_PREVIOUS_VALUE_id, 'long_name', 43, &
        'Parameter/Flag previous value before action');
    loc=67; call cke(sub,ret,loc);
    ! ret=nf_put_att_text(ncid, HISTORY_PREVIOUS_VALUE_id, 'conventions', 22, &
    !    'Argo reference table 7');
    !loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_real(ncid, HISTORY_PREVIOUS_VALUE_id, '_FillValue', NF_REAL,1, FILLNC);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_INDEX_DIMENSION_dims = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_INDEX_DIMENSION', NF_CHAR, NDIMS1, &
        HISTORY_INDEX_DIMENSION_dims,HISTORY_INDEX_DIMENSION_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_INDEX_DIMENSION_id, 'long_name', 80, &
        'Name of dimension to which HISTORY_START_INDEX and HISTORY_STOP_INDEX correspond');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_INDEX_DIMENSION_id, 'conventions', 28, &
        'C: N_CYCLE, M: N_MEASUREMENT');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_INDEX_DIMENSION_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_START_INDEX_dims = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_START_INDEX', NF_INT, NDIMS1, &
        HISTORY_START_INDEX_dims,HISTORY_START_INDEX_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_START_INDEX_id, 'long_name', 29, &
        'Start index action applied on');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_int(ncid, HISTORY_START_INDEX_id, '_FillValue', NF_INT,1, FILLINT);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_STOP_INDEX_dims = N_HISTORY_dimid;
     ret=nf_def_var(ncid, 'HISTORY_STOP_INDEX', NF_INT, NDIMS1, &
        HISTORY_STOP_INDEX_dims,HISTORY_STOP_INDEX_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_STOP_INDEX_id, 'long_name', 28, &
        'Stop index action applied on');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_int(ncid, HISTORY_STOP_INDEX_id, '_FillValue',NF_INT, 1, FILLINT);
    loc=69; call cke(sub,ret,loc);
    !================================================================================
    HISTORY_QCTEST_dims(2) = N_HISTORY_dimid;
    HISTORY_QCTEST_dims(1) = STRING16_dimid;
     ret=nf_def_var(ncid, 'HISTORY_QCTEST', NF_CHAR, NDIMS2, &
        HISTORY_QCTEST_dims,HISTORY_QCTEST_id);
    loc=66; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_QCTEST_id, 'long_name', 60, &
        'Documentation of tests performed, tests failed (in hex form)');
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_QCTEST_id, 'conventions', 69, &
        'Write tests performed when ACTION=QCP$; tests failed when ACTION=QCF$');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, HISTORY_QCTEST_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !================================================================================
end subroutine def_nctraj_vars

subroutine put_nctraj_data(caller,nctype,ncid)

	character(*), intent( in ) :: caller        ! who is calling me?
	character(*), intent( in ) :: nctype        ! prof/traj/tech
	integer,      intent( in ) :: ncid          ! file id
	
	integer :: iret
        integer :: ret = 0;
	integer :: loc = 1;
	
	character(len=15), parameter :: sub = 'put_nctraj_data'

	integer :: itp	
	!Variables to format NC data.
	character(len=64)  :: c64
	character(len=32)  :: c32
	
	!-------------------------------------------------------------------------------------------------------------- 
	write(c64, '(a64)') chr_PROJECT_NAME 
	 ret=nf_put_var_text(ncid, PROJECT_NAME_id,adjustl(c64));
	loc=10; call cke(sub,ret,loc);
	!-------------------------------------------------------------------------------------------------------------- 
	write(c64,'(a64)')var_PI_NAME
	 ret=nf_put_var_text(ncid, PI_NAME_TRAJ_id, adjustl(c64));
	loc=10; call cke(sub,ret,loc);
	!--------------------------------------------------------------------------------------------------------------
	itp=0;
	if(hasPRES_traj)then; itp=itp+1; TRAJECTORY_PARAMETERS(itp) = 'PRES            '; endif
	if(hasTEMP_traj)then; itp=itp+1; TRAJECTORY_PARAMETERS(itp) = 'TEMP            '; endif
	if(hasPSAL_traj)then; itp=itp+1; TRAJECTORY_PARAMETERS(itp) = 'PSAL            '; endif
	 ret=nf_put_var_text(ncid, TRAJECTORY_PARAMETERS_id,TRAJECTORY_PARAMETERS); call cke(sub,ret,36);
	!--------------------------------------------------------------------------------------------------------------  
	DATA_CENTRE=chr_DATA_CENTRE;
	 ret=nf_put_var_text(ncid, DATA_CENTRE_id, DATA_CENTRE);
	loc=78; call cke(sub,ret,loc);
	!--------------------------------------------------------------------------------------------------------------
        write(DATA_STATE_INDICATOR,'(a4)')chr_DATA_STATE_INDICATOR
	 ret=nf_put_var_text(ncid, DATA_STATE_INDICATOR_id, DATA_STATE_INDICATOR);
	loc=80; call cke(sub,ret,loc);
	!================================================================================
        write(PLATFORM_TYPE,'(a32)')chr_PLATFORM_TYPE
	 ret=nf_put_var_text(ncid, PLATFORM_TYPE_id, PLATFORM_TYPE);
	loc=45; call cke(sub,ret,loc);
	!================================================================================
	write(c32,'(a32)')var_FLOAT_SERIAL_NO
	 ret=nf_put_var_text(ncid, FLOAT_SERIAL_NO_id, adjustl(c32));
	loc=26; call cke(sub,ret,loc);
	!================================================================================
	write(c32,'(a32)')var_FIRMWARE_VERSION
	 ret=nf_put_var_text(ncid, FIRMWARE_VERSION_id, adjustl(c32));
	loc=53; call cke(sub,ret,loc);
	!================================================================================
        write(WMO_INST_TYPE,'(a4)')var_WMO_INST_TYPE
	 ret=nf_put_var_text(ncid, WMO_INST_TYPE_id, WMO_INST_TYPE);
	loc=81; call cke(sub,ret,loc);
	!================================================================================
        write(POSITIONING_SYSTEM,'(a8)')chr_POSITIONING_SYSTEM
	 ret=nf_put_var_text(ncid, POSITIONING_SYSTEM_id, POSITIONING_SYSTEM);
	loc=82; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT;
	   JULD_TRAJ_ARR(i)=double_trajectory_parameter_measurement(JULD_TRAJ,i)
	enddo
	count = n_measurement
	start = 1 ! ... used to have start(1)
        ! NOTE NF_UNLIMITED This is the call that increments the UNLIMITED dimension for traj
	 ret=nf_put_vara_double(ncid, JULD_TRAJ_id,start,count, JULD_TRAJ_ARR);
	loc=83; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_STATUS_id, JULD_STATUS_TRAJ);
	loc=84; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_QC_id, JULD_QC_TRAJ);
	loc=85; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT; JULD_ADJUSTED(i) = FILLDBLE;; enddo
	 ret=nf_put_var_double(ncid, JULD_ADJUSTED_id, JULD_ADJUSTED);
	loc=86; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT; JULD_ADJUSTED_STATUS(i) = FILLCHAR; enddo
	 ret=nf_put_var_text(ncid, JULD_ADJUSTED_STATUS_id, JULD_ADJUSTED_STATUS);
	loc=87; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT; JULD_ADJUSTED_QC(i) = FILLCHAR; enddo
	 ret=nf_put_var_text(ncid, JULD_ADJUSTED_QC_id, JULD_ADJUSTED_QC);
	loc=88; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT;
	  if(double_trajectory_parameter_measurement(LAT_TRAJ,i) /= FILLDBLE)then
	     LATITUDE_TRAJ(i)=double_trajectory_parameter_measurement(LAT_TRAJ,i)
	  endif
	enddo
	count = n_measurement
	start = 1 ! ... used to have start(1)
	 ret=nf_put_vara_double(ncid, LATITUDE_id,start,count, LATITUDE_TRAJ);
	loc=89; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT; 
	   if(double_trajectory_parameter_measurement(LONG_TRAJ,i) /= FILLDBLE)then
	      LONGITUDE_TRAJ(i)=double_trajectory_parameter_measurement(LONG_TRAJ,i)
	   endif
	enddo
	 ret=nf_put_vara_double(ncid, LONGITUDE_id, start,count,LONGITUDE_TRAJ);
	loc=90; call cke(sub,ret,loc);
	!================================================================================
	!do i=1,N_MEASUREMENT; 
	!if(char_trajectory_parameter_measurement(POS_ACY,i) /= ' ' .and. char_trajectory_parameter_measurement(POS_ACY,i) /= 'I')&
	!then
	 !read(trajectory_parameter_measurement(POS_ACY,i),*)POSITION_ACCURACY(i);
	 !POSITION_ACCURACY(i)=char_trajectory_parameter_measurement(POS_ACY,i)
	!endif
	!enddo
	 ret=nf_put_var_text(ncid, POSITION_ACCURACY_id, POSITION_ACCURACY);
	loc=91; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, POSITION_QC_id, POSITION_QC_TRAJ);
	loc=92; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_int(ncid, CYCLE_NUMBER_id, CYCLE_NUMBER_TRAJ);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	do i=1,N_MEASUREMENT; CYCLE_NUMBER_ADJUSTED(i) = FILLINT; enddo
	 ret=nf_put_var_int(ncid, CYCLE_NUMBER_ADJUSTED_id, CYCLE_NUMBER_ADJUSTED);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	do i=1,n_measurement; 
	  if(int_trajectory_parameter_measurement(MCCODE,i) /= FILLINT)then
	     !read(trajectory_parameter_measurement(MCCODE,i),*)MEASUREMENT_CODE(i);
             MEASUREMENT_CODE(i)=int_trajectory_parameter_measurement(MCCODE,i)
	  endif
	enddo
	 ret=nf_put_var_int(ncid, MEASUREMENT_CODE_id, MEASUREMENT_CODE);
	loc=93; call cke(sub,ret,loc);
	!================================================================================
	!do i=1,N_MEASUREMENT;
	!   if(char_trajectory_parameter_measurement(SAT_NAME,i) /= ' ')then
	!      !read(trajectory_parameter_measurement(SAT_NAME,i),*)SATELLITE_NAME(i);
	!      SATELLITE_NAME(i)=char_trajectory_parameter_measurement(SAT_NAME,i)
	!   endif
	!enddo
	 ret=nf_put_var_text(ncid, SATELLITE_NAME_id, SATELLITE_NAME);
	loc=94; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_DESCENT_START_id,JULD_DESCENT_START);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_DESCENT_START_STATUS_id,JULD_DESCENT_START_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_FIRST_STABILIZATION_id,JULD_FIRST_STABILIZATION);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_FIRST_STABILIZATION_STATUS_id,JULD_FIRST_STABILIZATION_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_DESCENT_END_id,JULD_DESCENT_END);
	 ret=nf_put_var_double(ncid, JULD_DESCENT_END_id,JULD_DESCENT_END);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_DESCENT_END_STATUS_id,JULD_DESCENT_END_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_ASCENT_START_id,JULD_ASCENT_START);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_ASCENT_START_STATUS_id,JULD_ASCENT_START_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_DEEP_ASCENT_START_id,JULD_DEEP_ASCENT_START);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_DEEP_ASCENT_START_STATUS_id,JULD_DEEP_ASCENT_START_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================	
	 ret=nf_put_var_double(ncid, JULD_ASCENT_END_id,JULD_ASCENT_END);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_ASCENT_END_STATUS_id,JULD_ASCENT_END_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_PARK_START_id,JULD_PARK_START);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_PARK_START_STATUS_id,JULD_PARK_START_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_PARK_END_id,JULD_PARK_END);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_PARK_END_STATUS_id,JULD_PARK_END_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_DEEP_DESCENT_END_id,JULD_DEEP_DESCENT_END);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_DEEP_DESCENT_END_STATUS_id,JULD_DEEP_DESCENT_END_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_TRANSMISSION_START_id,START_OF_TRANS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_TRANSMISSION_START_STATUS_id,JULD_TRANSMISSION_START_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_int(ncid, CYCLE_NUMBER_INDEX_id,CYCLE_NUMBER_INDEX);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_FIRST_MESSAGE_id,JULD_FIRST_MESSAGE);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_FIRST_MESSAGE_STATUS_id,JULD_FIRST_MESSAGE_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_LAST_MESSAGE_id,JULD_LAST_MESSAGE);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_LAST_MESSAGE_STATUS_id,JULD_LAST_MESSAGE_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_FIRST_LOCATION_id,JULD_FIRST_LOCATION);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_FIRST_LOCATION_STATUS_id,JULD_FIRST_LOCATION_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_LAST_LOCATION_id,JULD_LAST_LOCATION);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_LAST_LOCATION_STATUS_id,JULD_LAST_LOCATION_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_double(ncid, JULD_TRANSMISSION_END_id,END_OF_TRANS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, JULD_TRANSMISSION_END_STATUS_id,JULD_TRANSMISSION_END_STATUS);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, GROUNDED_id,GROUNDED_TRAJ);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_int(ncid, CONFIG_MISSION_NUMBER_id,CONFIG_MISSION_NUMBER_TRAJ);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
	 ret=nf_put_var_text(ncid, DATA_MODE_id,DATA_MODE_TRAJ);
	loc=76; call cke(sub,ret,loc);
	!================================================================================
        ! Calling sub-routine to fill <PARAM> data section
	call put_nctraj_params(sub,ncid);
	!================================================================================
        ! Calling sub-routine to fill history data section
	call nctraj_calhistrec(sub,ncid);
	!================================================================================

end subroutine put_nctraj_data
!====================================================================

subroutine nctraj_calhistrec(caller,ncid)

   character(*), intent( in ) :: caller        ! who is calling me?
   integer,      intent( in ) :: ncid          ! file id
   character(len=16), parameter   :: sub = 'nctraj_calhistrec';
   
   integer :: nhist_traj = 0;

   HISTORY_INSTITUTION = "AO  ";
   !count2(1)=4;  ret=nf_put_vara_text(ncid, HISTORY_INSTITUTION_id,start2,count2,HISTORY_INSTITUTION);
   HISTORY_SOFTWARE_RELEASE = " ";
   ! ret=nf_put_var_text(ncid, HISTORY_SOFTWARE_RELEASE_id,HISTORY_SOFTWARE_RELEASE);
   HISTORY_REFERENCE= "c64";
   ! ret=nf_put_var_text(ncid, HISTORY_REFERENCE_id,HISTORY_REFERENCE);
   HISTORY_DATE = file_date_time;
   ! ret=nf_put_var_text(ncid, HISTORY_DATE_id,HISTORY_DATE);
   HISTORY_PARAMETER= " "; 
   ! ret=nf_put_var_text(ncid, HISTORY_PARAMETER_id,HISTORY_PARAMETER);
   HISTORY_START_INDEX = FILLNC;
   ! ret=nf_put_var_text(ncid, HISTORY_START_INDEX_id,HISTORY_START_INDEX);
   HISTORY_STOP_INDEX = FILLNC;
   ! ret=nf_put_var_text(ncid, HISTORY_STOP_INDEX_id,HISTORY_STOP_INDEX);

   nhist_traj = nhist_traj + 1;
   HISTORY_STEP = "ARGQ"; HISTORY_SOFTWARE = "QCPL"; HISTORY_ACTION = "QCP$"; HISTORY_QCTEST = '0';
   call put_nctraj_histrec(sub,ncid,nhist_traj)
   nhist_traj = nhist_traj + 1;
   HISTORY_STEP = "ARGQ"; HISTORY_SOFTWARE = "QCPL"; HISTORY_ACTION = "QCF$"; HISTORY_QCTEST = '0';
   call put_nctraj_histrec(sub,ncid,nhist_traj)

end subroutine nctraj_calhistrec

!====================================================================pres PRES
subroutine put_nctraj_histrec(caller,ncid,nhist_traj)


   character(*), intent( in )     :: caller
   integer, intent( in )          :: ncid      ! LNetCDF file id
   character(len=16), parameter   :: sub = 'nctraj_histrec';
   integer,      intent( in ) :: nhist_traj		! history record to write
   integer   count2(2),start2(2);
   integer :: iret
    integer :: ret = 0;


   start2(1) = 1; !varies by string length
!OBSOLETE   count = 1;
   start2(2) = nhist_traj; count2(2)=1;
   count2(1)=4;  ret=nf_put_vara_text(ncid, HISTORY_INSTITUTION_id,start2,count2,HISTORY_INSTITUTION);
   count2(1)=4;  ret=nf_put_vara_text(ncid, HISTORY_STEP_id,start2,count2,HISTORY_STEP);
   count2(1)=4;  ret=nf_put_vara_text(ncid, HISTORY_SOFTWARE_id,start2,count2,HISTORY_SOFTWARE);
   count2(1)=4;  ret=nf_put_vara_text(ncid, HISTORY_SOFTWARE_RELEASE_id,start2,count2,HISTORY_SOFTWARE_RELEASE);
   count2(1)=4;  ret=nf_put_vara_int(ncid, HISTORY_REFERENCE_id,start2,count2,HISTORY_REFERENCE);
   count2(1)=14;  ret=nf_put_vara_text(ncid, HISTORY_DATE_id,start2,count2,HISTORY_DATE);
   count2(1)=4;  ret=nf_put_vara_text(ncid, HISTORY_ACTION_id,start2,count2,HISTORY_ACTION);
                 ret=nf_put_vara_text(ncid, HISTORY_START_INDEX_id,start2,count2,HISTORY_START_INDEX);
                 ret=nf_put_vara_text(ncid, HISTORY_STOP_INDEX_id,start2,count2,HISTORY_STOP_INDEX);
   count2(1)=16;  ret=nf_put_vara_text(ncid, HISTORY_QCTEST_id,start2,count2,HISTORY_QCTEST);

end subroutine put_nctraj_histrec
!====================================================================pres PRES
subroutine def_nctraj_param(caller,ncid,PAR,PAR_ids);
character(*), intent( in )     :: caller
integer, intent( in )          :: ncid      ! LNetCDF file id
character(*), intent( in )      :: PAR      ! PARAMETER to define
integer, intent(out )          :: PAR_ids(5);
integer :: DATA_dims(NDIMS1);
character(len=48) :: nam

integer :: loc
integer :: iret
    integer :: ret = 0;
character(len=16), parameter   :: sub = 'def_nctraj_param';

DATA_dims(1)         = N_MEASUREMENT_dimid; 

nam=trim(PAR);                     ret=nf_def_var(ncid,nam,NF_REAL,NDIMS1,DATA_dims,PAR_ids(1)); loc=1; call cke(sub,ret,loc);
nam=trim(PAR)//"_QC";              ret=nf_def_var(ncid,nam,NF_CHAR,NDIMS1,DATA_dims,PAR_ids(2)); loc=2; call cke(sub,ret,loc);
nam=trim(PAR)//"_ADJUSTED";        ret=nf_def_var(ncid,nam,NF_REAL,NDIMS1,DATA_dims,PAR_ids(3)); loc=3; call cke(sub,ret,loc);
nam=trim(PAR)//"_ADJUSTED_QC";     ret=nf_def_var(ncid,nam,NF_CHAR,NDIMS1,DATA_dims,PAR_ids(4)); loc=4; call cke(sub,ret,loc);
nam=trim(PAR)//"_ADJUSTED_ERROR";  ret=nf_def_var(ncid,nam,NF_REAL,NDIMS1,DATA_dims,PAR_ids(5)); loc=5; call cke(sub,ret,loc);

end subroutine def_nctraj_param

  !====================================================================
 
  !====================================================================
  subroutine def_nctraj_param_att(caller,ncid,ipar,PAR,PAR_ids);
    character(*), intent( in )     :: caller
    integer, intent( in )          :: ncid      ! LNetCDF file id
    integer, intent( in )          :: ipar	! parameter number
    character(*), intent( in )      :: PAR      ! PARAMETER to define
    integer, intent(out )          :: PAR_ids(5);

    character(len=48) :: nam
    character(len=14) :: unit; integer :: lunit;	! unit name, length of unit name
    character(len=41) :: lnam; integer :: llnam
    character(len=86) :: lenam; integer :: llenam
    character(len=42) :: snam; integer :: lsnam

    character(len=1)  :: PAR_qcfl = ' ';

    integer :: loc
    integer :: iret
    integer :: ret = 0;
    character(len=17), parameter   :: sub = 'ncp_def_param_att';

    unit = PAR_unit(ipar); lunit = len(trim(unit));
    lnam = PAR_lnam(ipar); llnam = len(trim(lnam));
    snam = PAR_snam(ipar); lsnam = len(trim(snam));
    lenam = PAR_lenam(ipar); llenam = len(trim(lenam));

    ! attributes for PAR

     ret=nf_put_att_text(ncid, PAR_ids(1), 'long_name',     llnam, trim(lnam));
     ret=nf_put_att_text(ncid, PAR_ids(1), 'standard_name', lsnam, trim(snam));
     ret=nf_put_att_real(ncid, PAR_ids(1), '_FillValue',    NF_REAL, 1, PAR_fill(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(1), 'units',         lunit, trim(unit));
     ret=nf_put_att_real(ncid, PAR_ids(1), 'valid_min',     NF_REAL, 1,PAR_vmin(ipar));
     ret=nf_put_att_real(ncid, PAR_ids(1), 'valid_max',     NF_REAL, 1,PAR_vmax(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(1), 'C_format',      len(trim(DEFCFOR(ipar))), DEFCFOR(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(1), 'FORTRAN_format',len(trim(DEFFFOR(ipar))), DEFFFOR(ipar));
    if(PAR == "PRES")then
        ret=nf_put_att_text(ncid, PAR_ids(1), 'axis',1, 'Z');
    endif

    ! attributes for PAR_qc

     ret=nf_put_att_text(ncid, PAR_ids(2), 'long_name',   12, 'quality flag' );
     ret=nf_put_att_text(ncid, PAR_ids(2), 'conventions', 22, 'Argo reference table 2' );
     ret=nf_put_att_text(ncid, PAR_ids(2), '_FillValue',   1, ' ');

   ! calculating resolution for p,t,s variables
    if(pres_gain_phy /= 0 .and. PAR == "PRES")then
       DEFRSLN(ipar)= 1.0/pres_gain_phy
    elseif(temp_gain_phy /= 0 .and. PAR == "TEMP")then
       DEFRSLN(ipar) = 1.0/temp_gain_phy
    elseif(psal_gain_phy /= 0 .and. PAR == "PSAL")then
       DEFRSLN(ipar) = 1.0/psal_gain_phy
    endif
    
     ! attributes for PAR_adjusted
     ret=nf_put_att_text(ncid, PAR_ids(3), 'long_name',     llnam, trim(lnam));
     ret=nf_put_att_text(ncid, PAR_ids(3), 'standard_name', lsnam, trim(snam));
     ret=nf_put_att_real(ncid, PAR_ids(3), '_FillValue',    NF_REAL, 1, PAR_fill(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(3), 'units',         lunit, trim(unit));
     ret=nf_put_att_real(ncid, PAR_ids(3), 'valid_min',     NF_REAL, 1,PAR_vmin(ipar));
     ret=nf_put_att_real(ncid, PAR_ids(3), 'valid_max',     NF_REAL, 1,PAR_vmax(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(3), 'C_format',      len(trim(DEFCFOR(ipar))), DEFCFOR(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(3), 'FORTRAN_format',len(trim(DEFFFOR(ipar))), DEFFFOR(ipar));
     ret=nf_put_att_real(ncid, PAR_ids(3), 'resolution',    NF_REAL, 1, DEFRSLN(ipar));
    if(PAR == "PRES")then
        ret=nf_put_att_text(ncid, PAR_ids(3), 'axis',1, 'Z');
    endif
    
    ! attributes for PAR_adjusted_qc

     ret=nf_put_att_text(ncid, PAR_ids(4), 'long_name',   12, 'quality flag' );
     ret=nf_put_att_text(ncid, PAR_ids(4), 'conventions', 22, 'Argo reference table 2' );
     ret=nf_put_att_text(ncid, PAR_ids(4), '_FillValue',   1, ' ');


    ! attributes for PAR_error

     ret=nf_put_att_text(ncid, PAR_ids(5), 'long_name',     llenam, trim(lenam));
     ret=nf_put_att_real(ncid, PAR_ids(5), '_FillValue',    NF_REAL, 1, PAR_fill(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(5), 'units',         lunit, trim(unit));
     ret=nf_put_att_text(ncid, PAR_ids(5), 'C_format',      len(trim(DEFCFOR(ipar))), DEFCFOR(ipar));
     ret=nf_put_att_text(ncid, PAR_ids(5), 'FORTRAN_format',len(trim(DEFFFOR(ipar))), DEFFFOR(ipar));
     ret=nf_put_att_real(ncid, PAR_ids(5), 'resolution',    NF_REAL, 1, DEFRSLN(ipar));

    !Resetting the array to default just in case it's been changed
    DEFRSLN(1:MAXPAR) = (/  0.1,     0.001,   0.0001,    0.001,    0.001   /)

  end subroutine def_nctraj_param_att

  !====================================================================
 
  !====================================================================
  subroutine put_nctraj_param(caller,ncid,ipar,PAR,PAR_ids);

    implicit none

    character(*), intent( in )     :: caller
    integer, intent( in )          :: ncid       ! LNetCDF file id
    integer, intent( in )          :: ipar       ! PARAMETER number
    character(*), intent( in )      :: PAR       ! PARAMETER name to load (only used for annnotation)
    integer, intent( in )          :: PAR_ids(5) ! PARAMETER nc ids;

    integer :: iret
    integer :: ret = 0;
    integer :: loc
    integer :: ireg(MAXPRF), iadj(MAXPRF);
!obsolete    integer :: count(1), start(1)
    integer :: count, start
    real    :: DATAREG(MAXLEV); character(len=1) :: FLAGREG(MAXLEV);
    real    :: DATAADJ(MAXLEV); character(len=1) :: FLAGADJ(MAXLEV);

    character(len=16), parameter   :: sub = 'put_nctraj_param';
    if(DBLEV /= 37)then;
	if(DEBUG)then; write(msg,'(a," load param number ",i2," variable: ",a)')sub,ipar,PAR; call wrtdbg(sub,msg); endif
    endif
    count = n_measurement; 
    start = 1;
  
     !if(DEBUG),*)"n_measurement == ",n_measurement; 
      do i=1,n_measurement ! n_measurement now has values from meta file and profile zero or 1's positions
	if(double_trajectory_parameter_measurement(PRES_TRAJ,i) /= FILLDBLE .and. &
	   double_trajectory_parameter_measurement(PRES_TRAJ,i) /= -999.0)then	
		!read(trim(trajectory_parameter_measurement(PRES_TRAJ,i)),*)pres_traj_1d(i)
  
		pres_traj_1d(i)=double_trajectory_parameter_measurement(PRES_TRAJ,i)
	endif
	if(double_trajectory_parameter_measurement(TEMP_TRAJ,i) /= FILLDBLE .and. &
	   double_trajectory_parameter_measurement(TEMP_TRAJ,i) /= -999.0)then
		!read(trajectory_parameter_measurement(TEMP_TRAJ,i),*)temp_traj_1d(i)
		temp_traj_1d(i)=double_trajectory_parameter_measurement(TEMP_TRAJ,i)
	endif
        if(double_trajectory_parameter_measurement(PSAL_TRAJ,i) /= FILLDBLE .and. &
	    double_trajectory_parameter_measurement(PSAL_TRAJ,i) /= -999.0)then
		!read(trajectory_parameter_measurement(PSAL_TRAJ,i),*)psal_traj_1d(i)
	        psal_traj_1d(i)=double_trajectory_parameter_measurement(PSAL_TRAJ,i)
	endif
      enddo

!obsolete   write(msg,'( a,":",i5,i5,i3,i3,f10.2)')PAR,ncid, PAR_ids(1),start, count, pres_traj_1d(1);
!obsolete   call wrttrc(sub,msg);
!obsolete   do i=1,71
!obsolete      write(TRC,'(f15.5,f15.5,f15.5)') pres_traj_1d(i), temp_traj_1d(i), psal_traj_1d(i);
!obsolete   enddo 

   if(PAR == "PRES") ret=nf_put_vara_real(ncid, PAR_ids(1), start, count, pres_traj_1d);  loc=204; call cke(sub,ret,loc);
!obsolete   if(PAR == "PRES") write(TRC,'(a,i15)')'AAAAA',ret;
   
   if(PAR == "TEMP") ret=nf_put_vara_real(ncid, PAR_ids(1), start, count, temp_traj_1d);  loc=205; call cke(sub,ret,loc);
!obsolete   if(PAR == "TEMP") write(TRC,'(a,i15)')'BBBBB',ret;
   
   if(PAR == "PSAL") ret=nf_put_vara_real(ncid, PAR_ids(1), start, count, psal_traj_1d);  loc=206; call cke(sub,ret,loc);
!obsolete   if(PAR == "PSAL") write(TRC,'(a,i15)')'CCCCC',ret;

!obsolete   if(PAR == "PRES")then
!obsolete    ret=nf_put_vara_real(ncid, PAR_ids(1), start, count, pres_traj_1d);  loc=204; call cke(sub,ret,loc);
!obsolete    write(msg,'(a,i15)')'AAAAA',ret;call wrttrc(sub,msg);
!obsolete   endif
!obsolete   if(PAR == "TEMP")then
!obsolete    ret=nf_put_vara_real(ncid, PAR_ids(1), start, count, temp_traj_1d);  loc=205; call cke(sub,ret,loc);
!obsolete    write(msg,'(a,i15)')'BBBBB',ret;call wrttrc(sub,msg);
!obsolete   endif
!obsolete   if(PAR == "PSAL")then
!obsolete    ret=nf_put_vara_real(ncid, PAR_ids(1), start, count, psal_traj_1d);  loc=206; call cke(sub,ret,loc);
!obsolete    write(msg,'(a,i15)')'CCCCC',ret;call wrttrc(sub,msg);
!obsolete    endif

   if(PAR == "PRES") ret=nf_put_vara_text(ncid, PAR_ids(2),start,n_measurement,pres_qc_traj);  loc=207; call cke(sub,ret,loc);
   if(PAR == "TEMP") ret=nf_put_vara_text(ncid, PAR_ids(2),start,n_measurement,temp_qc_traj);  loc=208; call cke(sub,ret,loc);
   if(PAR == "PSAL") ret=nf_put_vara_text(ncid, PAR_ids(2),start,n_measurement,psal_qc_traj);  loc=209; call cke(sub,ret,loc);
   
  end subroutine put_nctraj_param

  !====================================================================
  subroutine def_nctraj_params(caller,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    integer,      intent( in ) :: ncid          ! file id

    integer :: iret
    integer :: ret = 0;
    integer :: loc = 0;
    integer :: ipar;
    integer :: TMP_ids(5);	! PAR PAR_QC PAR_ADJ PAR_ADJ_QC PAR_ERR

    character(len=16), parameter :: sub = 'nctraj_parms'

    N_PARAM = nparam;

    if(DBLEV>0)then;write(msg,*)'In ',sub,' hasNCPAR_traj: ',hasNCPAR_traj; call wrtdbg(sub,msg); endif
    
    do ipar=1,MAXPAR
      if(ncparname(ipar) /= "DOXY")then
	if(hasNCPAR_traj(ipar))then
	    if(DBLEV>0)then;
		write(msg,'("Define PARAMETER array for ipar: ",i2," par: ",a)')ipar,ncparname(ipar);
		call wrtdbg(sub,msg);
	    endif
	    call def_nctraj_param(caller,ncid,ncparname(ipar),TMP_ids);
	    call def_nctraj_param_att(sub,ncid,ipar,ncparname(ipar),TMP_ids);
	    do inca=1,5
		VAR_ids(inca,ipar) = TMP_ids(inca)
	    enddo
	    write(msg,*)'TMPids @1 ',TMP_ids; call wrtdbg(sub,msg);
	else
	    if(DBLEV>0)then; write(msg,'("Do NOT define PARAMETER array for ipar: ",i2," par: ",a)')ipar,ncparname(ipar); endif
	endif
      endif
    enddo

    if(hasDOXY(3))then;write(msg,'("W DOXYA will NOT be written to profile nc file")');call wrtdbg(sub,msg);endif
    if(hasDOXY(4))then;write(msg,'("W DOXYB will NOT be written to profile nc file")');call wrtdbg(sub,msg);endif
    if(hasDOXY(5))then;write(msg,'("W DOXYC will NOT be written to profile nc file")');call wrtdbg(sub,msg);endif

    if(DBLEV>0)then; write(msg,'("<< Leave >>")'); call wrtdbg(sub,msg); endif

    !==================================================================

    end subroutine def_nctraj_params

    !==================================================================


  subroutine put_nctraj_params(caller,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    integer,      intent( in ) :: ncid          ! file id 

    integer :: iret
    integer :: ret = 0;
    integer :: loc = 0;
    integer :: ipar
    integer :: TMP_ids(5);      ! PAR PAR_QC PAR_ADJ PAR_ADJ_QC PAR_ERR

    integer :: nparout

    character(len=16), parameter :: sub = 'nctraj_parms'

    if(DBLEV>0)then; write(msg,'("<< Enter ",a," >>")')sub; call wrtdbg(sub,msg); endif

    ! NOTE - do NOT fill the nc variable PROFILE_CNDC_QC, it is not qc'd, so leave it as FILL
    nparout = 0;
    do ipar=1,MAXPAR
      if(ncparname(ipar) /= "DOXY")then
        if(hasNCPAR_traj(ipar))then
	    nparout = nparout + 1
	    do inca=1,5;
               TMP_ids(inca) = VAR_ids(inca,ipar);
            enddo
	    if(DEBUG) write(msg,'("PUTS calls PUT for ipar: ",i2," PAR: ",a," ids: ",5i3)')ipar,ncparname(ipar),TMP_ids
            call put_nctraj_param(sub,ncid,ipar,ncparname(ipar),TMP_ids)
	else
	    if(DEBUG) write(msg,'("PUTS skips PUT for ipar: ",i2," PAR: ",a," hasNCPAR_traj=F")')ipar,ncparname(ipar)
	endif
	if(nparout == 0)then
            write(errmsg(1),'("NON-FATAL ERROR 284 in nctraj_params mod:nctraj_arrays")')
            write(errmsg(2),'("NON-FATAL ERROR 284 There were NO traj parameters identified")')
            write(errmsg(3),'("NON-FATAL ERROR 284  wmoid: ",a," profile: ",a)')trim(wmoid),trim(profile)
            call error_report(sub,284,3,errmsg); ! put_params in nctraj_arrays NO traj parameters identified"
            filmsg(LTRAJ:LTRAJ) = 'f'; ! NON-FATAL ERROR 284 in nctraj_arrays, nctraj_params
        endif
	if(DEBUG) call wrtdbg(sub,msg);
    endif
    enddo
    if(DBLEV>0)then; write(msg,'("<< Leave ",a," >>")')sub; call wrtdbg(sub,msg); endif

    !==================================================================

    end subroutine put_nctraj_params

! ===============================================================================
! Subroutine to decode the position block from the QC Files which have Measurement Codes.
! NOTE:  Not sure if this is the right place for this subroutine, need to check with Vicki
! ===============================================================================
   subroutine add_position_to_trajmc (caller, cycle_in_process, juld_incoming)

    character(*), intent( in ) :: caller         ! who is calling me?
    integer,      intent( in ) :: cycle_in_process
    !logical,      intent( out ) :: is_position_added
    integer :: iyr,imo,idy,ihh,imn,isc,idate(6) !n_meas_index
    double precision,  intent( in ) :: juld_incoming
    equivalence (idate(1),iyr), (idate(2),imo),(idate(3),idy),(idate(4),ihh),(idate(5),imn),(idate(6),isc)
    character (len=1) :: result_juld_stat

    character(len=18), parameter :: sub = 'add_posn_to_trajmc'

             do i=1,ppos
                ! "Iridium" position info should NOT be added in Traj file
                if((prev_SAT(i) == 'I' .and. trans_type == 'IRIDIUM').or.(prev_RTPQC(i)=='8')) cycle 
                ! Ref: Claudia's email on 11/05/2015 on "traj.nc" ,  don't add position prev_RTPQC(i)=='8'
                ! since it indicates "interpolated pos", Exclude-Interpolated-position in traj file.
                iyr = prev_year(i)
                imo = prev_mon(i)
                idy = prev_day(i)
                ihh = prev_hh(i)
                imn = prev_mm(i)
                isc = prev_sc(i)
                if((juldays1950(idate)<= juld_incoming) .AND. (.NOT. pos_added_flag(i)))then
               ! John Gilson says: GPS time is only accurate to 1 minute (no seconds transmitted), so time of last position may be
               ! before end of transmission if GPS fix time time is rounded down. He als wants the last position (MC703) to
               ! come after MC800. Revising the if statement from (juldays1950(idate) <= juld_incoming) to
               ! (juldays1950(idate) <= juld_incoming - double(1./24./60.))
                    CYCLE_NUMBER_TRAJ(n_measurement)=cycle_in_process;
                    call fill_posn_qc(sub,n_measurement,i);
                    call fill_juld_status('POS',result_juld_stat);
                    JULD_STATUS_TRAJ(n_measurement) = result_juld_stat
                    !write(trajectory_parameter_measurement(MCCODE,n_measurement),*)'703';
                    int_trajectory_parameter_measurement(MCCODE,n_measurement)=703
                    double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
                    MC703_count=MC703_count+1; !counts how many 703s were added yet in the cycle.
                    call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
                    call fill_lat_lon_line(i,n_measurement)
                    if(MC703_count==1)then ! This is the first valid location in this cycle.
                        JULD_FIRST_LOCATION(cyc_count) = juldays1950(idate);
                        JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_stat;
                    endif
                    JULD_LAST_LOCATION(cyc_count) = juldays1950(idate) !this could be last location in this cycle.
                    JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_stat;
                    pos_added_flag(i) = .true. ! Marking the flag as position line is added to the traj array
                    n_measurement = n_measurement + 1
                    
                    !For FAST SURFACE ACTIVATED cycles, MC690 will occur after first MC703.
                    !Note: MC690 is NOT a position code, it is added here only if occurs after first GPS fix. 
                    if(fst_srf_actvtd .and. last_rise_pres_juld /= FILLDBLE)then
                         if(last_rise_pres_juld >= juldays1950(idate))then
                             double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=last_rise_pres_juld;
                             call fill_juld_status('OTH',result_juld_stat)
                             JULD_STATUS_TRAJ(n_measurement)=result_juld_stat;
                             call fill_juld_qc(lr_yr,lr_mo,lr_dy,lr_hh,lr_mn,lr_sc,n_measurement) !use the JULD_QC from read_pqc
                             double_trajectory_parameter_measurement(PRES_TRAJ,n_measurement)=last_rise_pres;
                             pres_qc_traj(n_measurement) = '0';
                             int_trajectory_parameter_measurement(MCCODE,n_measurement)=LAST_RISE_PRES_MC;
                             CYCLE_NUMBER_TRAJ(n_measurement)=cycle_in_process;
                             n_measurement=n_measurement + 1;
                        endif
                    endif
                endif
             enddo
  end subroutine add_position_to_trajmc

! ===============================================================================
  subroutine fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_meas)
  
  integer, intent(in) :: iyr,imo,idy,ihh,imn,isc,n_meas
  logical :: gooddate
  if(iyr == 9999)then
        JULD_QC_TRAJ(n_meas) = '9';
  else
     gooddate = new_check_date(iyr,imo,idy,ihh,imn,isc);
     if(gooddate)then
         JULD_QC_TRAJ(n_meas) = '1';
     else
         JULD_QC_TRAJ(n_meas) = '4';
     endif
  endif
  
  !Ref: John Gilson's email on 08/24/2016, Subj : "a deep solo traj file"
  if(JULD_QC_TRAJ(n_meas) == ' ' .or. JULD_QC_TRAJ(n_meas) == '9')JULD_STATUS_TRAJ(n_meas)=JULD_QC_TRAJ(n_meas); 

  end subroutine fill_juld_qc
! ===============================================================================

! ===============================================================================
 subroutine fill_juld_status(status_of,result_juld_status)
  !use test_arrays

  character(len=*), intent(in) :: status_of
  character(len=*), intent(out) :: result_juld_status
  result_juld_status=FILLCHAR;
  
  if(status_of .eq. 'SOT' )then
     if( inst_type(1:5) == 'APEX_' .or. inst_type(1:6)=='PALACE')then ! all ARGOS (with the exception of PROVOR - need to look at these)
        result_juld_status = '3'
    elseif(  inst_type(1:7) == 'APEXIR_' .or. inst_type(1:8) == 'APEXDIR_')then
	result_juld_status = '2'
    elseif(  inst_type(1:7) == 'SOLOIR_' .or. inst_type(1:8) == 'SOLODIR_' .or. inst_type(1:8) == 'SOLO2IR_')then
	result_juld_status = '2'
    elseif(  inst_type(1:8) == 'NAVISIR_'.or. inst_type(1:8) == 'ALAMOIR_' .or. inst_type(1:7) == 'ALTOIR_' )then
        result_juld_status = '2'
     elseif( inst_type(1:5) == 'SOLO_')then
	result_juld_status = '3'
     !else
     !   result_juld_status = '9'
     endif
  elseif(status_of .eq. 'POS' )then
     if( inst_type(1:5) == 'APEX_' .OR. inst_type(1:5) == 'SOLO_' .OR. inst_type(1:6)=='PALACE')then ! all ARGOS
        result_juld_status = '4'
     elseif(inst_type(1:7) == 'APEXIR_' .OR. inst_type(1:7) == 'SOLOIR_' .OR. &
     inst_type(1:8) == 'APEXDIR_' .OR. inst_type(1:8) == 'SOLO2IR_')then ! all IR_, all OF_, not true for interpolated positions
        result_juld_status = '2'
     elseif(inst_type(1:8) == 'SOLODIR_' .OR. inst_type(1:8) == 'NAVISIR_' .or. inst_type(1:8) == 'ALAMOIR_' &
     .or. inst_type(1:7) == 'ALTOIR_' )then
	result_juld_status = '2'
     endif
  elseif(status_of .eq. 'MET' )then
        result_juld_status = '1'
  elseif(status_of .eq. 'MSG' )then   ! FIRST or LAST MESSAGE from the Sat for MC702 or MC704.
        if(inst_type(1:7) == 'APEXIR_' .OR. inst_type(1:8) == 'NAVISIR_')then  ! since the time is guestimated based on the other times trasmitted by float.
            result_juld_status = '3' ! As per discussion with Claudia and Megan(SIO) on 09/21/2017.
        else
            result_juld_status = '2'	
        endif	
  elseif(status_of .eq. 'OTH' )then
        result_juld_status = '2'
  endif
 
  end subroutine fill_juld_status
! ===============================================================================
! ==============================================================================
subroutine compact(str)
! Converts multiple spaces and tabs to single spaces; deletes control characters;
! removes initial spaces.

character(len=*), intent(in out):: str
character(len=1):: ch
character(len=len_trim(str)):: outstr
integer :: isp,lenstr,k,ich
str=adjustl(str)
lenstr=len_trim(str)
outstr=' '
isp=0
k=0

do i=1,lenstr
  ch=str(i:i)
  ich=iachar(ch)
  select case(ich)
    case(9,32)     ! space or tab character
      if(isp==0) then
        k=k+1
        outstr(k:k)=' '
      endif
      isp=1
    case(33:)      ! not a space, quote, or control character
      k=k+1
      outstr(k:k)=ch
      isp=0
  end select
enddo
str=adjustl(trim(outstr))

end subroutine compact

! ===============================================================================
subroutine fill_lat_lon_line(ipos,n_measurement)

  integer, intent(in) :: n_measurement,ipos
! Fill Satellite Name if its an ARGOS transmission
if(this_class_present)then
   SATELLITE_NAME(n_measurement) = prev_SAT(ipos)
   double_trajectory_parameter_measurement(LAT_TRAJ,n_measurement) = prev_lat(ipos)
   double_trajectory_parameter_measurement(LONG_TRAJ,n_measurement) = prev_lon(ipos)
   POSITION_ACCURACY(n_measurement) = prev_posOrClass(ipos)
else
   !If no CLASS is present, fill the POSITION_ACCURACY with satellite info
   If(prev_SAT(ipos) /= 'I' .and. abs(prev_lat(ipos)) <= 90 .and. prev_RTPQC(ipos) /= '9' &
   .and. prev_RTPQC(ipos) /= '8')then
      POSITION_ACCURACY(n_measurement) = 'G'
      double_trajectory_parameter_measurement(LAT_TRAJ,n_measurement) = prev_lat(ipos);
      double_trajectory_parameter_measurement(LONG_TRAJ,n_measurement) = prev_lon(ipos);
   else
      double_trajectory_parameter_measurement(LAT_TRAJ,n_measurement) = FILLPOSN;
      double_trajectory_parameter_measurement(LONG_TRAJ,n_measurement) = FILLPOSN;
   endif
endif


end subroutine fill_lat_lon_line
! ===============================================================================
subroutine fill_from_lat_lon_line_argos(n_measurement,prevcyc) !For type ARGOS

  integer, intent(in out) :: n_measurement
  integer, intent(in)     :: prevcyc;
  character(len=1)        :: result_juld_status
  integer :: iyr,imo,idy,ihh,imn,isc
  integer :: idate(6),mcode_toadd;

  character(len=20), parameter :: sub = 'fill_from_lat_lon_line_argos'

  equivalence (idate(1),iyr), (idate(2),imo),(idate(3),idy),(idate(4),ihh),(idate(5),imn),(idate(6),isc) 
     CYCLE_NUMBER_INDEX(cyc_count)=prevcyc;
     do i=1,ppos
        if(DEBUG)then;write (msg, *) " Extracting traj in cycle -", prevcyc;  call wrtdbg("fill_from",msg);endif
        iyr = prev_year(i)
        imo = prev_mon(i)
        idy = prev_day(i)
        ihh = prev_hh(i)
        imn = prev_mm(i)
        isc = prev_sc(i)
        if(iyr == 9999 .and. prev_lat(i) > 90) cycle;  ! Date&Time are fill values, skip and go to the next pos line.
        if(prev_RTPQC(i) == '8') cycle; !Exclude-Interpolated-position in traj file, As per discussion with Claudia - 11/14/2018.
        if(i .eq. 1 .or.  i .eq. ppos)then
          if(abs(prev_lat(i)) > 90 .AND. abs(prev_lon(i)) > 180 )then ! is a 999 line
              if(i .eq. 1)then
                mcode_toadd=702;
                if(iyr /= 9999)then
                    JULD_FIRST_MESSAGE(cyc_count)=juldays1950(idate);
                    call fill_juld_status('MSG',result_juld_status);
                    JULD_FIRST_MESSAGE_STATUS(cyc_count) = result_juld_status;
                endif
                if(i .eq. ppos)then ! if that is the only line.
                    n_measurement = n_measurement + 1;
                    CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
                    int_trajectory_parameter_measurement(MCCODE,n_measurement)=mcode_toadd; 
                    if(iyr /= 9999)then
                        double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                        JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                        call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc	
                    endif
                endif 
              endif
              if( i .eq. ppos)then ! else is omitted because there can be two cases. i=ppos OR i = ppos = 1
                mcode_toadd=704;
                if(iyr /= 9999)then
                    JULD_LAST_MESSAGE(cyc_count)=juldays1950(idate)
                    call fill_juld_status('MSG',result_juld_status);
                    JULD_LAST_MESSAGE_STATUS(cyc_count) = result_juld_status;
                endif
              endif
              n_measurement = n_measurement + 1;
              CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
              int_trajectory_parameter_measurement(MCCODE,n_measurement)=mcode_toadd; 
              if(iyr /= 9999)then
                  double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                  JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                  call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
              endif
          else ! NOT a 999 line
              if(i .eq. 1)then
              	 n_measurement = n_measurement + 1;
                 CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;     
                 int_trajectory_parameter_measurement(MCCODE,n_measurement)=702;
                 if(iyr /= 9999)then
                     double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                     call fill_juld_status('MSG',result_juld_status);
                     JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                     call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc               
                     JULD_FIRST_MESSAGE(cyc_count)=juldays1950(idate)
                     JULD_FIRST_MESSAGE_STATUS(cyc_count) = result_juld_status;
                     JULD_FIRST_LOCATION(cyc_count)=juldays1950(idate);
                     call fill_juld_status('POS',result_juld_status);
                     JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_status;
                 endif
              endif        
              n_measurement = n_measurement + 1;
              CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
              int_trajectory_parameter_measurement(MCCODE,n_measurement)=703;
              if(iyr /= 9999)then
                 double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                 call fill_juld_status('POS',result_juld_status);
                 JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                 call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
              endif
              call fill_posn_qc(sub,n_measurement,i);
              call fill_lat_lon_line(i,n_measurement); 
              if(i .ne. 1 .and. JULD_FIRST_LOCATION(cyc_count)==FILLDBLE)then
                  if(iyr /= 9999)then
                      JULD_FIRST_LOCATION(cyc_count)=juldays1950(idate)
                      call fill_juld_status('POS',result_juld_status);
                      JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_status;
                  endif
              endif
              if(i .eq. ppos)then
                   n_measurement = n_measurement + 1;
                   int_trajectory_parameter_measurement(MCCODE,n_measurement)=704;
                   if(iyr /= 9999)then 
                       double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                       call fill_juld_status('MSG',result_juld_status);
                       JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                       call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
                       JULD_LAST_MESSAGE(cyc_count)=juldays1950(idate)
                       JULD_LAST_MESSAGE_STATUS(cyc_count) = result_juld_status;
                       JULD_LAST_LOCATION(cyc_count)=juldays1950(idate)
                       call fill_juld_status('POS',result_juld_status);
                       JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                   endif
                   CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
              endif
              if(i .eq. 1 .and. ppos.eq. 2 .and. abs(prev_lat(ppos)) > 90 .AND. abs(prev_lon(ppos)) > 180 )then
                  if(iyr /= 9999)then
                      JULD_LAST_LOCATION(cyc_count)=juldays1950(idate);
                      call fill_juld_status('POS',result_juld_status);
                      JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                  endif
              endif
          endif
        else ! if(i .eq. 1 .or.  i .eq. ppos)then          i>1 .or. i<ppos
          n_measurement = n_measurement + 1;
          CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
          int_trajectory_parameter_measurement(MCCODE,n_measurement)=703;
          if(iyr /= 9999)then
              double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
              call fill_juld_status('POS',result_juld_status);
              JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
              call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
          endif
          call fill_posn_qc(sub,n_measurement,i);
          call fill_lat_lon_line(i,n_measurement);
          ! Fill FIRST  and LAST MESSAGE and LOCATION If
          if( i .eq. 2 .and. JULD_FIRST_LOCATION(cyc_count)==FILLDBLE)then
              if(iyr /= 9999)then
                  JULD_FIRST_LOCATION(cyc_count)=juldays1950(idate)
                  call fill_juld_status('POS',result_juld_status);
                  JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_status;
              endif
              if(ppos-1 .eq. 2 .and. JULD_LAST_LOCATION(cyc_count)==FILLDBLE)then !If there are only 3 in the position blk only 1 of them is a not 999 line
                  if(iyr /= 9999)then
                      JULD_LAST_LOCATION(cyc_count)=juldays1950(idate)
                      call fill_juld_status('POS',result_juld_status);
                      JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                  endif
              endif
          elseif( i .eq. ppos-1 .and. JULD_LAST_LOCATION(cyc_count)==FILLDBLE)then
              if(iyr /= 9999)then
                  JULD_LAST_LOCATION(cyc_count)=juldays1950(idate)
                  call fill_juld_status('POS',result_juld_status);
                  JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
              endif
          endif
        endif ! if(i .eq. 1 .or.  i .eq. ppos)then
     enddo
end subroutine fill_from_lat_lon_line_argos
! ===============================================================================
subroutine fill_from_lat_lon_line_irid(n_measurement,prevcyc) !For type IRIDIUM

  integer, intent(in out) :: n_measurement
  integer, intent(in)     :: prevcyc;
  character(len=1)        :: result_juld_status
  integer :: iyr,imo,idy,ihh,imn,isc
  integer :: idate(6),mcode_toadd;

  character(len=20), parameter :: sub = 'fill_from_lat_lon_line_irid'

  equivalence (idate(1),iyr), (idate(2),imo),(idate(3),idy),(idate(4),ihh),(idate(5),imn),(idate(6),isc) 
       
     if(int_trajectory_parameter_measurement(MCCODE,n_measurement) < 700)then !MC703 are to be added now, before MC700 gets added.
        CYCLE_NUMBER_INDEX(cyc_count)=prevcyc;
        do i=1,ppos
           if(DEBUG)then;write (msg, *) " Extracting traj in cycle -", prevcyc;  call wrtdbg("fill_from",msg);endif
           iyr = prev_year(i)
           imo = prev_mon(i)
           idy = prev_day(i)
           ihh = prev_hh(i)
           imn = prev_mm(i)
           isc = prev_sc(i)
           if(iyr == 9999 .and. prev_lat(i) > 90) cycle;  ! Date&Time, and position are fill values, skip and go to the next pos line.
           if(prev_RTPQC(i) == '8') cycle; !Exclude-Interpolated-position in traj file, As per discussion with Claudia - 11/14/2018
           if(i .eq. 1 .or.  i .eq. ppos)then
             if(abs(prev_lat(i)) > 90 .AND. abs(prev_lon(i)) > 180 )then ! is a 999 line
                 if(iyr /= 9999)then
                   if(i .eq. 1)then
                     first_msg_juld=juldays1950(idate);
                     fm_yr=iyr; fm_mo=imo ; fm_dy=idy ;  fm_hh=ihh ; fm_mn=imn ; fm_sc=isc ;
                   endif
                   if(i .eq. ppos)then ! else is omitted because there can be two cases. i=ppos OR i = ppos = 1
                     last_msg_juld=juldays1950(idate);
                     lm_yr=iyr; lm_mo=imo ; lm_dy=idy ;  lm_hh=ihh ; lm_mn=imn ; lm_sc=isc ;
                   endif
                 endif
             else ! NOT a 999 line 
                   if(iyr /= 9999)then 
                      if(i .eq. 1)then
              	        first_msg_juld=juldays1950(idate);
                        fm_yr=iyr; fm_mo=imo ; fm_dy=idy ;  fm_hh=ihh ; fm_mn=imn ; fm_sc=isc ;
                        JULD_FIRST_LOCATION(cyc_count)=juldays1950(idate);
                        call fill_juld_status('POS',result_juld_status);
                        JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_status;
                      endif
                   endif   
                   n_measurement = n_measurement + 1;
                   CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
                   int_trajectory_parameter_measurement(MCCODE,n_measurement)=703;
                   if(iyr /= 9999)then
                       double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                       call fill_juld_status('POS',result_juld_status);
                       JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                       call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
                   endif
                   call fill_posn_qc(sub,n_measurement,i);
                   call fill_lat_lon_line(i,n_measurement);
                if(i .ne. 1 .and. JULD_FIRST_LOCATION(cyc_count)==FILLDBLE)then
                    if(iyr /= 9999)then
                        JULD_FIRST_LOCATION(cyc_count)=juldays1950(idate)
                        call fill_juld_status('POS',result_juld_status);
                        JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_status;
                    endif
                endif
                if(i .eq. ppos)then
                    if(iyr /= 9999)then
                        last_msg_juld=juldays1950(idate);
                        lm_yr=iyr; lm_mo=imo ; lm_dy=idy ;  lm_hh=ihh ; lm_mn=imn ; lm_sc=isc ;
                        JULD_LAST_LOCATION(cyc_count)=juldays1950(idate)
                        call fill_juld_status('POS',result_juld_status);
                        JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                    endif
                endif
                if(i .eq. 1 .and. ppos.eq. 2 .and. abs(prev_lat(ppos)) > 90 .and. abs(prev_lon(ppos)) > 180 )then
                    if(iyr /= 9999)then
                        JULD_LAST_LOCATION(cyc_count)=juldays1950(idate);
                        call fill_juld_status('POS',result_juld_status);
                        JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                    endif
                endif
            endif
          else ! belongs to - "if(i .eq. 1 .or.  i .eq. ppos)then"          i>1 .or. i<ppos
                n_measurement = n_measurement + 1;
                CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
                int_trajectory_parameter_measurement(MCCODE,n_measurement)=703;
                if(iyr /= 9999)then
                    double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = juldays1950(idate);
                    call fill_juld_status('POS',result_juld_status);
                    JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
                    call fill_juld_qc(iyr,imo,idy,ihh,imn,isc,n_measurement) !use the JULD_QC from read_pqc
                endif
                call fill_posn_qc(sub,n_measurement,i);
                call fill_lat_lon_line(i,n_measurement);
             
               ! Fill FIRST  and LAST MESSAGE and LOCATION If
               if( i .eq. 2 .and. JULD_FIRST_LOCATION(cyc_count)==FILLDBLE)then
                     if(iyr /= 9999)then
                         first_msg_juld=juldays1950(idate);
                         fm_yr=iyr; fm_mo=imo ; fm_dy=idy ;  fm_hh=ihh ; fm_mn=imn ; fm_sc=isc ;
                         JULD_FIRST_LOCATION(cyc_count)=juldays1950(idate);
                         call fill_juld_status('POS',result_juld_status);
                         JULD_FIRST_LOCATION_STATUS(cyc_count) = result_juld_status;
                         if(ppos-1 .eq. 2 .and. JULD_LAST_LOCATION(cyc_count)==FILLDBLE)then !If there are only 3 in the position blk only 1 of them is a not 999 line
                            JULD_LAST_LOCATION(cyc_count)=juldays1950(idate)
                            call fill_juld_status('POS',result_juld_status);
                            JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                         endif 
                     endif
               elseif( i .eq. ppos-1 .and. JULD_LAST_LOCATION(cyc_count)==FILLDBLE)then
                     if(iyr /= 9999)then
                         JULD_LAST_LOCATION(cyc_count)=juldays1950(idate);
                         call fill_juld_status('POS',result_juld_status);
                         JULD_LAST_LOCATION_STATUS(cyc_count) = result_juld_status;
                     endif
               endif
          endif ! if(i .eq. 1 .or.  i .eq. ppos)then
        enddo
     else   !MC703 and MC700 are already added, now add MC702 and MV704.
        n_measurement = n_measurement + 1;
        CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
        int_trajectory_parameter_measurement(MCCODE,n_measurement)=702; 
        if (first_msg_juld /= FILLDBLE)then
            double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = first_msg_juld;
            call fill_juld_status('MSG',result_juld_status);
            JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
            call fill_juld_qc(fm_yr,fm_mo,fm_dy,fm_hh,fm_mn,fm_sc,n_measurement)
            JULD_FIRST_MESSAGE(cyc_count)=first_msg_juld;
            JULD_FIRST_MESSAGE_STATUS(cyc_count) = result_juld_status;
        endif    
        
        n_measurement = n_measurement + 1;
        CYCLE_NUMBER_TRAJ(n_measurement)=prevcyc;
        int_trajectory_parameter_measurement(MCCODE,n_measurement)=704; 
        if (last_msg_juld /= FILLDBLE)then
            double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) = last_msg_juld;
            call fill_juld_status('MSG',result_juld_status);
            JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
            call fill_juld_qc(lm_yr,lm_mo,lm_dy,lm_hh,lm_mn,lm_sc,n_measurement)
            JULD_LAST_MESSAGE(cyc_count)=last_msg_juld;
            JULD_LAST_MESSAGE_STATUS(cyc_count) = result_juld_status;
        endif
         
     endif
end subroutine fill_from_lat_lon_line_irid
! ===============================================================================

! ===============================================================================
! To fill QC flags corresponding to the values in Core Argo Param arrays in Traj file.
subroutine fill_param_qc(param_type_qc,n_meas)

  integer, intent(in out) :: n_meas,param_type_qc
  integer :: ig, tmp_val
  real, parameter :: range_min(4) = (/   -5.0, -2.5,  2.0,  -0.5 /)
  real, parameter :: range_max(4) = (/20000.0, 40.0, 41.0, 600.0 /) ! P max range just a ridiculous number
  character(len=1), parameter :: char_BAD = '4'
  character(len=13), parameter :: sub = 'fill_param_qc'
  
   
      if(double_trajectory_parameter_measurement(param_type_qc,n_meas) == FILLDBLE)return; ! fill value
      if(double_trajectory_parameter_measurement(param_type_qc,n_meas) == FILL)return;     ! fill value
      if(double_trajectory_parameter_measurement(param_type_qc,n_meas) == FILLNC)return;   ! fill value
  
      if(param_type_qc == PRES_TRAJ)pres_qc_traj(n_meas) = '0' ! Indicates "Not QCed" yet
      if(param_type_qc == TEMP_TRAJ)temp_qc_traj(n_meas) = '0' ! Indicates "Not QCed" yet
      if(param_type_qc == PSAL_TRAJ)psal_qc_traj(n_meas) = '0' ! Indicates "Not QCed" yet
      
      
      ! Do the Grey List check , adapted from read_meta.f90   
      ! Check for multiple grey list and date ranges too here!      
!     if(add_to_grey_list .and. ngl > 0)then! there is atleast one grey_list for this float.
!          do ig=1,ngl
!              if(param_type_qc == PRES_TRAJ)then
!                  if(param_on_grey_list(ig,1) .and. grey_flag(ig,1)>1)write(pres_qc_traj(n_meas),"(i0)")grey_flag(ig,1);
!              elseif(param_type_qc == TEMP_TRAJ)then 
!                  if(param_on_grey_list(ig,2) .and. grey_flag(ig,2)>1)write(temp_qc_traj(n_meas),"(i0)")grey_flag(ig,2);
!              elseif(param_type_qc == PSAL_TRAJ)then 
!                  if(param_on_grey_list(ig,3) .and. grey_flag(ig,3)>1)write(psal_qc_traj(n_meas),"(i0)")grey_flag(ig,3);
!              endif
!          enddo
!      endif
      
      !Perofrm Range check , adapted from qc06_gorss_test.f90
      if(param_type_qc == PRES_TRAJ)then
          if(double_trajectory_parameter_measurement(param_type_qc,n_meas)< range_min(1))pres_qc_traj(n_meas)=char_BAD;
          if(double_trajectory_parameter_measurement(param_type_qc,n_meas)> range_max(1))pres_qc_traj(n_meas)=char_BAD;
      elseif(param_type_qc == TEMP_TRAJ)then
          if(double_trajectory_parameter_measurement(param_type_qc,n_meas)< range_min(2))temp_qc_traj(n_meas)=char_BAD;
          if(double_trajectory_parameter_measurement(param_type_qc,n_meas)> range_max(2))temp_qc_traj(n_meas)=char_BAD;  
      elseif(param_type_qc == PSAL_TRAJ)then
          if(double_trajectory_parameter_measurement(param_type_qc,n_meas)< range_min(3))psal_qc_traj(n_meas)=char_BAD;
          if(double_trajectory_parameter_measurement(param_type_qc,n_meas)> range_max(3))psal_qc_traj(n_meas)=char_BAD; 
      endif
    
     ! Perofrm Regional Range check, adpapt from qc07_regional_test.
     ! Before doing qc07, What will be the lat and lon for calling "is_profile_regional" ????????????????????????
     ! call is_profile_regional(lat,lon);
     	     
end subroutine fill_param_qc
!====================================================================
end module nctraj_arrays
! ===============================================================================
