! ===========================================================================================================
! module netcdf_arrays.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 201911?5 VH     Moved MAX_MEASUREMENT from netcdf_arrays to to data_arrays.
! 20190301 JN     Handle the files with no primary core argo profile, but only secondary profile (from SIO & PMEL)
! 20181218 JN     Exclude interpolated positions in the Traj.nc file
! 20181109 JN     Double MAX_MEASUREMENTto 200000
! 20180724 JN     fill_posn_qc (when called by extraj) Do NOT issue WARNING 199 if n_meas=1, the launch posiiton 
! 20180417 VH     General WARNING revisions.
! 20180307 VH     "no primary profile" changes merged into the new BR version
! 20180307 VH     Extensive changes to create BR file
! 20180129 VH     Numerous changes for biop.
! 20180111 JN     "no primary profile": account for nprf_start>1 (VH merged 20180307)
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:
 
! This module contains variable/array definitions and functions/subroutines used by ALL make_nc<> functions.
! It contains vars/arrs that all nc funcitns must see, and contains the subroutines the create a new nc file,
! define definitions, define 'common' varialbes (those present in all versions of nc files) and to put the
! values for these common vars.
! ===========================================================================================================

module netcdf_arrays

use communication                                                ! module
use data_arrays                                                  ! module

implicit none

! ===========================================================================================================

include 'netcdf.inc'

! ===========================================================================================================
! moedule varialbe/array declarations
! ===========================================================================================================

integer :: DATE_TIME_dimid; integer,parameter :: DATE_TIME_len = 14

! These are used by both ncmeta & nctraj
integer :: count_enter_traj,first_profile_no

integer :: n_config_param = 0;
integer :: n_launch_config_param  = 0;
integer :: n_combined_launch_config_param = 0

integer :: STRING1024_dimid; integer,parameter :: STRING1024_len = 1024
integer :: STRING256_dimid ; integer,parameter :: STRING256_len  =  256
integer :: STRING128_dimid ; integer,parameter :: STRING128_len  =  128
integer :: STRING64_dimid  ; integer,parameter :: STRING64_len   =  64
integer :: STRING32_dimid  ; integer,parameter :: STRING32_len   =  32
integer :: STRING16_dimid  ; integer,parameter :: STRING16_len   =  16
integer :: STRING8_dimid   ; integer,parameter :: STRING8_len    =  8
integer :: STRING4_dimid   ; integer,parameter :: STRING4_len    =  4
integer :: STRING2_dimid   ; integer,parameter :: STRING2_len    =  2

! NOTE on N_CALIB - the "calibration" informatin in th metafile can better be termed "conversin"
! factor - the CALIB dimension is set to 1 to allow for insertion of "real" calibration
! information by the delayed_mode operators

integer :: N_PROF							! how many profiles do we have?
integer :: N_PROF_dimid        ; integer :: N_PROF_len                               ! prof only
integer :: N_PARAM_dimid       ; integer :: N_PARAM_len                              ! prof traj
integer :: N_LEVELS_dimid      ; integer :: N_LEVELS_len                             ! prof only
integer :: N_CALIB_dimid       ; integer :: N_CALIB_len        = 1                   ! prof only SEE NOTE ABOVE
integer :: N_HISTORY_dimid     ; integer :: N_HISTORY_len_prof = NF_UNLIMITED        ! prof unlimited dimension
integer :: N_HISTORY_len_traj = 25                                                   ! val for traj (John request?)
! From UM3.1: N_CYCLE is the number of collected cycles performed by the float.
integer :: N_CYCLE_dimid       ; integer :: N_CYCLE_len                              ! traj only
integer :: N_MEASUREMENT_dimid ; integer :: N_MEASUREMENT_len   = NF_UNLIMITED	! traj unlimited dimension
integer :: N_TECH_PARAM_dimid  ; integer :: N_TECH_PARAM_len = NF_UNLIMITED		! tech unlimited dimension
integer :: N_SENSOR_dimid      ; integer :: N_SENSOR_len
integer :: N_CONFIG_PARAM_dimid  ; integer :: N_CONFIG_PARAM_len
integer :: N_MISSIONS_dimid    ; integer :: N_MISSIONS_len = NF_UNLIMITED		! meta unlimited dimension
integer :: N_LAUNCH_CONFIG_PARAM_dimid ; integer :: N_LAUNCH_CONFIG_PARAM_len
integer :: N_POSITIONING_SYSTEM_dimid ; integer :: N_POSITIONING_SYSTEM_len
integer :: N_TRANS_SYSTEM_dimid ; integer :: N_TRANS_SYSTEM_len

! Here are all the nf_< > variables, dimensins, id's etc, for the COMMON nc entries

integer, parameter :: NDIMS1 = 1;
integer, parameter :: NDIMS2 = 2;
integer, parameter :: NDIMS3 = 3;
integer, parameter :: NDIMS4 = 4;
integer, parameter :: ND1 = 1; ! shorter version of NDIMS1
integer, parameter :: ND2 = 2;! shorter version of NDIMS2
integer, parameter :: ND3 = 3;! shorter version of NDIMS3
integer, parameter :: ND4 = 4;! shorter version of NDIMS4

integer :: DATA_TYPE_id,DATA_TYPE_dims(NDIMS1)
character(len=16) :: DATA_TYPE16;
character(len=32) :: DATA_TYPE32;

integer :: VAR_ids(NCARRS,MAXPAR);                    ! nc ids for ALL variables
integer, parameter :: MAX_PARAM_TYPE = 7 ;

integer :: PLATFORM_NUMBER_id,      PLATFORM_NUMBER_dims;             character(len=8) :: PLATFORM_NUMBER;
integer :: PLATFORM_NUMBER_id_vec,  PLATFORM_NUMBER_dims_vec(NDIMS2); character(len=8) :: PLATFORM_NUMBER_vec(MAXPRF);

integer :: DATA_CENTRE_id,            DATA_CENTRE_dims(NDIMS2);             character(len=2) :: DATA_CENTRE(MAXPRF);
integer :: DATA_CENTRE_id_vec, DATA_CENTRE_dims_vec(NDIMS2); character(len=2) :: DATA_CENTRE_vec(MAXPRF);


integer :: FORMAT_VERSION_id,FORMAT_VERSION_dims(NDIMS1)

integer :: HANDBOOK_VERSION_id,HANDBOOK_VERSION_dims(NDIMS1)
character(len=14) :: HANDBOOK_VERSION = '1.2';      ! The most recent is Dec 2002

integer :: REF_DATE_TIME_id,REF_DATE_TIME_dims(NDIMS1)
character(len=14) :: REFERENCE_DATE_TIME = '19500101000000';            ! 1950 01/01 00:00:00

integer   :: itime8(8), izoneh
integer   ::iyr, imm, idy, ihr, imn, isc
character :: hist_date_time*40;
character :: file_date_time*14; public file_date_time

integer :: DATE_CREATION_id,DATE_CREATION_dims(NDIMS1)

integer :: DATE_UPDATE_id,DATE_UPDATE_dims(NDIMS1)

character(len=1),parameter :: ncflag(0:9) = (/ " ", "1", "2", "3", "4", "5", "6", "7", "8", " " /);
                                             !  0    1    2    3    4    5    6    7    8    9
character(len=1),parameter :: FILLNCFLAG =     " ";

integer :: startTraj2(2),countTraj2(2)

! ====================================================
! from ncprof_arraus, so the arrays are visible to all

integer :: N_PARAM;		! depends on which parameters are present

!                                        PRES     TEMP     CNDC     PSAL    DOXY
real,parameter :: PAR_fill(MAXPAR) = (/ FILLNC,  FILLNC,  FILLNC,  FILLNC, FILLNC /)
! ! 20Jun2016 - separate ncprof & nctraj defs for valic_min,valid_max and long_name ncprof=v3.03, nctraj=v3.1
! mvd real,parameter :: PAR_vmin(MAXPAR) = (/    0.0,    -2.5,     0.0,     2.0,  0.0    /)
! mvd real,parameter :: PAR_vmax(MAXPAR) = (/12000.0,    40.0,     8.5,    41.0,   650.0  /) ! VALID MAX CNDC??? WRONG FIX
real,parameter :: PAR_rsln(MAXPAR) = (/   0.01,   0.001,   0.001,     0.001, 0.001  /)  ! Set same as PSAL

character(len=86) :: PAR_lenam(MAXPAR) =  (/ &
'Contains the error on the adjusted values as determined by the delayed mode QC process', &             ! PRES
'Contains the error on the adjusted values as determined by the delayed mode QC process', &             ! TEMP
'Contains the error on the adjusted values as determined by the delayed mode QC process', &             ! CNDC
'Contains the error on the adjusted values as determined by the delayed mode QC process', &             ! PSAL
'Contains the error on the adjusted values as determined by the delayed mode QC process' /)             ! DOXY
character(len=42) :: PAR_snam(MAXPAR) =  (/       &
'sea_water_pressure                        ', &         ! PPRES
'sea_water_temperature                     ', &         ! TEMP
'sea_water_electrical_conductivity         ', &         ! CNDC
'sea_water_salinity                        ', &         ! PSAL
'moles_of_oxygen_per_unit_mass_in_sea_water' /)         ! DOXY
character(len=14) :: PAR_unit(MAXPAR) =  (/ &
'decibar       ', &                                     ! PRES
'degree_Celsius', &                                     ! TEMP
'mhos/m        ', &                                     ! CNDC
'psu           ', &                                     ! PSAL
'micromole/kg  ' /)                                     ! DOXY

integer,parameter :: DEFIRES(MAXPAR) = (/  1,       3,       5,         3,        3       /)
real              :: DEFRSLN(MAXPAR) = (/  0.1,     0.001,   0.0001,    0.001,    0.001   /) ! default resolution
! FIX agree w traj character(len=6)  :: DEFCFOR(MAXPAR) = (/ '%7.1f ','%10.3f','%12.5f',  '%10.3f', '%10.3f' /) ! default C_format
! FIX agree w traj character(len=5)  :: DEFFFOR(MAXPAR) = (/ 'F7.1 ', 'F10.3', 'F12.5',   'F10.3',  'F10.3'  /)  !default FORTRAN_format

! Keep these for now for the traj job ... 
character(len=6)  :: DEFCFOR(MAXPAR) = (/ '%7.2f ','%10.3f','%12.5f',  '%10.3f', '%10.3f' /) ! default C_format
character(len=5)  :: DEFFFOR(MAXPAR) = (/ 'F7.2 ', 'F10.3', 'F12.5',   'F10.3',  'F10.3'  /)  !default FORTRAN_format

! FIX ! default C_format                                123456   123456   123456     123456    123456
! FIX character(len=6)  :: DEFCFORARGOS(MAXPAR) =   (/ '%7.1f ','%10.3f','%12.5f',  '%10.3f', '%10.3f' /)	! Argos format
! FIX character(len=6)  :: DEFCFORIRIDIUM(MAXPAR) = (/ '%7.2f ','%10.4f','%12.5f',  '%12.5f', '%12.5f' /)	! Iridium format 
! FIX !default FORTRAN_format
! FIX character(len=6)  :: DEFFFORARGOS(MAXPAR) =   (/ 'F7.1  ','F10.3 ','F12.5 ',  'F10.3 ', 'F10.3 '  /)    ! Argos format
! FIX character(len=6)  :: DEFFFORIRIDIUM(MAXPAR) = (/ 'F7.2  ','F10.4 ','F12.5 ',  'F12.5 ', 'F12.5 '  /)    ! Iridium format
! FIX use the lines above before installation into operations
character(len=6)  :: DEFCFORARGOS(MAXPAR) =   (/ '%7.1f ','%10.3f','%12.5f',  '%10.3f', '%10.3f' /)
character(len=6)  :: DEFCFORIRIDIUM(MAXPAR) = (/ '%7.1f ','%10.3f','%12.5f',  '%10.3f', '%10.3f' /)
character(len=6)  :: DEFFFORARGOS(MAXPAR) =   (/ 'F7.1 ', 'F10.3', 'F12.5',   'F10.3',  'F10.3'  /)
character(len=6)  :: DEFFFORIRIDIUM(MAXPAR) = (/ 'F7.1 ', 'F10.3', 'F12.5',   'F10.3',  'F10.3'  /)


character(len=1) :: POSITION_QC_TRAJ(MAXTRAJ)=FILLCHAR;

!                                         123456789012345678901234567890123456789012345678901 - 46 & 51
character(len=46) :: DC_long_name_prof = 'Data centre in charge of float data processing'	! Used in ncprof_arrays.f90
character(len=51) :: DC_long_name_meta = 'Data centre in charge of float real-time processing'	! Used in ncmeta_arrays.f90
character(len=46) :: DC_long_name_traj = 'Data centre in charge of float data processing'	! Used in nctraj_arrays.f90
character(len=46) :: DC_long_name_tech = 'Data centre in charge of float data processing'	! Used here in tech comon

! ===========================================================================================================

double precision,parameter :: JULD_RESOLUTION = 1.0/24.0/3600.0;

! ===========================================================================================================



contains
 
! ===========================================================================================================
! module functions/subroutines
! ===========================================================================================================

subroutine dbg_ncmsg(sub,nctyp,ncjob,msgnc); ! Make a general purpose msg for nc jobs
    character(*), intent( in )     :: sub
    character(*), intent( in )     :: nctyp
    character(*), intent( in )     :: ncjob
    character(*), intent( in )     :: msgnc

    write(msg,'("==== NC ",a," ",a," "a)')nctyp,ncjob,msgnc;
    call wrtdbg(sub,msg);

end subroutine dbg_ncmsg
! ===========================================================================================================
subroutine dbg_ncbeg(sub,nctyp,ncjob); ! Print the BEG message
    character(*), intent( in )     :: sub
    character(*), intent( in )     :: nctyp
    character(*), intent( in )     :: ncjob

    write(msg,'("==== NC ",a," ",a," BEG ================================================")')nctyp,ncjob;
    call wrtdbg(sub,msg)

end subroutine dbg_ncbeg
! ===========================================================================================================
subroutine dbg_ncend(sub,nctyp,ncjob); ! Print the END message
    character(*), intent( in )     :: sub
    character(*), intent( in )     :: nctyp
    character(*), intent( in )     :: ncjob

    write(msg,'("==== NC ",a," ",a," END ================================================")')nctyp,ncjob;
    call wrtdbg(sub,msg);

end subroutine dbg_ncend
! ===========================================================================================================
subroutine dbg_ncret(sub,nctyp,ncjob,rtn); ! Make a message about the return value send by an nc job
    character(*), intent( in )     :: sub
    character(*), intent( in )     :: nctyp
    character(*), intent( in )     :: ncjob
    character(*), intent( in )     :: rtn

    write(msg,'("==== NC ",a," ",a," <<<<< Leave ",a," ret= ",a)')nctyp,ncjob,trim(sub),rtn;
    call wrtdbg(sub,msg);
end subroutine dbg_ncret
! ===========================================================================================================
subroutine end_define_mode(caller,ncid)

    character(*), intent( in ) :: caller        ! prg/sub/funct that is calling
    integer,      intent( in ) :: ncid          ! file id

    integer :: ret = 0;
    integer :: loc = 0;

    character(len=15), parameter :: sub = 'end_define_mode'

    if(DEBUG)then;write(msg,'("NetCDF End define mode called frm ",a)')trim(caller);call wrtdbg(sub,msg);endif
    ret=nf_enddef(ncid);                                                                                     call cke(sub,ret,98);

end subroutine end_define_mode

subroutine close_ncfile(caller,ncid)

    character(*), intent( in ) :: caller        ! prg/sub/funct that is calling
    integer,      intent( in ) :: ncid          ! file id

    integer :: ret = 0;
    integer :: loc = 0;

    character(len=15), parameter :: sub = 'close_ncfile'

    if(DEBUG)then;write(msg,'("NetCDF Close the ncfile")');call wrtdbg(sub,msg);endif
    ret=nf_close(ncid);                                                                                      call cke(sub,ret,99);

end subroutine close_ncfile

subroutine cke(caller,ret,loc);

    character(*), intent( in ) :: caller        ! prg/sub/funct that is calling
    integer,      intent( in ) :: ret          ! Return status of the nf_< > call
    integer,      intent( in ) :: loc           ! call location in the caller

    !character(len=80) :: ncerrmsg            ! message for any nc errors
    !character(len=80) :: nf_stderror
    !external nf_stderror
    integer :: status, tloc

    character(len=11), parameter :: sub = 'CKE check nc errorr'

    ! Add gloc to loc - gloc set by prof=+100,meta=+200,traj=+300,tecj=+400,biop=!500,biot=!600
    tloc = loc + gloc;
 
    if(DEBUG .and. DBLEV > 90)then
	if(len_trim(caller) > 20)then
	    write(msg,'("CKE ret: ",i4," loc: ",i5," nctype: ",a," caller: ",a20)')ret,tloc,gnctype,caller(1:20)
	else
	    write(msg,'("CKE ret: ",i4," loc: ",i5," nctype: ",a," caller: ",a20)')ret,tloc,gnctype,trim(caller)
	endif
	call wrtdbg(sub,msg);
    endif

    if(ret /= NF_NOERR)then
	write(msg,'("CKE ret: ",i4," loc: ",i5," nctype: ",a," caller: ",a20," ERROR detected")')ret,tloc,gnctype,trim(caller);
	call wrtlog(sub,msg);
	write(errmsg(1),'("NON-FATAL ERROR 253 from CKE in netcdf_arrays")');
	write(errmsg(2),'("NON-FATAL ERROR 253 Error ret: ",i10," in nc caller: ",a," at loc ",i6)')ret,caller,tloc
	call error_report(caller,253,2,errmsg); ! cke in netcdf_arrays error during call to netcdf lib
    endif

end subroutine cke

subroutine def_common_vars(caller,nctype,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/tech
    integer,      intent( in ) :: ncid          ! file id

    integer :: ret = 0;
    integer :: loc = 0;

    character(len=15), parameter :: sub = 'def_common_vars'

    ! TEST COMMON - Pretend to put common vars - done for real later
    if(nctype == 'tech')then;
	if(DEBUG) write(DBG,'("COMMON+ ",a," spc Define PLATFORM_NUMBER")')nctype;
	PLATFORM_NUMBER_dims = STRING8_dimid;
	ret=nf_def_var(ncid,'PLATFORM_NUMBER', NF_CHAR,1, PLATFORM_NUMBER_dims,PLATFORM_NUMBER_id);          call cke(sub,ret, 9);
	ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id,'long_name', 23, 'Float unique identifier');             call cke(sub,ret,10);
	ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id,'conventions', 30, 'WMO float identifier : A9IIIII');    call cke(sub,ret,11);
	ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id,'_FillValue', 1, ' ');                                   call cke(sub,ret,12);
    endif
    if(DEBUG) write(DBG,'("COMMON  ",a," act Define DATA_TYPE")')nctype;
    if(nctype=='tech')then
	DATA_TYPE_dims(1) = STRING32_dimid;
    elseif(nctype=='biop')then
	DATA_TYPE_dims(1) = STRING32_dimid;
    else
	DATA_TYPE_dims(1) = STRING16_dimid;
    endif
    ret=nf_def_var(ncid,'DATA_TYPE', NF_CHAR,NDIMS1, DATA_TYPE_dims,DATA_TYPE_id);                         call cke(sub,ret, 5);
    ret=nf_put_att_text(ncid,DATA_TYPE_id,'long_name', 9,'Data type');                                     call cke(sub,ret, 6);

    if(nctype == 'prof' .and. profver3p0)then
	if(DEBUG) write(DBG,'("netcdf FVER nctype is prof, ver is 3.0, do NOT add DATA_TYPE:CONVENTIONS")');
    else                                                             ! FORMAT_VERSION)=3.1
	if(DEBUG) write(DBG,'("netcdf FVER nctype is NOT V3.0/prof, so DO add DATA_TYPE:CONVENTIONS")');
	ret=nf_put_att_text(ncid,DATA_TYPE_id,'conventions', 22, 'Argo reference table 1');                  call cke(sub,ret, 7);
    endif

    ret=nf_put_att_text(ncid,DATA_TYPE_id,'_FillValue',1,fillchar);
    !================================================================
    if(DEBUG) write(DBG,'("COMMON  ",a," all Define FORMAT_VERSION")')nctype;
    FORMAT_VERSION_dims(1) = STRING4_dimid;
    ret=nf_def_var(ncid,'FORMAT_VERSION', NF_CHAR,NDIMS1, FORMAT_VERSION_dims,FORMAT_VERSION_id);            call cke(sub,ret,25);
    ret=nf_put_att_text(ncid,FORMAT_VERSION_id,'long_name', 19,'File format version');                       call cke(sub,ret,26);
    ret=nf_put_att_text(ncid,FORMAT_VERSION_id,'_FillValue',1,fillchar);                                     call cke(sub,ret,27);
    !================================================================
    if(DEBUG) write(DBG,'("COMMON  ",a," all Define HANDBOOK_VERSION")')nctype;
    HANDBOOK_VERSION_dims(1) = STRING4_dimid;
    ret=nf_def_var(ncid,'HANDBOOK_VERSION', NF_CHAR,NDIMS1, HANDBOOK_VERSION_dims,HANDBOOK_VERSION_id);      call cke(sub,ret,28);
    ret=nf_put_att_text(ncid,HANDBOOK_VERSION_id,'long_name', 21,'Data handbook version');                   call cke(sub,ret,29);
    ret=nf_put_att_text(ncid,HANDBOOK_VERSION_id,'_FillValue',1,fillchar);                                   call cke(sub,ret,30);
    !================================================================
    if(nctype == 'prof' .or. nctype == 'traj' .or. nctype == "biop" .or. nctype == "biot")then;
	if(DEBUG) write(DBG,'("COMMON  ",a," spc Define REFERENCE_DATE_TIME")')nctype;
        REF_DATE_TIME_dims(1) = DATE_TIME_dimid;
        ret=nf_def_var(ncid,'REFERENCE_DATE_TIME',NF_CHAR,NDIMS1,REF_DATE_TIME_dims,REF_DATE_TIME_id);       call cke(sub,ret,36);
        ret=nf_put_att_text(ncid,REF_DATE_TIME_id,'long_name', 33,'Date of reference for Julian days');      call cke(sub,ret,37);
        ret=nf_put_att_text(ncid,REF_DATE_TIME_id,'conventions',14,'YYYYMMDDHHMISS');                        call cke(sub,ret,38);
        ret=nf_put_att_text(ncid,REF_DATE_TIME_id,'_FillValue',1,fillchar);                                  call cke(sub,ret,39);
    endif
    !================================================================
    if(nctype == 'tech')then;
	if(DEBUG) write(DBG,'("COMMON  ",a," Define DATA_CENTRE")')nctype;
        DATA_CENTRE_dims = STRING2_dimid;
        ret=nf_def_var(ncid,'DATA_CENTRE', NF_CHAR,NDIMS1, DATA_CENTRE_dims,DATA_CENTRE_id);  call cke(sub,ret,31); ! For nctech
        ret=nf_put_att_text(ncid,DATA_CENTRE_id,'long_name', 46, DC_long_name_tech);          call cke(sub,ret,33);
        ret=nf_put_att_text(ncid,DATA_CENTRE_id,'conventions', 22, 'Argo reference table 4'); call cke(sub,ret,34);
        ret=nf_put_att_text(ncid,DATA_CENTRE_id,'_FillValue', 1, ' ');                        call cke(sub,ret,35);
    endif
    !================================================================
    if(DEBUG) write(DBG,'("COMMON ",a," all Define DATE_CREATION")')nctype;
    DATE_CREATION_dims(1) = DATE_TIME_dimid;
    ret=nf_def_var(ncid,'DATE_CREATION', NF_CHAR,NDIMS1, DATE_CREATION_dims, DATE_CREATION_id);              call cke(sub,ret,40);
    ret=nf_put_att_text(ncid,DATE_CREATION_id,'long_name', 21,'Date of file creation');                      call cke(sub,ret,41);
    ret=nf_put_att_text(ncid,DATE_CREATION_id,'conventions',14,'YYYYMMDDHHMISS');                            call cke(sub,ret,42);
    ret=nf_put_att_text(ncid,DATE_CREATION_id,'_FillValue',1,fillchar);                                      call cke(sub,ret,43);
    !================================================================
    if(DEBUG) write(DBG,'("COMMON ",a," all Define DATE_UPDATE")')nctype;
    DATE_UPDATE_dims(1) = DATE_TIME_dimid;
    ret=nf_def_var(ncid,'DATE_UPDATE', NF_CHAR,NDIMS1, DATE_UPDATE_dims, DATE_UPDATE_id);                    call cke(sub,ret,44);
    ret=nf_put_att_text(ncid,DATE_UPDATE_id,'long_name', 27,'Date of update of this file');                  call cke(sub,ret,45);
    ret=nf_put_att_text(ncid,DATE_UPDATE_id,'conventions',14,'YYYYMMDDHHMISS');                              call cke(sub,ret,46);
    ret=nf_put_att_text(ncid,DATE_UPDATE_id,'_FillValue',1,fillchar);                                        call cke(sub,ret,47);
    if(DEBUG) write(DBG,'("COMMON ",a," all Define DATE_UPDATE")')nctype;
    !================================================================
    if(nctype == 'prof' .or. nctype == 'biop')then;
      if(DEBUG) write(DBG,'("COMMON+ ",a," Define PLATFORM_NUMBER/N_PROF")')nctype
      PLATFORM_NUMBER_dims_vec(2) = N_PROF_dimid;
      PLATFORM_NUMBER_dims_vec(1) = STRING8_dimid;
      ret=nf_def_var(ncid,'PLATFORM_NUMBER',NF_CHAR,NDIMS2,PLATFORM_NUMBER_dims_vec,PLATFORM_NUMBER_id_vec); call cke(sub,ret,48);
      ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id_vec,'long_name', 23, 'Float unique identifier');           call cke(sub,ret,49);
      ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id_vec,'conventions', 30, 'WMO float identifier : A9IIIII');  call cke(sub,ret,50);
      ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id_vec,'_FillValue', 1, ' ');                                 call cke(sub,ret,51);
    elseif(nctype == 'meta' .or. nctype == 'traj')then
	if(DEBUG) write(DBG,'("COMMON+ ",a," Define PLATFORM_NUMBER")')nctype
        PLATFORM_NUMBER_dims = STRING8_dimid;
        ret=nf_def_var(ncid,'PLATFORM_NUMBER', NF_CHAR,1, PLATFORM_NUMBER_dims,PLATFORM_NUMBER_id);          call cke(sub,ret, 9);
        ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id,'long_name', 23, 'Float unique identifier');             call cke(sub,ret,10);
        ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id,'conventions', 30, 'WMO float identifier : A9IIIII');    call cke(sub,ret,11);
        ret=nf_put_att_text(ncid,PLATFORM_NUMBER_id,'_FillValue', 1, ' ');                                   call cke(sub,ret,12);
    endif

end subroutine def_common_vars

subroutine put_common_data(caller,nctype,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/tech
    integer,      intent( in ) :: ncid          ! file id

    integer :: ret = 0;
    integer :: loc = 0;

    character(len=15), parameter :: sub = 'put_common_data'

    !if(nprf_start == 2)then; N_PROF= N_PROF +1; nprf = nprf + 1; endif

    if(nctype == 'prof')then
        FORMAT_VERSION = prof_FORMAT_VERSION
    elseif(nctype == 'biop')then
        FORMAT_VERSION = prof_FORMAT_VERSION
    elseif(nctype == 'meta')then
        FORMAT_VERSION = meta_FORMAT_VERSION
    elseif(nctype == 'tech')then
        FORMAT_VERSION = tech_FORMAT_VERSION
    elseif(nctype == 'traj')then
        FORMAT_VERSION = traj_FORMAT_VERSION
    elseif(nctype == 'biot')then
        FORMAT_VERSION = traj_FORMAT_VERSION
    endif

    if(nctype == 'prof' .or. nctype == 'biop')then;
       if(DBLEV==45)then; write(msg,*)'NetCDF COMMON put PLATFORM_NUMBER for ',nctype; call wrtlog(sub,msg); endif
       do i=1,N_PROF; PLATFORM_NUMBER_vec(i) = wmoid; enddo
       ret=nf_put_var_text(ncid,PLATFORM_NUMBER_id_vec, PLATFORM_NUMBER_vec);                                call cke(sub,ret,73);
    else
       PLATFORM_NUMBER = wmoid;
       ret=nf_put_var_text(ncid,PLATFORM_NUMBER_id, PLATFORM_NUMBER);                                        call cke(sub,ret,73);
    endif

    if(nctype == 'prof')then;
	DATA_TYPE16 = 'Argo profile    ';
	ret=nf_put_var_text(ncid,DATA_TYPE_id, DATA_TYPE16); 
    elseif(nctype == 'biop')then;
	DATA_TYPE32 = 'B-Argo profile                  ';
	ret=nf_put_var_text(ncid,DATA_TYPE_id, DATA_TYPE32);                                                  call cke(sub,ret,25);
    elseif(nctype == 'meta')then;
	DATA_TYPE16 = 'Argo meta-data  ';
	ret=nf_put_var_text(ncid,DATA_TYPE_id, DATA_TYPE16);
    elseif(nctype == 'traj')then;
	DATA_TYPE16 = 'Argo trajectory ';
	ret=nf_put_var_text(ncid,DATA_TYPE_id, DATA_TYPE16);
    elseif(nctype == 'tech')then;
       DATA_TYPE32 = 'Argo technical data              ';
	ret=nf_put_var_text(ncid,DATA_TYPE_id, DATA_TYPE32);                                                  call cke(sub,ret,25);
    endif

    ret=nf_put_var_text(ncid,FORMAT_VERSION_id, FORMAT_VERSION);                                             call cke(sub,ret,26);

    if(DBLEV==45)then; write(msg,*)'NetCDF COMMON put HANDBOOK_VERSION for ',nctype; call wrtlog(sub,msg); endif
    ret=nf_put_var_text(ncid,HANDBOOK_VERSION_id, HANDBOOK_VERSION);                                         call cke(sub,ret,27);


    if(nctype == 'prof' .or. nctype == 'traj' .or. nctype == "biop" .or. nctype == "biot")then;
        if(DBLEV==45)then; write(msg,*)'NetCDF COMMON put REFERENCE_DATE_TIME for ',nctype; call wrtlog(sub,msg); endif
      ret=nf_put_var_text(ncid,REF_DATE_TIME_id, REFERENCE_DATE_TIME);                                       call cke(sub,ret,28);
    endif

    if(nctype == 'prof' .or. nctype == 'biop')then;
	do i=1,N_PROF; DATA_CENTRE_vec(i) = chr_DATA_CENTRE; enddo
	ret=nf_put_var_text(ncid,DATA_CENTRE_id, DATA_CENTRE_vec);                                            call cke(sub,ret,78);
    else
	ret=nf_put_var_text(ncid,DATA_CENTRE_id,'AO');                                                        call cke(sub,ret,10);
    endif

    if(DBLEV==45)then; write(msg,*)'NetCDF COMMON put DATE_CREATION for ',nctype; call wrtlog(sub,msg); endif
    ret=nf_put_var_text(ncid,DATE_CREATION_id, file_date_time);                                              call cke(sub,ret,29);

    if(DBLEV==45)then; write(msg,*)'NetCDF COMMON put DATE_UPDATE  for ',nctype; call wrtlog(sub,msg); endif
    ! notest file_date_time = 'DUMMY_DATE_UPD'
    ret=nf_put_var_text(ncid,DATE_UPDATE_id, file_date_time);                                                call cke(sub,ret,20);

end subroutine put_common_data

 !--------------------------------------------------------------------------------------------------
 ! To scan content of meta file and to return the number of lines
 ! which contains both of the incoming literal
 !---------------------------------------------------------------------------------------------------
subroutine count_linesof_occurence_2strings(in_literal1, in_literal2, occurence_count_result )	! If the flag is ok, run the test

   character(len=25),parameter       :: sub = "cnt_lnsof_ocrnce_2st";
   character(len=*), intent( in )    :: in_literal1, in_literal2
   integer, intent(out)              :: occurence_count_result
   integer :: idx
   character (len=40) :: first40char
   occurence_count_result = 0
    
    do idx = 1,nmet
            first40char=metline(idx)(1:40)
	    if(INDEX(first40char, in_literal1)> 0 .AND. &
	    INDEX(first40char, in_literal2)> 0)then
	        occurence_count_result = occurence_count_result + 1
	    endif
    enddo
end subroutine count_linesof_occurence_2strings

 ! ===============================================================================
subroutine fill_posn_qc(caller,n_meas,zpos)

character(*), intent(in) :: caller        ! who is calling me?
integer,      intent(in) :: n_meas,zpos

 character(len=12), parameter :: sub = 'fill_posn_qc'
 
 if(zpos == 0)then
    if(caller(1:12) == "EXTRACT_TRAJ")then
	if(n_meas > 1)then              !Ignore if it is launch position n_meas=1
	    write(wrnmsg,'("WARNING 199 in fill_posn_qc zpos=ZERO, but is called by: ",a)')trim(caller); ! WRN
	    call warning(sub,199,wrnmsg); ! WARNING 199 in netcdf_arrays.f90 subroutine fill_posn_qc ! WRN
	endif
    else
	write(errmsg(1),'("NON-FATAL ERROR 254 in fill_posn_qc called by ",a)')trim(caller);
	write(errmsg(2),'("NON-FATAL ERROR 254 zpos is ZERO, blows out array prev_RTLQC2()")');
	write(errmsg(3),'("NON-FATAL ERROR 254 POSITION_QC_TRAJ(n_meas) set to zero for n_meas: ",i6)')n_meas
        write(errmsg(4),'("NON-FATAL ERROR 254 fill_posn_qc is being called by: ",a)')trim(caller);
	call error_report(sub,254,4,errmsg); ! fill_posn_qc in netcdf_arrays zpos is ZERO, blows out array
    endif
    POSITION_QC_TRAJ(n_meas) = '0';
    return
 endif  

 if(prev_lat(zpos) > 99.0)return;  ! it's a 999 line.

 if( prev_RTPQC(zpos) == "4" .or. prev_RTLQC(zpos) == "4" .or. prev_RTSQC(zpos) == "4" )then
     POSITION_QC_TRAJ(n_meas) = '4'
 else if( prev_RTPQC(zpos) == "0" .and. prev_RTLQC(zpos) == "0" .and. prev_RTSQC(zpos) ==  "0" )then
     POSITION_QC_TRAJ(n_meas) = '0'; ! For Storm Float that had problem with the Speed Check
 else if( prev_RTPQC(zpos) == "1" .and. prev_RTLQC(zpos) == "1" .and. prev_RTSQC(zpos) == "1" )then
     POSITION_QC_TRAJ(n_meas) = '1';
 else if( prev_RTPQC(zpos) == "1" .and. prev_RTLQC(zpos) == "1" .and. prev_RTSQC(zpos) ==  "0" )then !date/pos not fill, no speed test
     POSITION_QC_TRAJ(n_meas) = '0';
 else if( prev_RTPQC(zpos) == "5" .and. prev_RTLQC(zpos) == "1" .and. prev_RTSQC(zpos) ==  "1" )then !Eg. Float 5498 cycle 060
     POSITION_QC_TRAJ(n_meas) = '5';
 !Exclude-Interpolated-position in traj file, As per discussion with Claudia - 11/14/2018., commenting the line below.
 !else if( prev_RTPQC(zpos) == "8" .and. prev_RTLQC(zpos) == "1" .and. prev_RTSQC(zpos) ==  "1" )then !Eg. Float 6169 cycle 023
 !    POSITION_QC_TRAJ(n_meas) = '8';
 endif

  if(DBLEV==35)then; write(msg,*)"POSITION_QC_TRAJ = ",POSITION_QC_TRAJ(n_meas); call wrtdbg(sub,msg); endif

end subroutine fill_posn_qc

! ===============================================================================
  logical function new_check_date(iyr,imo,idy,ihh,imn,isc)

    use communication                       ! module
    use data_arrays				! module

    implicit none
    integer, intent( in ) :: iyr
    integer, intent( in ) :: imo
    integer, intent( in ) :: idy
    integer, intent( in ) :: ihh
    integer, intent( in ) :: imn
    integer, intent( in ) :: isc

    ! These are used to see if a date is in the future
    integer*8 :: loc_date_time, gmt_date_time		! local time and GMT
    integer*4 :: min_adj				! number of minutes to adjust convert local to gme
    character(len= 8) :: today_date
    character(len=10) :: today_time
    character(len= 5) :: today_zone
    integer :: today(8)
    character(len=8) :: yyyymmdd
    character(len=14) :: c14		! Used to adjust date to GMT
    integer itotst, itoday

    integer :: id, idays_mon(12) = (/31,28,31,30,31,30,31,31,30,31,30,31/)

    character(len=15), parameter   :: sub = 'TEST Arrays new_check_date';

    new_check_date = .false.               ! Assume the date is bad

    ! For the future test, convert iyr,imo,idy to char*8, then to integer
    write(yyyymmdd,'(i4,i2.2,i2.2)')iyr,imo,idy;
    read(yyyymmdd,*) itotst

    ! Check out the datd_and_time call

    call date_and_time( today_date, today_time, today_zone, today)
    write(c14,'(i4,5i2.2)')today(1),today(2),today(3),today(5),today(6),today(7); read(c14,*)loc_date_time;
    min_adj = today(4); call adjust_minutes(loc_date_time,min_adj,gmt_date_time)
    if(DBLEV==35)then; write(msg,'("GMT loc_date_time: ",i14)')loc_date_time; call wrtlog(sub,msg); endif
    if(DBLEV==35)then; write(msg,'("GMT gmt_date_time: ",i14)')gmt_date_time; call wrtlog(sub,msg); endif

    ! find the test (today's) date
    read(today_date,*)itoday
    if(itotst > itoday)then
	write(msg,'("W BAD date in the future - date: ",i8," > today: ",i8)')itotst,itoday;
	call wrttrc(sub,msg); return
    endif
    if(iyr< 1997)then;
	write(msg,'("check_cdate W BAD YEAR (<1997): ",i4)')iyr;;
	call wrttrc(sub,msg); return;
    endif
    if(imo< 1)then;;
	write(msg,'("W BAD MON (< 1): ",i2)')imo;
	call wrttrc(sub,msg); return;  
    endif
    if(imo>12)then;;
	write(msg,'("W BAD MON (>12): ",i2)')imo;;
	call wrttrc(sub,msg); return;  
    endif
    if(idy< 1)then;;
	write(msg,'("W BAD DAY (< 1): ",i2)')idy;;
	call wrttrc(sub,msg); return;  
    endif
    id=idays_mon(imo); if(imo==2)then; if(mod(iyr,4).eq.0)then; id = id+1; endif; endif ! Leap year
    if(idy>id)then;
	write(msg,'("W BAD DAY (>",i2,"): ",i2)')id,idy;
	call wrttrc(sub,msg);return;
    endif
    if(ihh< 0)then;;
	write(msg,'("W BAD HOUR (< 0): ",i2)')ihh;;
	call wrttrc(sub,msg);return;  
    endif
    if(ihh>23)then;;
	write(msg,'("W BAD HOUR (>23): ",i2)')ihh;;
	call wrttrc(sub,msg);return;  
    endif
    if(imn< 0)then;;
	write(msg,'("W BAD MIN (< 0): ",i2)')imn;;
	call wrttrc(sub,msg);return;  
    endif
    if(imn>59)then;;
	write(msg,'("W BAD MIN (>59): ",i2)')imn;;
	call wrttrc(sub,msg);return;  
    endif
    if(isc< 0)then;;
	write(msg,'("W BAD SEC (< 0): ",i2)')isc;;
	call wrttrc(sub,msg);return;  
    endif
    if(isc>59)then;;
	write(msg,'("W BAD SEC (>59): ",i2)')isc;;
	call wrttrc(sub,msg);return;  
    endif 
    new_check_date = .true.                ! If it make it here, we have a good date
  end function new_check_date
!========================================================================================
integer function create_ncfile(caller,nctype,ncfile,ncid)

    character(*), intent( in ) :: caller      ! who is calling me?
    character(*), intent( in ) :: nctype      ! prof/traj/tech
    character(*), intent( in ) :: ncfile      ! output nc file name
    integer,      intent(out ) :: ncid        ! file id

    logical :: file_exist
    character(len=256) :: commandline
    integer :: ret = 0;
    integer ::  status
    integer :: loc = 0;

    integer :: time;                        ! this function retunrs local time, timezone info
    integer :: tarray(9)                  ! array holds UTC time

    character(len=40) :: references = 'http://www.argodatamgt.org/Documentation';
    character(len=52) :: comment_on_res = 'PRES variable resolution depends on measurement code';

    integer :: occurence_count = 0;

    character(len=15), parameter :: sub = 'create_ncfile'
    character(len=4) :: nctup;

    create_ncfile = IO_OK-1; out_open = .false.;	! Assume failure

    call set_uppercase(nctype,nctup)

    ! Does the file already exist - if yes the delete it

    inquire(file=ncfile,exist=file_exist)
    if(file_exist)then
        write(commandline,'("/usr/bin/rm -f ",a)')trim(ncfile)
        call system(commandline)
        inquire(file=ncfile,exist=file_exist)
        if(file_exist)then
            write(errmsg(1),'("NON-FATAL ERROR 255 netcdf_arrays")');
            write(errmsg(2),'("NON-FATAL ERROR 255 OPENING FILE new ncfile (old file not removed) at Z")');
            write(errmsg(3),'("NON-FATAL ERROR 255 file: ",A)')trim(ncfile);
	    create_ncfile = 255; out_open = .false.;
            call error_report(sub,255,3,errmsg); ! create_ncfile in netcdf_arrays old file not removed
	
        endif
    endif
    write(msg,'("==== NC ",a," FOPEN "," about to create ncfile: ",a)')nctup,trim(ncfile); call wrtdbg(sub,msg);
    ret=nf_create(ncfile, NF_CLOBBER, ncid); 							call cke(sub,ret, 0);
    ! Did we actually create a file?
    inquire(file=ncfile,exist=file_exist)
    if(file_exist)then
	write(msg,'("==== NC ",a," FOPEN "," file created: ",a)')nctup,trim(ncfile); call wrtdbg(sub,msg);
    else
	write(msg,'("==== NC ",a," FOPEN "," ERR file NOT created: ",a)')nctup,trim(ncfile); call wrtdbg(sub,msg);
	if(DEBUG)then
	    write(errmsg(1),'("FATAL ERROR 956 DBG=T NC ",a," FOPEN "," in netcdf_arrays")')nctup
	    write(errmsg(2),'("FATAL ERROR 956 DBG=T NC FOPEN Error this nc file NOT created for profile: ",a)')trim(ncfile);
	    write(errmsg(3),'("FATAL ERROR 956 DBG=T NC FOPEN Error trying to open ncfile: ",a)')trim(ncfile);
	    call error_exit(caller,956,3,errmsg); ! DBG=T create_ncfile in netcdf_arrays - DBG=T cannot open new ncfile
	else
	    write(errmsg(1),'("NON-FATAL ERROR 256 DBG=F NC ",a," FOPEN "," in netcdf_arrays")')nctup;
	    write(errmsg(2),'("NON-FATAL ERROR 256 DBG=F create_ncfile NC FOPEN Error file NOT created: ",a)')trim(ncfile);
	    call error_report(caller,256,2,errmsg); ! DBG=F create_ncfile in netcdf_arrays old file NOT created
	    create_ncfile = IO_OK-1; out_open = .false.;  return; ! Report failure
	endif
    endif
    ! How many profiles do we have (this is INDEPENDENT of earlier nprf definitions)
    do iprf=MAXPRF,1,-1
      if(hasPRES(iprf))then; N_PROF = iprf; exit; endif;       ! exit the loop, not the program
    enddo
    if(DBLEV==37)then; write(msg,*)'In netcdf_arrays, N_PROF: ',N_PROF; call wrtdbg(sub,msg); endif

    ! How many parameters do we have?

   
    if(nctype == 'biop')then
	N_PARAM_len = nbparout ! In function create_ncfile, for nctype=biop - PRES/DOXY are already included in nvparout
    else
	if(hasPRES(0))then; nparam=1;; ncparout(nparam) = "PRES"; endif
	if(hasTEMP(0))then; nparam=nparam + 1; ncparout(nparam) = "TEMP"; endif
	if(hasCNDC(0))then; nparam=nparam + 1; ncparout(nparam) = "CNDC"; endif
	if(hasPSAL(0))then; nparam=nparam + 1; ncparout(nparam) = "PSAL"; endif
	if(outRDOXY)then;    nparam=nparam + 1; ncparout(nparam) = "DOXY"; endif
	N_PARAM_len = nparam; ! In funcfile
    endif

    ! Add the dimensions for each of the filetypes
    if(DEBUG)then;write(msg,'("NetCDF add  dimensions for nctype: ",a)')nctype;call wrtdbg(sub,msg);endif
    if(nctype == 'prof')then
      ret=nf_def_dim(ncid,'DATE_TIME', DATE_TIME_len, DATE_TIME_dimid)
      ret=nf_def_dim(ncid,'STRING256', STRING256_len, STRING256_dimid)
      ret=nf_def_dim(ncid,'STRING64',  STRING64_len,  STRING64_dimid)
      ret=nf_def_dim(ncid,'STRING32',  STRING32_len,  STRING32_dimid)
      ret=nf_def_dim(ncid,'STRING16',  STRING16_len,  STRING16_dimid)
      ret=nf_def_dim(ncid,'STRING8',   STRING8_len,   STRING8_dimid)
      ret=nf_def_dim(ncid,'STRING4',   STRING4_len,   STRING4_dimid)
      ret=nf_def_dim(ncid,'STRING2',   STRING2_len,   STRING2_dimid)
      N_PROF_len=npres; N_LEVELS_len=nlev; 				! In function create_ncfile nctype=prof
      !if(nprf_start == 2)then;N_PROF_len = N_PROF_len+1;endif		! In function create_ncfile nctype=prof
      ret=nf_def_dim(ncid,'N_PROF',    N_PROF_len,    N_PROF_dimid)
      ret=nf_def_dim(ncid,'N_PARAM',   N_PARAM_len,   N_PARAM_dimid)		! prof
      ret=nf_def_dim(ncid,'N_LEVELS',   N_LEVELS_len, N_LEVELS_dimid)
      ret=nf_def_dim(ncid,'N_CALIB',   N_CALIB_len,   N_CALIB_dimid)
      ret=nf_def_dim(ncid,'N_HISTORY', N_HISTORY_len_prof, N_HISTORY_dimid)
    elseif(nctype == 'biop')then
      ret=nf_def_dim(ncid,'DATE_TIME', DATE_TIME_len, DATE_TIME_dimid)
      ret=nf_def_dim(ncid,'STRING256', STRING256_len, STRING256_dimid)
      ret=nf_def_dim(ncid,'STRING64',  STRING64_len,  STRING64_dimid)
      ret=nf_def_dim(ncid,'STRING32',  STRING32_len,  STRING32_dimid)
      ret=nf_def_dim(ncid,'STRING16',  STRING16_len,  STRING16_dimid)
      ret=nf_def_dim(ncid,'STRING8',   STRING8_len,   STRING8_dimid)
      ret=nf_def_dim(ncid,'STRING4',   STRING4_len,   STRING4_dimid)
      ret=nf_def_dim(ncid,'STRING2',   STRING2_len,   STRING2_dimid)
      N_PROF_len=npres; N_LEVELS_len=nlev; 				! In function create_ncfile nctype=biop
      !if(nprf_start == 2)then;N_PROF_len = N_PROF_len+1;endif		! In function create_ncfile nctype=biop
      ret=nf_def_dim(ncid,'N_PROF',    N_PROF_len,    N_PROF_dimid)
      ret=nf_def_dim(ncid,'N_PARAM',   N_PARAM_len,   N_PARAM_dimid)		! biop
      ret=nf_def_dim(ncid,'N_LEVELS',   N_LEVELS_len, N_LEVELS_dimid)
      ret=nf_def_dim(ncid,'N_CALIB',   N_CALIB_len,   N_CALIB_dimid)
      ret=nf_def_dim(ncid,'N_HISTORY', N_HISTORY_len_prof, N_HISTORY_dimid)
    elseif(nctype == 'meta')then
      ret=nf_def_dim(ncid,'DATE_TIME',    DATE_TIME_len,    DATE_TIME_dimid)
      ret=nf_def_dim(ncid,'STRING1024',   STRING1024_len,   STRING1024_dimid)
      ret=nf_def_dim(ncid,'STRING256',    STRING256_len,    STRING256_dimid)
      ret=nf_def_dim(ncid,'STRING128',    STRING128_len,    STRING128_dimid)
      ret=nf_def_dim(ncid,'STRING64',     STRING64_len,     STRING64_dimid)
      ret=nf_def_dim(ncid,'STRING32',     STRING32_len,     STRING32_dimid)
      ret=nf_def_dim(ncid,'STRING16',     STRING16_len,     STRING16_dimid)
      ret=nf_def_dim(ncid,'STRING8',      STRING8_len,      STRING8_dimid)
      ret=nf_def_dim(ncid,'STRING4',      STRING4_len,      STRING4_dimid)
      ret=nf_def_dim(ncid,'STRING2',      STRING2_len,      STRING2_dimid)
      
      if(n_config_param > 0)then
	 N_CONFIG_PARAM_len = n_config_param
      else !Just in case no Config info has been found set launch config as config.
         N_CONFIG_PARAM_len = n_launch_config_param
      endif

      ! NO, LEAVE IT AT NF_UNLIMITED - N_MISSIONS_len = cnt_missions
      N_LAUNCH_CONFIG_PARAM_len = n_launch_config_param       !     >>>>>>>>>>> Calculate later -> JN
      ! Calculating Number of positioning system based on the type and positioning indicator
      if(trans_type == 'ARGOS')then
         N_POSITIONING_SYSTEM_len = 1
      else
         if(has_irid_positioning)then
            N_POSITIONING_SYSTEM_len = 2
         else
            N_POSITIONING_SYSTEM_len = 1
         endif
      endif
      N_TRANS_SYSTEM_len = 1              !     >>>>>>>>>>> Calculate later -> JN
      N_PARAM_len = nparam_meta ! So that the N_PARAM won't become 0.
      ret=nf_def_dim(ncid,'N_PARAM',   N_PARAM_len,   N_PARAM_dimid)		! meta
      ret=nf_def_dim(ncid,'N_SENSOR',     N_SENSOR_len,     N_SENSOR_dimid)
      ret=nf_def_dim(ncid,'N_CONFIG_PARAM', N_CONFIG_PARAM_len, N_CONFIG_PARAM_dimid)
      ret=nf_def_dim(ncid,'N_LAUNCH_CONFIG_PARAM', N_LAUNCH_CONFIG_PARAM_len, N_LAUNCH_CONFIG_PARAM_dimid)
      ret=nf_def_dim(ncid,'N_MISSIONS',   N_MISSIONS_len, N_MISSIONS_dimid); ! ncmeta MISSIONS_len = NF_UNLIMITED
      ret=nf_def_dim(ncid,'N_POSITIONING_SYSTEM',N_POSITIONING_SYSTEM_len, N_POSITIONING_SYSTEM_dimid)
      ret=nf_def_dim(ncid,'N_TRANS_SYSTEM',N_TRANS_SYSTEM_len, N_TRANS_SYSTEM_dimid)
    elseif(nctype == 'traj')then
      N_PARAM_len = nparam_traj ! So that the N_PARAM won't become 0.
      ! From UM3.1: N_CYCLE is the number of collected cycles performed by the float.
      N_CYCLE_len = cyc_count ! So using the count variable to find the count of cycles
      ret=nf_def_dim(ncid,'DATE_TIME', DATE_TIME_len, DATE_TIME_dimid)
      ret=nf_def_dim(ncid,'STRING64',  STRING64_len,  STRING64_dimid)
      ret=nf_def_dim(ncid,'STRING32',  STRING32_len,  STRING32_dimid)
      ret=nf_def_dim(ncid,'STRING16',  STRING16_len,  STRING16_dimid)
      ret=nf_def_dim(ncid,'STRING8',   STRING8_len,   STRING8_dimid)
      ret=nf_def_dim(ncid,'STRING4',   STRING4_len,   STRING4_dimid)
      ret=nf_def_dim(ncid,'STRING2',   STRING2_len,   STRING2_dimid)
      ret=nf_def_dim(ncid,'N_PARAM',   N_PARAM_len,   N_PARAM_dimid)		! traj
      ret=nf_def_dim(ncid,'N_CYCLE',   N_CYCLE_len,   N_CYCLE_dimid)
      ret=nf_def_dim(ncid,'N_HISTORY', N_HISTORY_len_traj, N_HISTORY_dimid)
      ret=nf_def_dim(ncid,'N_MEASUREMENT', N_MEASUREMENT_len, N_MEASUREMENT_dimid);
    elseif(nctype == 'biot')then
      N_CYCLE_len = cyc_count ! So using the count variable to find the count of cycles
      ret=nf_def_dim(ncid,'DATE_TIME', DATE_TIME_len, DATE_TIME_dimid)
      ret=nf_def_dim(ncid,'STRING64',  STRING64_len,  STRING64_dimid)
      ret=nf_def_dim(ncid,'STRING32',  STRING32_len,  STRING32_dimid)
      ret=nf_def_dim(ncid,'STRING16',  STRING16_len,  STRING16_dimid)
      ret=nf_def_dim(ncid,'STRING8',   STRING8_len,   STRING8_dimid)
      ret=nf_def_dim(ncid,'STRING4',   STRING4_len,   STRING4_dimid)
      ret=nf_def_dim(ncid,'STRING2',   STRING2_len,   STRING2_dimid)
      ret=nf_def_dim(ncid,'N_PARAM',   N_PARAM_len,   N_PARAM_dimid)		! biot
      ret=nf_def_dim(ncid,'N_CYCLE',   N_CYCLE_len,   N_CYCLE_dimid)
      ret=nf_def_dim(ncid,'N_HISTORY', N_HISTORY_len_traj, N_HISTORY_dimid)
      ret=nf_def_dim(ncid,'N_MEASUREMENT', N_MEASUREMENT_len, N_MEASUREMENT_dimid);
    elseif(nctype == 'tech')then
      ret=nf_def_dim(ncid,'DATE_TIME',    DATE_TIME_len,    DATE_TIME_dimid)
      ret=nf_def_dim(ncid,'STRING128',    STRING128_len,    STRING128_dimid)
      ret=nf_def_dim(ncid,'STRING32',     STRING32_len,     STRING32_dimid)
      ret=nf_def_dim(ncid,'STRING8',      STRING8_len,      STRING8_dimid)
      ret=nf_def_dim(ncid,'STRING4',      STRING4_len,      STRING4_dimid)
      ret=nf_def_dim(ncid,'STRING2',      STRING2_len,      STRING2_dimid)
      ret=nf_def_dim(ncid,'N_TECH_PARAM', N_TECH_PARAM_len, N_TECH_PARAM_dimid)
    endif

    !
    !  --- Get today's date for history and DATE_CREATION/UPDATE.
    ! DO NOT USE THIS, USE TIME() AS ARGUMENT IN GMTIMT call date_and_time(date8,time10,zone5,itime8)

    call gmtime( time(), tarray )      ! report current time in UTC

    ! From the man page for gmtime
    ! After January 19, 2038, at 3:14:07 GMT, the time() value  of
    ! seconds  since  January  1,  1970  will  exceed the range of
    ! INTEGER*4. To calculate such dates with these routines,  use
     ! the 64-bit versions and an INTEGER*8 argument.

    iyr = tarray(6) + 1900;            ! 6 Year - 1900
    imm = tarray(5) + 1;            ! 5 Months since January (0 - 11)
    idy = tarray(4);                  ! 4 Day of month (1 - 31)
    ihr = tarray(3);                  ! 2 3 Hours (0 - 23)
    imn = tarray(2);                  ! 2 Minutes (0 - 59)
    isc = tarray(1);                  ! 1 Seconds (0 - 61)

    ! in users man: 2011-04-22T06:00:00Z creation
    if(DBLEV==37)then; ihr=0; imn=0;isc=0; write(msg,'("LEV37 set ihr,imn,isc=0")'); call wrtdbg(sub,msg); endif
    write(hist_date_time,'(i4.4,"-",i2.2,"-",i2.2,"T",i2.2,":",i2.2,":",i2.2,"Z creation")') &
      iyr, imm, idy, ihr, imn, isc
    if(DEBUG) hist_date_time = 'YYYY-MM-DDTHH:MM:SSZ creation'
    write(file_date_time,'(i4,5i2.2)')iyr, imm, idy, ihr, imn, isc

    if(DEBUG .and. DBLEV>0) file_date_time = 'YYYYMMDDHHMMSS';	! for nc file comparison purposes 
    if(DEBUG)then; write(msg,*)'hist_date_time: ',hist_date_time; call wrtdbg(sub,msg); endif
    if(DEBUG)then; write(msg,*)'file_date_time: ',file_date_time; call wrtdbg(sub,msg); endif

    ! Finally, add the global attributes

    if(nctype == 'prof')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'title', 27,'Argo float vertical profile');         call cke(sub,ret, 1); ! prof
    elseif(nctype == 'biop')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'title', 27,'Argo float vertical profile');         call cke(sub,ret, 1); ! biop
    elseif(nctype == 'meta')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'title', 24,'Argo float metadata file');            call cke(sub,ret, 1); ! meta
    elseif(nctype == 'traj')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'title', 26,'Argo float trajectory file');          call cke(sub,ret, 1); ! traj
    elseif(nctype == 'biot')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'title', 30,'Bio-Argo float trajectory file');      call cke(sub,ret, 1); ! biot
    elseif(nctype == 'tech')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'title', 30,'Argo float technical data file');      call cke(sub,ret, 1); ! tech
    else
      write(errmsg,'("This should not have happened, close the nc file and return")')
    endif
    ret=nf_put_att_text(ncid,NF_GLOBAL, 'institution',          4,'AOML');                      call cke(sub,ret, 2); ! all
    ret=nf_put_att_text(ncid,NF_GLOBAL, 'source',              10,'Argo float');                call cke(sub,ret, 3); ! all
    ret=nf_put_att_text(ncid,NF_GLOBAL, 'history',             29,hist_date_time);              call cke(sub,ret, 4); ! all
    ret=nf_put_att_text(ncid,NF_GLOBAL, 'references',          40,references);                  call cke(sub,ret, 5); ! all
    ret=nf_put_att_text(ncid,NF_GLOBAL, 'comment',              9,'free text');                 call cke(sub,ret, 6); ! all
    if(nctype == 'prof' .or. nctype == 'biop' )then;
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'user_manual_version',lumv,prof_USER_MANUAL);       call cke(sub,ret, 7); ! prf/bio
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'Conventions',         15,prof_global_conventions); call cke(sub,ret, 8); ! prf/bio
    else
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'user_manual_version',  3,'3.1');                   call cke(sub,ret, 7); ! mttf
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'Conventions',         15,'Argo-3.1 CF-1.6');       call cke(sub,ret, 8); ! mttf
    endif
    if(nctype == 'prof' .or. nctype == 'biop')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'featureType',         17,'trajectoryProfile');     call cke(sub,ret, 9); ! orf/bio
    elseif(nctype == 'traj')then
	ret=nf_put_att_text(ncid,NF_GLOBAL, 'featureType',         10,'trajectory');            call cke(sub,ret, 9); ! traj
    else
      write(msg,*)'netcdf_arrays - global atts - This may be an ERROR, but tech has not featureType attirbute'
      call wrtlog(sub,msg);
    endif
    ! FIX agree w traj if(nctype == 'traj' .and. hasMC)then
	! ret=nf_put_att_text(ncid,NF_GLOBAL, 'comment_on_resolution', 52, comment_on_res);                    call cke(sub,ret,10);
    ! FIX agree w traj endif

! I guress it all worked, got not complaints

  create_ncfile = IO_OK; out_open = .true.;

end function create_ncfile 
!========================================================================================
end module netcdf_arrays
!========================================================================================
