! ===========================================================================================================
! integer function qcsub/qc08_pres_increase.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc08_pres_increase(caller,infop)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: infop(3)

character(len=4), parameter :: sub = 'QC08'
character(len=21) ,parameter :: QCTEST='Pressure Increasing'
integer :: result;

real, parameter :: filter_tolerance  =  20.0;						! for Spike check
real, parameter :: filter_tolerance1 = 102.0; integer, parameter :: filter_width1 =  3; ! for median filter 1
real, parameter :: filter_tolerance2 = 120.0; integer, parameter :: filter_width2 =  7; ! for median filter 2
real, parameter :: filter_tolerance3 = 200.0; integer, parameter :: filter_width3 = 11; ! for median filter 3

integer :: ifl, itype, beg, end,k,kx,d_aft,d_bef,ilev_tmp; 
real :: rd_aft, rd_bef
integer :: proftot_tmp,ngood,nbad,nskip
integer :: j

integer :: bottom		! FIX this workaround for bottom of the secondary profile

real :: profdata_tmp2(MAXLEV),profdata_tmp1(MAXLEV),profdata_tmp(MAXLEV),profdata_bad(MAXLEV);

logical :: orphan_pres_found = .false.		! Were any orphan pres levels found
real    :: value1, value2  			! Values upon which to base the conditions

integer :: ilev_vector_tmp(MAXLEV);
integer :: tlev

! ===========================================================================================================
! executable statements

ifl = pres_incr_ncol; 

result = NOTD ! cannot be determined

itype=infop(1); beg=infop(2);  end=infop(3); ! run characteristics

if(DBGIO) call dbg_enter_type(sub,parmname(itype))

if(nlev==0)then;
    write(msg,'(a," W nlev=0, no test performed")')sub;
    call wrttrc(sub,msg);
    go to 99;
endif

do idat=beg,end,2											! idat=beg,end
    if(hasDATA(idat))then									! if hasDATA
        if(proftot(idat) > 0)then								! if ntot>0
	    ! FIX this workaround - find the bottom of the profile
	    bottom = 0;
	    do ilev = nlev,1,-1
		if(profflag(idat,ilev) /= 9)then
		    bottom = ilev;
		    exit;			! exit the loop, not the sub
		endif
	    enddo
	    if(DBLEV==30)then; write(msg,*)'QC08 bottom: ',bottom,' nlev: ',nlev; call wrtdbg(sub,msg); endif
	    ngood = 0;nbad = 0;nskip=0;
            ! First, filter out pres already identified as bad (create temporary pressure vector)
            call  fill_profvect_tmp(proftot_tmp,profdata_tmp,ilev_vector_tmp,ifl)
    	    if(proftot_tmp .eq. 0 .or. proftot_tmp .eq. 1)then
		if(proftot_tmp .eq. 0)then;
		    write(msg,'("QC08 There are NO good levs")');
	        else
		    write(msg,'("QC08 There is only ONE goo lev")');
		endif
		write(msg,'(a,", mark all levs as NOQC.")')trim(msg);
		call wrttrc(sub,msg);
	        do ilev = 1,nlev
	            testflag(idat,ilev,ifl) = NOQC; 
  	        enddo
		go to 11
	    elseif(proftot_tmp .eq. 2)then
	        if(profdata_tmp(2) .le. profdata_tmp(1))then
	            testflag(idat,1,ifl) = BAD;nbad = nbad + 1;ngood=ngood-1;
	            testflag(idat,2,ifl) = BAD;nbad = nbad + 1;ngood=ngood-1;
	        endif
	     	go to 11
	    endif
	    ! Next, eliminate any orphan pres from the filtered array
	    ! 17Oct2014 Added
	    !
	    ! For pres EXACTLY zero, test these:
	    !	1. pres at level i must be EXACTLY zero
	    !	2. abs(pres(i) - pres(i-1)) > 1.9
	    !	3. abs(pres(i) - pres(i+1)) > 1.9
	    !
	    ! mark level i bad if al lthree are met
	    !
	    do ilev = 2,(proftot_tmp-1)
		if(profdata_tmp(ilev) == 0.0)then
		    value1 = profdata_tmp(ilev) - profdata_tmp(ilev-1)
		    if(abs(value1) > 1.9)then;
			value2 = profdata_tmp(ilev) - profdata_tmp(ilev+1)
			if(abs(value2) > 1.9)then
			    value2 = abs(profdata_tmp(ilev) - profdata_tmp(ilev+1))
			    if(abs(value2) > 1.9)then
				orphan_pres_found = .true.;
				profflag(idat,ilev_vector_tmp(ilev))     = BAD;
				testflag(idat,ilev_vector_tmp(ilev),ifl) = BAD;
			    endif
			endif
		    endif
		endif
	    enddo

	    !
	    ! If any orphan pres levels were found, re-filter to eliminate tem
	    if(orphan_pres_found)then
		call  fill_profvect_tmp(proftot_tmp,profdata_tmp,ilev_vector_tmp,ifl)
	    endif

            !! 
            if (proftot_tmp .lt. filter_width1)  go to 11
	    if(DBLEV== 8)then;
		write(msg,'("QC08 LEV08 Call median filter with width/tolerence: ",i3,f8.1)')filter_width1,filter_tolerance1
		call wrtdbg(sub,msg);
	    endif
            call median_check(proftot_tmp,profdata_tmp,ilev_vector_tmp,filter_width1,filter_tolerance1,ngood)
		
            call  fill_profvect_tmp(proftot_tmp,profdata_tmp,ilev_vector_tmp,ifl)
	    
       !! Spike Up and Spike Down Cases
            do k = 1,2
	        kx=1;
		profdata_tmp2(kx) = profdata_tmp(1);
		do ilev = 2,proftot_tmp-1
		    d_bef = profdata_tmp(ilev)-profdata_tmp(ilev-1);
		    d_aft = profdata_tmp(ilev+1)-profdata_tmp(ilev);
		    
		   ilev_tmp=ilev_vector_tmp(ilev)
		    
		    if(d_bef.le.0.0.and.abs(d_aft).gt.filter_tolerance)then
			profflag(idat,ilev_tmp)     = BAD;
		        testflag(idat,ilev_tmp,ifl) = BAD;nbad = nbad + 1;ngood=ngood-1;
		    endif
		    
		    if(skip_test(profflag(idat,ilev_tmp)))then
			if(DBLEV<0) write(msg,'("empty option - deeps code from crashing")')
		    elseif(rd_aft.le.0.0.and.abs(rd_bef).gt.filter_tolerance)then
		        testflag(idat,ilev_tmp,ifl) = BAD;nbad = nbad + 1;ngood=ngood-1;
		    endif

	        enddo ! ilev = 2,proftot_tmp-1
                call  fill_profvect_tmp(proftot_tmp,profdata_tmp,ilev_vector_tmp,ifl)

            enddo ! k = 1,2
            if (proftot_tmp .lt. filter_width2)  goto 11 
	    call median_check(proftot_tmp,profdata_tmp,ilev_vector_tmp,filter_width2,filter_tolerance2,ngood)
            call fill_profvect_tmp(proftot_tmp,profdata_tmp,ilev_vector_tmp,ifl)
	    if (proftot_tmp .lt. filter_width3) goto 11
	    call median_check(proftot_tmp,profdata_tmp,ilev_vector_tmp,filter_width3,filter_tolerance3,ngood)
            call fill_profvect_tmp(proftot_tmp,profdata_tmp,ilev_vector_tmp,ifl)
        endif											! if proftot(idat)>0
        11 continue
	! 16Oct2014 - moved the actual monotoinically increasing test to the END
        do ilev=1,bottom-1
            if (ilev<bottom)then
                if(skip_test(profflag(idat,ilev)))then
		    if(DBLEV<0) write(msg,'("empty option - deeps code from crashing")')
                else
                    tlev = ilev;
                    5 continue              ! RE-CHECK for non-BAD pres
                    tlev = tlev + 1;
                    if(tlev > bottom)then;                          ! No more good values to check against
                        write(msg,'("qc08 Lcurrent lev last good data: ",i6)')ilev; call wrtdbg(sub,msg)
                        exit;                                       ! exit the loop, not the program
                    endif
                    if(skip_test(profflag(idat,tlev)))then
			if(DBLEV==30)then; 
		    	    write(msg,'("QC08 MONO/INC xx1 pres: ",f8.1," ilev: ",i5," tlev: ",i5," already BAD, try next lev")') &
				profdata(idat,ilev),ilev,tlev; call wrtdbg(sub,msg);
			endif
                        go to 5;                                    ! RE-CHECK f/or non-BAD pres
                    endif
                    if (profdata(idat,ilev) >= profdata(idat,tlev))then
                        testflag(idat,ilev,ifl) = BAD; nbad = nbad + 1;
			if(DBLEV==30)then;
			    write(msg,'("QC08 MONO/INC xx3 pres: ",f8.1," ilev: ",i5," tlev: ",i5," set to prob_bad")') &
				profdata(idat,ilev),ilev,tlev; call wrtdbg(sub,msg);
			endif			
                    else
                        testflag(idat,ilev,ifl) = GOOD; ngood = ngood + 1;  ! Al is ok at this level
			if(DBLEV==30)then;
			    write(msg,'("QC08 MONO/INC xx4 pres: ",f8.1," ilev: ",i5," tlev: ",i5," is GOOD")') &
				profdata(idat,ilev),ilev,tlev; call wrtdbg(sub,msg);
			endif
                    endif
                endif
            endif
        enddo
	call evaluate_flags(sub,QCTEST,datname(idat),idat,ifl,beg,result); maxresult=max(maxresult,result);
	! If we do pres adj, copy the flags from the regular column to the adjusted column
	if(do_pres_adj)then; do ilev=1,nlev; testflag(idat+1,ilev,ifl) = testflag(idat,ilev,ifl); enddo; endif
    endif
enddo		! idat

99 continue	! SKIP TO THE END

! The result returned is the max flag of those evaluated
qc08_pres_increase = maxresult;
if(DBGIO) call dbg_leave_type(sub,parmname(itype),result);

return
end function qc08_pres_increase
! ===========================================================================================================
