! ===========================================================================================================
! integer function qc14_density_inversion.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180619 VH     Eliminate duplicate def of sigma_stp (vargo)
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Included:
!
! double precision function ptem(p_bar,t,s,pr)
! double precision function atg(p,t,s)
! double precision function sigma_stp(p_bar,t,s)
! ===========================================================================================================
! Notes
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc14_density_inversion(caller,pdat,tdat,sdat)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: pdat
integer,      intent( in ) :: tdat
integer,      intent( in ) :: sdat

integer :: ifl
integer :: j	! used when DBLEV==14
character(len=16), parameter :: sub = 'QC14 with P/T/S '
character(len=21) ,parameter :: QCTEST='Density Inversion'

character(len=8) :: ch8
integer :: result,proftot_tmp

double precision, parameter :: t90to68 = 1.00024
double precision sigma_stp
double precision ptem
double precision bpres(MAXLEV)
double precision bpref(MAXLEV)                             ! pres & reference pressure in bars
double precision  pref(MAXLEV)                            
double precision, parameter :: threshold = 0.03            ! bad if density gradient is larger than this
double precision temp68(MAXLEV);			   ! for t90tot68
double precision thetai68(MAXLEV),thetap68(MAXLEV)         ! theta68
double precision thetai(MAXLEV),thetap(MAXLEV)             ! potential temp for p(i),p(i+1)
double precision sigmai(MAXLEV), sigmap(MAXLEV)            ! sigma-theta, arrays for p(i),p(i+1)
double precision delta(MAXLEV)                             ! pot. density gradient between i & i+1

integer :: ngood,nbad,nskip; 
real :: profdata_pres(MAXLEV),profdata_temp(MAXLEV),profdata_psal(MAXLEV)
real :: dfv=DBLE(FILL)
integer :: ilev_vector_tmp(MAXLEV);

! ===========================================================================================================
! executable statements

ifl = dengrad_ncol; 

result = NOTD ! cannot be determined

if(DBLEV==14)then; write(msg,'("QC14 pdat,tdat,sdat,ifl: ",4i4)')pdat,tdat,sdat,ifl; call wrtdbg(sub,msg); endif

if(DBGIO) call dbg_enter(sub)

if(proftot(pdat) == 0 .or. proftot(tdat) == 0 .or. proftot(sdat) == 0)then
    write(msg,'("PRES or TEMP or PSAL) have no good data: ",3i5)')proftot(pdat),proftot(tdat),proftot(sdat)
    go to 9999;
endif

if(DBLEV==14)then
    if(pdat==p1r)then; ch8='QC14 p1r'; elseif(pdat==p2r)then; ch8='QC14 p2r'; else; ch8='QC14 ???'; endif
    write(msg,'(a," RAW AAA pres/temp/psal BEFORE ANY filtering")')ch8; call wrtdbg(sub,msg);
    do ilev=1,nlev
	write(msg,'(a," RAW AAA ilev: ",i5,5x," data: ",f9.2,2f9.3," flags: ",2x 3i2,i4)')ch8,ilev, &
	    profdata(pdat,ilev),profdata(tdat,ilev),profdata(sdat,ilev), &
	    profflag(pdat,ilev),profflag(tdat,ilev),profflag(sdat,ilev),testflag(pdat,ilev,ifl)
	call wrtdbg(sub,msg);
    enddo
endif

call fill_profvect_tmp_3param(proftot_tmp,profdata_pres,profdata_temp,profdata_psal,pdat,tdat,sdat,ilev_vector_tmp)

if(DBLEV==14)then
    write(msg,'(a," FIL AAA Filtered pres array - BEFORE ANY processing")')ch8; call wrttrc(sub,msg);
    if(proftot_tmp == nlev)then;
	write(msg,'(a," FIL AAA No points filtered out")')ch8; call wrtdbg(sub,msg);
    elseif(proftot_tmp == 0)then;
	write(msg,'(a," FIL AAA ALL points filtered out, test not performed")')ch8; call wrtdbg(sub,msg);
    else
	do ilev=1,proftot_tmp
	    write(msg,'(a," FIL AAA ilev: ",i5,5x," data: ",f9.2,2f9.3)')ch8,ilev, &
		profdata_pres(ilev),profdata_temp(ilev),profdata_psal(ilev)
	    call wrtdbg(sub,msg);
	enddo
    endif
endif

if(proftot_tmp < 3)then
    write(msg,'("profvect filter (3p) finds only ",i2," good point(s)")')proftot_tmp; call wrtlog(sub,msg);
    return
endif

if(DBLEV==14) write(msg,'(a," FIL BBB Filtered pres array after temp68 & dbars to bars")')ch8; call wrtdbg(sub,msg)
do i= 1,proftot_tmp
    temp68(i) = profdata_temp(i)*t90to68                               ! convert t90 to t66
    bpres(i)=profdata_pres(i) / 10.0 ! decibars to bars
    if(DBLEV==14)then;
	write(msg,'(a," BBB ilev: ",i5," pres/temp68: ",2f12.3)')ch8,ilev,bpres(i),temp68(i); call wrtdbg(sub,msg);
    endif
enddo

do i=1,proftot_tmp-1
    pref(i) = (profdata_pres(i) + profdata_pres(i+1))/2.0   ! ref pres in db
    bpref(i) = pref(i) / 10.0            ! ref pres in bar
enddo
pref(proftot_tmp)=FILL
bpref(proftot_tmp)=FILL

do i=1,proftot_tmp-1
    thetai68(i) = ptem(bpres(i),temp68(i),dble(profdata_psal(i)),bpref(i))       ! potential temp at i
    thetap68(i) = ptem(bpres(i+1),temp68(i+1),dble(profdata_psal(i+1)),bpref(i)) ! potential temp at i+1
    if(DBLEV==14)then;
	if(DBLEV==30)then; write(msg,'("QC14 LEV30 Loop 2 i,t90to68: ",i5,3x,f12.5)')i,t90to68; call wrtdbg(sub,msg); endif
	thetai(i) = thetai68(i)/t90to68
	thetap(i) = thetap68(i)/t90to68
	write(msg,2002) i,bpres(i),temp68(i),profdata_psal(i),bpref(i),thetai68(i),thetai(i),thetap68(i),thetap(i);
	call wrtdbg(sub,msg)
    endif
enddo
  2002 format(i5,f18.4,f18.4,f18.4,f18.4,f18.4,f18.4,f18.4,f18.4)

do i=1,proftot_tmp-1
    sigmai(i) = sigma_stp(bpref(i),thetai68(i),dble(profdata_psal(i)))        ! potential density at i
    sigmap(i) = sigma_stp(bpref(i),thetap68(i),dble(profdata_psal(i+1)))      ! potential density at i+1
    delta(i) = sigmai(i)-sigmap(i)
    if(delta(i) > threshold)then
        testflag(sdat,ilev_vector_tmp(i),ifl)=4
        testflag(sdat,ilev_vector_tmp(i+1),ifl)=4
    else
         testflag(sdat,ilev_vector_tmp(i+1),ifl)=1
    endif
enddo
!!!!

call evaluate_2v_flags(sub,QCTEST,datname(sdat),sdat,ifl,tdat,s1r,s4r,14,result)       ! evaluate the flags for this test

if(DBLEV==14)then
    msg(1:50)= ' lev flag       pres        temp        psal       '
    msg(51:127) ='  pref       delta     sigma-i   sigma-i+1     theta-i   theta-i+1    999.001'
    call wrtdbg(sub,msg)
    do i=1,proftot_tmp-1
         if(DBLEV==14)then
              write(msg,1001) i,testflag(sdat,i,ifl),profdata_pres(i),profdata_temp(i),profdata_psal(i), &
                  pref(i),delta(i),sigmai(i),sigmap(i),thetai(i),thetap(i)
              call wrtdbg(sub,msg)
        endif
        call wrtdbg(sub,msg)
        
    enddo
    write(msg,1002) proftot_tmp,testflag(sdat,proftot_tmp,ifl),profdata_pres(proftot_tmp), &
          profdata_temp(proftot_tmp),profdata_psal(proftot_tmp)
    call wrtdbg(sub,msg)
endif
1001 format(i4,i4,f12.3,f12.4,f12.4,f12.4,f12.7,f12.7,f12.7,f12.7,f12.7,'  -9999.001')
1002 format(i4,i4,f12.3,f12.4,f12.4,'   -9999.000   -9999.000   -9999.000   -9999.000   -9999.000   -9999.000  -9999.001')

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile

9999 continue		! Skip to the end

if(DBGIO) call dbg_leave(sub,result);

qc14_density_inversion = result;

iprf = REGPRFCORE(pdat); ! Is this a regular CORE ARGO profile?
if(iprf > 0)then; QCTEST_FLAG(iprf,14) = max(QCTEST_FLAG(iprf,14),result); endif

return
end function qc14_density_inversion

! ===========================================================================================================
! double precision function ptem(p_bar,t,s,pr)
! ===========================================================================================================
! Notes
!*
! function [theta] = ptem(p_bar,t,s,pr)
!
! COMPUTATION OF LOCAL POTENTIAL TEMPERATURE AT
! REFERENCE PRESSURE PR USING BRYDEN 1973
! POLYNOMIAL FOR ADIABATIC LAPSE RATE AND
! RUNGE-KUTTA FOURTH ORDER INTEGRATION ALGORITHM
!
! p_bar  pressure in bar
! t      temeprature in deg. C
! s      salinity in psu
! pr     reference pressure in bar
!
! References:   using atg80 procedure for adiabadic lapse rate
!               Fofonoff,N.,Deep-Sea Res.,24,489-491
!
! translated from matlab of Schmid to C
! by Xiangdong Xia(Jan 2005)
! adapted from ptem.c by VMH (Dec 2012)
!/
! ===========================================================================================================
double precision function ptem(p_bar,t,s,pr)

    double precision p_bar,t,s,pr,f1,f2,f3
    double precision h1,h2,xk,theta,q,p
    double precision sqrt2
    double precision atg
    double precision dbar,dbref ! TEST output in decibars

    sqrt2 = DBLE(sqrt(2.0))

    h1=pr-p_bar                 ! del_P
    h2=atg(p_bar,t,s)           !
    xk=h1*h2                    ! del_th
    theta=t+0.5*xk              ! th
    q=xk                        ! q
    p=p_bar+0.5*h1
    h2=atg(p,theta,s)
    xk=h1*h2
    f1=1-1/sqrt2 ! must be double sqrt(2)
    f2=2-sqrt2 ! must be double sqrt(2)
    f3=-2+3/sqrt2 ! must be double sqrt(2)
    theta=theta+f1*(xk-q)
    q=f2*xk+f3*q
    h2=atg(p,theta,s)
    xk=h1*h2
    f1=1+1/sqrt2 ! must be double sqrt(2)
    f2=2+sqrt2 ! must be double sqrt(2)
    f3=-2-3/sqrt2 ! must be double sqrt(2);
    theta=theta+f1*(xk-q)
    q=f2*xk+f3*q
    p=p+0.5*h1
    h2=atg(p,theta,s)
    xk=h1*h2
    theta=theta+(xk-2.0*q)/6.0
    ptem=theta

end function ptem
! ===========================================================================================================
! double precision function atg(p,t,s)
! ===========================================================================================================
! Notes

!*
!      VAX 11/750      1983      J.HOLTORFF
!
! p      pressure in dbar
! t      temeprature in deg. C
! s      salinity in psu
!
! translated to c from Schmid's matlab code
! by Xiangdong Xia(Jan 2005)
! adapted from atg.c by VMH (Dec 2012)
!/
! ===========================================================================================================

double precision function atg(p,t,s)

   double precision p,t,s
   double precision ds,h1,h2,h3,h4,h5
   ds=s-35.0
   h1 = ((-2.1687e-13*t + 1.8676e-11)*t - 4.6206e-10)*p
   h2 = (2.7759e-10*t - 1.1351e-8)*ds
   h3 = ((-5.4481e-12*t + 8.733e-10)*t - 6.7795e-8)*t
   h4 = (-4.2393e-7*t + 1.8932e-5)*ds
   h5 = ((6.6228e-9*t - 6.836e-7)*t + 8.5258e-5)*t + 3.5803e-4
   atg=(h1 + (h2 + h3 + 1.8741e-6))*p + h4 + h5

end function atg
! ===========================================================================================================
! double precision function sigma_stp(p_bar,t,s)
! ===========================================================================================================
! Notes
! Calculates sigma(S,T,p) for CTD-profilen,
!
! function sigma_stp=sigma_stp(p_bar,t,s)
!
! p_bar  pressure in bar
! t      temeprature in deg. C
! s      salinity in psu
!
! translated from claudia's matlab to C
! adapted from ieos802.c by VMH (Dec 2012)
!/
! ===========================================================================================================

double precision function sigma_stp(p_bar,t,s)

use communication                       ! module

    implicit none

    double precision p_bar,t,s
    double precision c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15
    double precision d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13,d14,d15
    double precision d16,d17,d18,d19,d20,d21,d22,d23,d24,d25,d26
    double precision t2,t3,t4,t5,s2,s3,sqrts3,p2,p3,rho_st,sigma_st,rk,h1,h2,alpha_stp,rho_stp
    double precision , parameter :: dfv = -999.0

    c1=999.842594
    c2=6.793952e-2
    c3=-9.095290e-3
    c4=1.001685e-4
    c5=-1.120083e-6
    c6=6.536332e-9
    c7=8.24493e-1
    c8=-4.0899e-3
    c9=7.6438e-5
    c10=-8.2467e-7
    c11=5.3875e-9
    c12=-5.72466e-3
    c13=1.0227e-4
    c14=-1.6546e-6
    c15=4.8314e-4

    d1=19652.21
    d2=148.4206
    d3=-2.327105
    d4=1.360477e-2
    d5=-5.155288e-5
    d6=3.239908
    d7=1.43713e-3
    d8=1.16092e-4
    d9=-5.77905e-7
    d10=8.50935e-5
    d11=-6.12293e-6
    d12=5.2787e-8
    d13=54.6746
    d14=-0.603459
    d15=1.09987e-2
    d16=-6.167e-5
    d17=7.944e-2
    d18=1.6483e-2
    d19=-5.3009e-4
    d20=2.2838e-3
    d21=-1.0981e-5
    d22=-1.6078e-6
    d23=1.91075e-4
    d24=-9.9348e-7
    d25=2.0816e-8
    d26=9.1697e-10

    t2 = t * t
    t3 = t2 * t
    t4 = t3 * t
    t5 = t4 * t
    s2 = s * s
    s3 = s2 * s
    sqrts3 = sqrt(s3)
    p2 = p_bar * p_bar
    p3 = p2 * p_bar

  !!     derive RHO(S,T,0)

    rho_st = c1 + c2 * t + c3 * t2 + c4 * t3 + c5 * t4 + c6 * t5
    rho_st = rho_st + c7 * s + c8 * t * s + c9 * t2 * s + c10 * t3 * s
    rho_st = rho_st + c11 * t4 * s + c12 * sqrts3 + c13 * sqrts3 * t
    rho_st = rho_st + c14 * sqrts3 * t2 + c15 * s2
    sigma_st = rho_st - 1000

  !!     derive K(S,T,p)

    rk = d1 + d2 * t + d3 * t2 + d4 * t3 + d5 * t4 + d6 * p_bar
    rk = rk + d7 * t * p_bar + d8 * t2 * p_bar + d9 * t3 * p_bar
    rk = rk + d10 * p2 + d11 * t * p2 + d12 * t2 * p2 + d13 * s
    rk = rk + d14 * t * s + d15 * t2 * s + d16 * t3 * s + d17 * sqrts3
    rk = rk + d18 * t * sqrts3 + d19 * t2 * sqrts3 + d20 * p_bar * s
    rk = rk + d21 * t * p_bar * s + d22 * t2 * p_bar * s
    rk = rk + d23 * p_bar * sqrts3 + d24 * p2 * s + d25 * t * p2 * s
    rk = rk + d26 * t2 * p2 * s

  !!     derive RHO(S,T,p)
    h1 = (1 - p_bar / rk)
    h2 = 1 / rho_st
    alpha_stp = h2 * h1
    h1 = (1 - p_bar / rk)
    rho_stp = rho_st / (1 - p_bar / rk)
    sigma_stp = rho_stp - 1000
    !if(DBLEV==14)then; write(msg,2015) p_bar,rk,sigma_stp; call wrtdbg(sub,msg); endif
2015 format('in sigma_stp p_bar,rk,sigma_stp: ',f18.4,f18.4,f18.4)

end function sigma_stp

! ===========================================================================================================

