! ===========================================================================================================
! integer function qc15_grey_list.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180619 VH     Set is_<>_greylised, to be used in the frozen_profile test
! 20180515 VH     Set <param>_is_greylisted if the parameer is on the greylist
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc15_grey_list(caller,pinfo,tinfo,sinfo,oinfo)

use communication		       ! module
use data_arrays			 ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: pinfo(3)
integer,      intent( in ) :: tinfo(3)
integer,      intent( in ) :: sinfo(3)
integer,      intent( in ) :: oinfo(3)

character(len=16), parameter :: sub = 'QC15 with P?T/S '
character(len=21) ,parameter :: QCTEST='Grey List'
integer :: result;

integer :: ifl, itype, beg, end;
integer :: test_year,test_mon,test_day
integer :: grey_year,grey_mon,grey_day
logical :: HIT
integer :: mxtflag; ! Maximum local test flag
! ===========================================================================================================
! executable statements

result = NOTD ! cannot be determined
mxtflag = 0;  ! cannot be determined

if(DBGIO) call dbg_enter(sub)

!
! First, see if we have any grey list records in the meta file
!
if(ngl == 0)then
    write(msg,'("There are no grey list entries for this float")'); call wrtlog(caller,msg);
    mxtflag = 1; ! no grey list entries = good profile
    result = PASS;			!This just signals success - ie, not failure
    go to 99; ! Skip to THE_END
endif
    
! Next, see if we are in a grey list time period
!
! First, get the profile time - use the later of the prof_JULD or time time of hteprof_JULD_LOCATION
test_year = JULD_year; if(JLOC_year > JULD_year) test_year = JLOC_year;
test_mon  = JULD_mon; if(JLOC_mon > JULD_mon) test_mon = JLOC_mon;
test_day  = JULD_day; if(JLOC_day > JULD_day) test_day = JLOC_day;
if(DBLEV==15)then;write(msg,1000)thiscyc,test_year,test_mon,test_day; call wrtdbg(sub,msg); endif
1000 format("QC15 profile cycle: ",i4," year/mon/day: ",i5,i3.2,i3.2);
1001 format("QC15 profile   igl: ",i4," year/mon/day: ",i5,i3.2,i3.2);

!!!!!!!!!!!!!!!!!!!!!!!!! note 4sp offset !!!!!!!!!!!!!!!!!!!!!!!!!
tgl = 0;
do igl=ngl,1,-1
    read(grey_list_beg(igl),'(i4,x,i2,x,i2)') grey_year,grey_mon,grey_day
    if(DBLEV==15)then;write(msg,1001)igl,grey_year,grey_mon,grey_day; endif;
    if(test_year .gt. grey_year)then
	if(DBLEV==15)then;write(msg,'(a,"QC15 HIT @1")')trim(msg); call wrtdbg(sub,msg); endif
	HIT = .true.; tgl = igl; go to 10; ! found a GL HIT
    elseif(test_year .eq. grey_year)then
	if(test_mon .gt. grey_mon)then
	    if(DBLEV==15)then;write(msg,'(a,"QC15 HIT @2")')trim(msg); call wrtdbg(sub,msg); endif
	    HIT = .true.; tgl = igl; go to 10; ! found a GL HIT
	elseif(test_mon .eq. grey_mon)then
	    if(test_day .ge. grey_day)then
		if(DBLEV==15)then;write(msg,'(a,"QC15 HIT @3")')trim(msg); call wrtdbg(sub,msg); endif
		HIT = .true.; tgl = igl; go to 10; ! found a GL HIT
	    endif
	endif
    endif
    if(DBLEV==15)then;write(msg,'(a," MIS")')trim(msg); call wrtdbg(sub,msg); endif
enddo
write(msg,'("There are grey list entries for this float, but they do not affect this profile")'); call wrtlog(caller,msg);
result = PASS; ! we made it all the way through without finding a hit - so all pass
if(DBLEV==15)then; write(msg,'("MIS ngl>0, but there are no grey list time matches")'); call wrttrc(sub,msg); endif
if(.not. HIT) go to 99 ! Skip to THE_END
10 continue ! ! found a GL HIT

! So we do have grey list changes
do itype = 1,3	! NO DOXY GREYLIST 4
    greyflag(itype) = grey_flag(tgl,itype);
    write(msg,*)'Parameter: ',parmname(itype),' ongl? ',param_on_grey_list(tgl,itype),' flag: ',greyflag(itype); 
    if(DBLEV==15)then
        if(param_on_grey_list(tgl,itype))then
	    write(msg,'("QC15 HIT for parameter: ",a4," use flag: ",i1)')parmname(itype),greyflag(itype);
        else
	    write(msg,'("QC15 MIS for parameter: ",a4," use flag: ",i1)')parmname(itype),greyflag(itype);
        endif
        call wrtdbg(sub,msg)
    endif
enddo
result = PASS;	! Assume all ok, only chnage if one is bad
if(param_on_grey_list(tgl,1))then
    if(greyflag(1) > 1)then
	mxtflag = max(greyflag(1),mxtflag);
	result = FAIL;	! At least one had bad flag
	call mark_profflag_param(sub,15,greyflag(1),pinfo); ! Mark all PRES cols
	pres_is_greylisted = .true.;
    endif
endif
if(param_on_grey_list(tgl,2))then
    if(greyflag(2) > 1)then
	mxtflag = max(greyflag(2),mxtflag);
	result = FAIL;  ! At least one had bad flag
	call mark_profflag_param(sub,15,greyflag(2),tinfo); ! Mark all TEMP cols
	temp_is_greylisted = .true.;
    endif
endif
if(param_on_grey_list(tgl,3))then
    if(greyflag(3) > 1)then
	mxtflag = max(greyflag(3),mxtflag);
	result = FAIL;  ! At least one had bad flag
	call mark_profflag_param(sub,15,greyflag(3),sinfo); ! Mark all PSAL cols
	psal_is_greylisted = .true.;
    endif
endif

! NO DOXY GREYLIST if(param_on_grey_list(tgl,4))then
! NO DOXY GREYLIST     if(greyflag(4) > 1)then
! NO DOXY GREYLIST 	result = FAIL;  ! At least one had bad flag
! NO DOXY GREYLIST 	call mark_profflag_param(sub,15,greyflag(4),oinfo); ! Mark all DOXY cols
! NO DOXY GREYLIST     endif
! NO DOXY GREYLIST endif

99 continue ! This is THE_END

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile

if(DBGIO) call dbg_leave(sub,result);

qc15_grey_list = result;

idat = P(2);
iprf = REGPRFCORE(idat); ! Is this a regular CORE ARGO profile?
if(iprf > 0)then; QCTEST_FLAG(iprf,15) = max(QCTEST_FLAG(iprf,15),mxtflag); endif

return
end function qc15_grey_list
! ===========================================================================================================
