! ===========================================================================================================
! integer function qc16_jump_check.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
! -1 = NOQC - cannot be determined
!  0 = GOOD - jump did NOT coour
!  1 = PROB_BAD - jump  DID coour
!

! Find out if the difference of the 2 means need to be Absolute

integer function qc16_jump_check(caller,pdat,info)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: pdat		! location of the pres column
integer,      intent( in ) :: info(3)

character(len=4), parameter :: sub = 'QC16'
character(len=21) ,parameter :: QCTEST='Jump Check'
integer :: mlev; ! Max number of levels, this & prev

integer :: itype
real    ::     profdata_local(MAXLEV),prof_mean,prev_mean,mean_diff,initial_value,shallower_pres;
real :: prof_sum, prof_num, prev_sum, prev_num
real    :: layer_top, layer_bot
real    :: prevtestdata(MAXLEV),prev_initial_value,prev_prof_data_local(MAXLEV);
real    :: good_prof_pres_data,good_prev_pres_data,bad_percent
integer :: prof_nskip,prev_nskip;
integer :: prevtestflag(MAXLEV)
logical :: skippres, skipdata

integer :: ngood,nbad,nskip;
integer ::prof_count,prev_count, prof_lev,prev_lev
real :: pres_dbars = 105.0
real :: val_to_compare
integer :: flag

integer :: result;

! ===========================================================================================================
! executable statements

result = NOQC ! cannot be determined
itype=info(1); idat=info(2); ! run characteristics - do only the unadjusted/primary temp/psal profile

if(DBGIO) call dbg_enter_type(sub,parmname(itype))

if((.not. hasDATA(idat)) .or. (proftot(idat) == 0))then                           
    write(msg,'("I idat: ",i2," parameter ",a," has not data")')idat,parmchar(itype); call wrttrc(sub,msg);
    go to 999;
endif
mlev=max(nlev,nprevlev);
if(DEBUG)then; write(msg,*)'qc16 nlev: ',nlev,' prevlev: ',nprevlev,' max: ',mlev;; call wrtdbg(sub,msg); endif
if(nprevlev==0)then; write(msg,'("W nlev=0 for PREV profile, no test performed")');call wrttrc(sub,msg); go to 999; endif
! ADMT standard: use deepest 100 dbar common to both profiles.
! Problem: some floats may have pressure increment of 99-101 dbar (or so)
! to make sure to use the same number of levels in such cases allow 105 dbar

if(parmchar(itype)=='S')then
    val_to_compare=0.5
    prevtestdata = prevpsaldata
    prevtestflag = prevpsalflag
elseif(parmchar(itype)=='T')then
    val_to_compare=1
    prevtestdata = prevtempdata
    prevtestflag = prevtempflag
endif
if(DBLEV==16)then
    write(msg,*)'LEV16 idat,pdat,nlev,nprevlev:  ',idat,pdat,nlev,nprevlev; call wrtdbg(sub,msg);
    if(profbot(idat) /= nlev)then;
	write(msg,'("QC16 INFO @1 profbot(idat) /= nlev: ",2i5)')profbot(idat),nlev; call wrtlog(sub,msg);
    else
	write(msg,'("QC16 INFO @2 profbot(idat) /= nlev: ",2i5)')profbot(idat),nlev; call wrtlog(sub,msg);
    endif
endif

good_prof_pres_data = 0.0;
do i=profbot(idat),1,-1
    skippres = skip_test(profflag(idat,i)); skipdata = skip_test(profflag(idat,i)); 
    if(.not. skippres .and. .not. skipdata)then
        good_prof_pres_data = max(good_prof_pres_data,profdata(pdat,i));
    endif
enddo

good_prev_pres_data = 0.0
do i=1,nprevlev,1
    if((.not. skip_test(prevtestflag(i))) .and. (.not. skip_test(prevpresflag(i))) )then
        good_prev_pres_data = max(good_prev_pres_data,prevpresdata(i));
     endif
enddo

shallower_pres =  min(good_prof_pres_data,good_prev_pres_data);
layer_bot = shallower_pres + 2.5;	! In case the two deepest pres values are close
layer_top = layer_bot - pres_dbars;	! Layer thickness, but a bit of splash
if(DBLEV==16)then;
    write(msg,'("LEV16")');							call wrtdbg(sub,msg);
    write(msg,'("LEV16 good_prev_pres_data: ",f12.2)')good_prev_pres_data;	call wrtdbg(sub,msg);
    write(msg,'("LEV16 good_prof_pres_data: ",f12.2)')good_prof_pres_data;	call wrtdbg(sub,msg);
    write(msg,'("LEV16      shallower_pres: ",f12.2)')shallower_pres;		call wrtdbg(sub,msg);
    write(msg,'("LEV16           layer_top: ",f12.2)')layer_top;		call wrtdbg(sub,msg);
    write(msg,'("LEV16           layer_bot: ",f12.2)')layer_bot;		call wrtdbg(sub,msg);
    write(msg,'("LEV16")');							call wrtdbg(sub,msg);
    write(msg,'("LEV16             lev          pres  flg     data flg ")'); call wrtdbg(sub,msg);
endif; ! LEV16
prof_sum = 0.0; prof_num = 0.0; prof_nskip = 0;
do i=1,nlev
    if((profdata(pdat,i) >= layer_top) .and. (profdata(pdat,i) <= layer_bot))then;
	skippres = skip_test(profflag(pdat,i)); skipdata = skip_test(profflag(idat,i));
	if(skippres .or. skipdata)then;
	    prof_nskip = prof_nskip + 1;;
	    if(DBLEV==16)then;
		write(msg,'("LEV16 prof lev: ",i5," BAD ",f10.2,i3,f10.2,i3)')i, &
		profdata(pdat,i),profflag(pdat,i),profdata(idat,i),profflag(idat,i)
		call wrtdbg(sub,msg);
	    endif
	else
	    prof_sum = prof_sum + profdata(idat,i); prof_num = prof_num + 1.0;
	    if(DBLEV==16)then;
		write(msg,'("LEV16 prof lev: ",i5," use ",f10.2,i3,f10.2,i3,"  num: ",f4.0," sum: ",f10.2)')i, &
		profdata(pdat,i),profflag(pdat,i),profdata(idat,i),profflag(idat,i),prof_num,prof_sum
		call wrtdbg(sub,msg);
	    endif
	endif
    endif
enddo
if(prof_num == 0)then
    write(msg,'("There are no valid pres/data pairs in profdata")'); call wrttrc(sub,msg);
    go to 999;
else
    prof_mean = prof_sum/prof_num;
endif
if(prof_nskip > 0)then;
    write(msg,'("There are ",i6," bad points in the prof layer")')prof_nskip;	call wrttrc(sub,msg);
    if(prof_nskip>= prof_num)then;
	write(msg,'("More than half the prof points are bad, do NOT  perform the check")'); call wrttrc(sub,msg);
	go to 999;
    endif
endif
if(DBLEV==16)then;
    write(msg,'("LEV16")');; call wrtdbg(sub,msg);
    write(msg,'("LEV16             lev          pres  flg     data flg ")'); call wrtdbg(sub,msg);
endif

prev_sum = 0.0; prev_num = 0.0; prev_nskip = 0;
do i=1,nprevlev
    if((prevpresdata(i) >= layer_top) .and. (prevpresdata(i) <= layer_bot))then;
	skippres = skip_test(prevpresflag(i)); skipdata = skip_test(prevtestflag(i));
	if(skippres .or. skipdata)then;
	    prev_nskip = prev_nskip + 1;
	    if(DBLEV==16)then;
		write(msg,'("LEV16 prev lev: ",i5," BAD ",f10.2,i3,f10.2,i3)')i, &
		prevpresdata(i),prevpresflag(i),prevtestdata(i),prevtestflag(i)
		call wrtdbg(sub,msg);
	    endif
	else
	    prev_sum = prev_sum + prevtestdata(i); prev_num = prev_num + 1.0;
	    if(DBLEV==16)then;
		write(msg,'("LEV16 prev lev: ",i5," use ",f10.2,i3,f10.2,i3,"  num: ",f4.0," sum: ",f10.2)')i, &
		prevpresdata(i),prevpresflag(i),prevtestdata(i),prevtestflag(i),prev_num,prev_sum
		call wrtdbg(sub,msg);
	    endif
	endif
    endif
enddo
if(prev_num == 0)then
    write(msg,'("There are no valid pres/data pairs in prevdata")'); call wrttrc(sub,msg);
    go to 999;
else
    prev_mean = prev_sum/prev_num;
endif
if(prev_nskip > 0.0)then;
    write(msg,'("There are ",i6," bad points in the prev layer")')prev_nskip;    call wrttrc(sub,msg);
    if(prev_nskip >= prev_num)then
        write(msg,'("More than half the prev points are bad, do NOT  perform the check")'); call wrttrc(sub,msg);
        go to 999;
    endif
endif

mean_diff = abs(prof_mean - prev_mean);
if(DBLEV==16)then;
    write(msg,'("LEV16")'); ; call wrtdbg(sub,msg);
    write(msg,'("LEV16 for prof, num/sum/mean: ",f6.0,x,2f12.2)')prof_num,prof_sum,prof_mean;; call wrtdbg(sub,msg);
    write(msg,'("LEV16 for prev, num/sum/mean: ",f6.0,x,2f12.2)')prev_num,prev_sum,prev_mean;; call wrtdbg(sub,msg);
    write(msg,'("LEV16")');; call wrtdbg(sub,msg);
    write(msg,'("LEV16 DIFF: ",f12.2)')mean_diff;; call wrtdbg(sub,msg);
    write(msg,'("LEV16")');; call wrtdbg(sub,msg);
endif 

if(mean_diff > val_to_compare)then
    result = PROB_BAD;
    if(DBLEV==16)then; write(msg,'("LEV16 Set the jump check flag to PROB_BAD")'); call wrtdbg(sub,msg); endif
    ! Change the overall flag for the given parameter
    do i=1,nlev
	if(profflag(idat,ilev) < PROB_BAD) profflag(idat,ilev) = PROB_BAD		! Change the overall lev flag
	if(idat==s1r .and. profflag(s1a,ilev) < PROB_BAD) profflag(s1a,ilev) = PROB_BAD	! Change the overall lev flag
    enddo
else
    result = GOOD;
    if(DBLEV==16)then; write(msg,'("LEV16 Set the jump check flag to GOOD")'); call wrtdbg(sub,msg); endif
endif
if(DBLEV==16)then; write(msg,'("LEV16")');; call wrtdbg(sub,msg); endif
999 continue
if(DBLEV==16)then; write(msg,'("LEV16 result: ",i6)')result;; call wrtdbg(sub,msg); endif
! 25Jun2014 - actually mark the parameter as probably_bad...
if(DBGIO) call dbg_leave_type(sub,parmname(itype),result);

! For the individual/full-profile tests, report the result directly to all files

write(msg,'("Sensor drift/Jump Check ",a1,3x,a)')parmchar(itype),flagreport(result); call wrttrc(sub,msg);

qc16_jump_check = result;
if(result == GOOD)then; flag = 1; elseif(result == PROB_BAD)then; flag = 3; else; flag = 0; endif

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile
if(DBLEV==16)then; write(msg,*)'LEV16, exit with result: ',result; call wrtdbg(sub,msg); endif

iprf = REGPRFCORE(idat); ! Is this a regular CORE ARGO profile?
if(iprf > 0)then; QCTEST_FLAG(iprf,16) = max(QCTEST_FLAG(iprf,16),result); endif

return
end function qc16_jump_check
! ===========================================================================================================
