! ===========================================================================================================
! integer function qc18_frozen_profile.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180920 VH     Initionaliae arrays for calculation of mean values (mean_data & prev_mean_data), use
!                 use mask for max & nean (min already used mask)
! 20180619 VH     Only issue the WARNING if the profile is NOT YET on the greylist, put lat into message.
! 20180417 VH     General WARNING revisions.
! 20171018 VH     Make sure a warning is issued if a profile has a possible frozen profile
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values: DIFFERENT FROM OTHER Tests
!  1 = Success, profile is frozen
!  0 = Success, profile is not frozen
! -1 = Failure, unable to finish test
!

integer function qc18_frozen_profile(caller,info)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: info(3)

character(len=4), parameter :: sub = 'QC18'
character(len=21) ,parameter :: QCTEST='Frozen Profile'
integer :: result;

integer :: ifl,itype, beg, end,pres_lev,top,bot;
character(len=2) :: tmpchr
integer :: pres_diff ,count,val_to_compare,slab_count,islabs;
! defined in data_arrays, and (now) set in read_meta integer :: FROZEN_PROFLE_INDICATOR = 0;
integer :: ngood,prev_mean_slab_count,mean_slab_count,delta_count ; real :: real, pcnt,loop_max,loop_counter,nskip
real :: prevqcdata(MAXLEV),min_slab,pres_initial_value,prev_pres_initial_value,prevpresdata_local(MAXLEV)
real :: thispres_data(MAXLEV),pres_data(MAXLEV), sum_profdata_slabs
real :: thisprof_data(MAXLEV),prof_data(MAXLEV),mean_data(MAXLEV),prev_mean_data(MAXLEV),delta_data(MAXLEV)
real :: max_delta_data, mean_delta_data,min_delta_data,max_val_to_compare,min_val_to_compare,mean_val_to_compare
logical :: currqc,prevqc
character(len=SZC) :: tmpmsg;
logical :: param_is_greylisted = .false.;
integer :: ist; ! Which status to report in warning/lognote
logical :: mask(MAXLEV) = .false.

data mean_data /MAXLEV*0.0/
data prev_mean_data /MAXLEV*0.0/

! ===========================================================================================================
! executable statements

result = IND_NOT;	! INDICATOR not evaluated
if(DBLEV==18)then; write(msg,*)'qc18 @1 set result = IND_NOT'; call wrtdbg(sub,msg); endif

itype=info(1); beg=info(2); end=info(3); idat = beg; ! run characteristics

if(DBGIO) call dbg_enter_type(sub,parmname(itype))

if(nprevlev==0)then; 
    write(msg,'("QC18 W prev nlev=0, no FROZEN test performed")');call wrttrc(sub,msg); call wrtlog(sub,msg);
    if(DEBUG)then; call wrtdbg(sub,msg); endif
    ! result is already set to NOTD
    go to 999;
endif

pres_diff=50
mean_slab_count=0
prev_mean_slab_count=0

if(nprevlev .eq. 0)go to 999;

call reverse_prevqcdata(sub);
if(parmchar(itype)=='S')then
    max_val_to_compare=0.3
    min_val_to_compare=0.001
    mean_val_to_compare=0.004
    prevqcdata = prevpsaldata
    if(psal_is_greylisted) param_is_greylisted = .true.;
    if(DBLEV==118)then; write(msg,*)"STARTING PSAL "; call wrtlog(sub,msg); endif
elseif(parmchar(itype)=='T')then
    max_val_to_compare=0.3
    min_val_to_compare=0.001
    mean_val_to_compare=0.02
    prevqcdata = prevtempdata
    if(temp_is_greylisted) param_is_greylisted = .true.;
    if(DBLEV==118)then; write(msg,*)"STARTING TEMP"; call wrtlog(sub,msg); endif
else
     write(msg,*)"Cannot determine of the profile is frozen for type",parmchar(itype); call wrtdbg(sub,msg);
     go to 999
endif

currqc = .true.;prevqc = .false.


if(currqc .and. .not. prevqc )then
    do pres_lev=1,nlev
        pres_data(pres_lev)= profdata(pbud(idat),pres_lev)
        prof_data(pres_lev)= profdata(idat,pres_lev)
   enddo
   thispres_data = pres_data;
   thisprof_data = prof_data;
   top = 1   
   bot = nlev
endif
if(hasDATA(idat))then                                                                        ! if hasDATA
    if(proftot(idat) > 0)then                                                                ! if ntot>0

        ngood = 0
        100 continue
        count = 0
        do ilev=top,bot,1
            if(DBLEV==118)then;write(msg,*)"tttprofdata(idat,ilev),profflag(idat,ilev)=",profdata(idat,ilev),&
                                    profflag(idat,ilev),prevqcdata(ilev); call wrtdbg(sub,msg);endif;
            if(skip_test(profflag(idat,ilev)))then; nskip = nskip + 1;  go to 99; endif ! Skip this level 
	    200 continue
            if(count==0)then
    	        sum_profdata_slabs = 0.0
                !pres_initial_value = thispres_data(ilev)
		if(.not. prevqc)then
                    pres_initial_value = pres_diff  * mean_slab_count
		else
                    pres_initial_value = pres_diff  * prev_mean_slab_count
		endif
                if (thispres_data(ilev)> pres_initial_value + pres_diff - 1)then
 		   if(.not. prevqc)then
                      mean_slab_count= mean_slab_count + 1
		   else
                      prev_mean_slab_count = prev_mean_slab_count + 1
		   endif
                   goto 200
		endif
                if((thispres_data(ilev) - pres_initial_value < pres_diff)  .and. &
                   (thispres_data(ilev) - pres_initial_value >= 0))then
                   count = count + 1
                   sum_profdata_slabs = thisprof_data(ilev)+sum_profdata_slabs;
                   if(DBLEV==118)then
			write(msg,*)"presI,presL,profdata,sum,count == 0",pres_initial_value,",",thispres_data(ilev), &
			    ",",thisprof_data(ilev),",",sum_profdata_slabs,",",count;
			call wrtlog(sub,msg);
		   endif
		endif
            else
                if((thispres_data(ilev) - pres_initial_value < pres_diff)  .and. &
                   (thispres_data(ilev) - pres_initial_value >= 0))then
                    count = count + 1
		    sum_profdata_slabs = thisprof_data(ilev)+sum_profdata_slabs;
		    if(ilev == bot)then
    	                if(DBLEV==118)then
			    write(msg,*)"presI ,presL,profdata,sum,count", pres_initial_value,",",thispres_data(ilev), &
				",",thisprof_data(ilev),",",sum_profdata_slabs,",",count
			    call wrtlog(sub,msg);
			endif
		        go to 300 ! If last pressure level 
		    endif
    	            if(DBLEV==118)then;
			 write(msg,*) "presI ,presL,profdata,sum,count", pres_initial_value,",",thispres_data(ilev), &
			    ",",thisprof_data(ilev),",",sum_profdata_slabs,",",count
			call wrtlog(sub,msg)
		    endif
                go to 400
                endif
		300 continue
                if(.not. prevqc )then
                    mean_slab_count= mean_slab_count + 1
                    mean_data(mean_slab_count) =  sum_profdata_slabs/count
                    if(DBLEV==118)then;
			write(msg,*)"CURRQC mean,mean_slab_count :: ",mean_data(mean_slab_count),",", mean_slab_count
			call wrtlog(sub,msg);
		    endif
                else
                    prev_mean_slab_count = prev_mean_slab_count + 1
                    prev_mean_data(prev_mean_slab_count) = sum_profdata_slabs/count
                    if(DBLEV==118)then;
			write(msg,*) "PREVQC prevmean,prev_mean_slab_count ::", prev_mean_data(prev_mean_slab_count),",", &
			    prev_mean_slab_count
			call wrtlog(sub,msg);
		    endif
		    !pres_initial_value = 0.0
                endif
                count = 0
		go to 200
                400 continue
            endif
        99 continue
        enddo
        if(.not. prevqc)then 
            if(DBLEV==18)then; write(msg,*)" Starting previous QC "; call wrtlog(sub,msg); endif
            prevqc = .true. ;
            thispres_data = prevpresdata;
            thisprof_data = prevqcdata;
            top = 1
            bot = nprevlev
	    pres_initial_value = 0.0
            go to 100
       endif
       if(DBLEV==118)then;
	    write(msg,*)" mean_slab_count, prev_mean_slab_count",mean_slab_count,prev_mean_slab_count
	    call wrtlog(sub,msg);
	endif
       slab_count=min(mean_slab_count,prev_mean_slab_count) ! min to match what is currently done in a1
       if(DBLEV==118)then;
	    write(msg,*)"mean_slab_count,prev_mean_slab_count ::",mean_slab_count,",",prev_mean_slab_count; call wrtdbg(sub,msg);
	endif
       if(slab_count .eq. 0)go to 999
       do islabs = 1,slab_count,1
           delta_data(islabs)= abs(mean_data(islabs)-prev_mean_data(islabs));
           mask(islabs)= .true.
           if(DBLEV==118)then
		write(msg,*)"mean,prevmean,deltadata,slabcnt",mean_data(islabs),",",prev_mean_data(islabs), &
		    ",", delta_data(islabs),",",slab_count
	 	call wrtlog(sub,msg);
	  endif 
       enddo

       max_delta_data = maxval(delta_data,1,mask);
       mean_delta_data = sum(delta_data,1,mask)/slab_count;
       min_delta_data = minval(delta_data,1,mask);

	write(msg,'("QC18 nl,mmm nlev; ",i4)')nlev;              call wrtlog(sub,msg);
	write(msg,'("QC18 nl,mmm mean: "f12.4)')mean_delta_data; call wrtlog(sub,msg);
	write(msg,'("QC18 nl,mmm  min: "f12.4)')min_delta_data;  call wrtlog(sub,msg);
	write(msg,'("QC18 n;,mmm  max: "f12.4)')max_delta_data;  call wrtlog(sub,msg);

       if(DBLEV==118)then
	    write(msg,*)" max,mean,min :: ",max_delta_data,",", max_val_to_compare,",", mean_delta_data, &
		",",mean_val_to_compare,",",min_delta_data,",",min_val_to_compare
	    call wrtlog(sub,msg)
       endif	
       if(itype == 2)then; temp_is_frozen = .false.; else; psal_is_frozen = .false.; endif
       if(DBLEV==118)then;
	    write(msg,*)'qc18 @2 itype: ',itype,' temp/psal_is_frozen: ',temp_is_frozen,psal_is_frozen
	    call wrtdbg(sub,msg);
	endif

       if((max_delta_data < max_val_to_compare).and. (mean_delta_data < mean_val_to_compare) &
          .and. (min_delta_data < min_val_to_compare))then
            result = IND_ON;  ! INDICATOR is ON, PROFILE MAY QUALIFY AS FROZEN
	    write(msg,'("FROZEN INDICATOR is ON, PROFILE MAY QUALIFY AS FROZEN for ",a)')parmname(itype);
	    call wrttrc(sub,msg); call wrtlog(sub,msg);
	    if(itype == 2)then; temp_is_frozen = .true.; else; psal_is_frozen = .true.; endif
	    if(DBLEV==118)then; write(msg,*)'qc18 @3a set result = IND_ON'; call wrtdbg(sub,msg); endif
	    ! jun 2015 John Gilson requested a different treatment for some of his floats.
	    ! These meta files will have an enntry" frozen profile rule" and Reyna's shell script wilol
	    ! then tyrn this into an env var: FROZEN_PROFILE_RULE - if zero, treat as normall - ir one,
	    ! mark that data with a flag of 4, and take other actions (done in qcplus after qc18 has
	    ! been called for both temp and psal
	    ! First, see if the env var is present
	    write(msg,'("QC18 FROZEN INDICATOR is ON, frozen_profile_rule is ",i2)') frozen_profile_rule
	    if(frozen_profile_rule == 1)then;
		write(msg,'(a," the frozen_profile_rule is 1, so the Gilson method will be used")')trim(msg)
		! The frozen profile rule is that if a profile is fronzen, mark all levels as BAD=4
		do idat=info(2),info(3)
		    if(hasDATA(idat))then;
			do ilev=1,nlev
			    profflag(idat,ilev) = BAD;	! Bad=4
			enddo
		    endif
		enddo
		write(tmpmsg,153)parmchar(itype); ist = 153;
		153 format("153 qc18 frozen profile ",a," FRZN_PRF;_RULE=1, mark 4 as if a flag")
	    else
		write(tmpmsg,154)parmchar(itype); ist = 154;
		154 format("154 qc18 frozen profile ",a," FRZN_PRFE_RULE=0, treat normallly")
	    endif
	    if(param_is_greylisted)then
		write(logmsg,'("LOGNOTE ",a)')trim(tmpmsg); ! LGN ist may mean LOGNOTE 153 or LOGNOTE 154
		call lognote(sub,ist,logmsg);               ! LGN ist may mean LOGNOTE 153 or LOGNOTE 154
	    else
		write(wrnmsg,'("WARNING ",a," LAT: ",f8.2)')trim(tmpmsg),real_LATITUDE; ! WRN
		call warning(sub,ist,wrnmsg);               ! WRN ist mmay men WARNING 153 or WARNING 154 ! WRN
	    endif
       else     
            result = IND_OFF;  ! INDICATOR is off PROFILE IS NOT FROZEN
            write(msg,'("FROZEN INDICATOR is OFF - PROFILE IS NOT FROZEN for ",a)')parmname(itype);
	    
	    call wrttrc(sub,msg); call wrtlog(sub,msg);
	    if(DBLEV==118)then; write(msg,*)'qc18 @3b set result = IND_OFF'; call wrtdbg(sub,msg); endif
       endif
       if(DBLEV==118)then;
	    write(msg,*)'qc18 @4 itype: ',itype,' temp/psal_is_frozen: ',temp_is_frozen,psal_is_frozen
	    call wrtdbg(sub,msg);
	endif
    endif ! if(proftot(idat) == 0)                                                                ! if ntot>0
endif                                                                                        ! if hasDATA

999 continue
if(DBLEV==18)then;
     if(result==IND_ON)then;
	write(msg,'(a," result is IND_ON for parameter: ",a)')sub,parmname(itype); call wrtdbg(sub,msg);
    else
	write(msg,'(a," result is IND_OFF for parameter: ",a)')sub,parmname(itype); call wrtdbg(sub,msg);
    endif
endif

! The result to return to the main is based on the first data field - pres/temp/psal/doxy for primary profile
if(DBGIO) call dbg_leave_type(sub,parmname(itype),result)

if(DBLEV==18)then;write(msg,*)"Frozen profile result = " ,result;call wrtdbg(sub,msg);endif;

qc18_frozen_profile = result;
! This result is an INDICATOR: one of: IND_NOT  = -1, IND_OFF  =  0, IND_ON   =  1 
! The result as an INDICATOR is sent back (ie, -1,0,1)
if(result == IND_OFF)then; result = 1; elseif(result == IND_ON)then; result = 3; else; result = 0; endif ! convert to FLAG value
! the flag version is used for QCTEST_FLAG, but thhis is all VERY CONFUSING - FIX this
iprf = REGPRFCORE(idat); ! Is this a regular CORE ARGO profile?
if(iprf > 0)then; QCTEST_FLAG(iprf,18) = max(QCTEST_FLAG(iprf,18),result); endif

return
end function qc18_frozen_profile
! ===========================================================================================================
