! ===========================================================================================================
! integer function qc19_deepest_pressure.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! This test requires that the profile has pressures that are not higher than
! DEEPEST_PRESSURE plus 10 percent.
!
! DEEPEST_PRESSURE value comes from the meta-data file of the float.
!
! Action: If there is a region of incorrect pressures, all pressures and 
! corresponding measurements should be flagged as bad data (flag '4'). All 
! pressures flagged as bad data and all of the associated temperatures and 
! salinities are removed from the TESAC distributed on the GTS.
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc19_deepest_pressure(caller,info)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: info(3)

character(len=4), parameter :: sub = 'QC19'
character(len=21) ,parameter :: QCTEST='Deepest Pressure Test'
integer :: result;
 
integer :: ifl, itype, beg, end;
integer :: ngud,nbad,nskp

real :: test_maximum_pres;

! ===========================================================================================================
! executable statements

ifl = deep_pres_ncol; 

result = NOTD ! cannot be determined

itype=info(1); beg=info(2); end=info(3); ! run characteristics

if(DBGIO) call dbg_enter_type(sub,parmname(itype))

test_maximum_pres = maximum_prof_pres*1.1;

if(DBLEV==19)then;
    write(msg,'("maximum_prof_pres: ",f12.2)')maximum_prof_pres; call wrtdbg(sub,msg); ! LEV19
    write(msg,'("test_maximum_pres: ",f12.2)')test_maximum_pres; call wrtdbg(sub,msg); ! LEV19
endif                                                                                  ! LEV19
do idat=beg,end											! idat=beg,end
    if(hasDATA(idat))then									! if hasDATA
	if(proftot(idat) > 0)then								! if ntot>0
	    ngud = 0; nbad = 0; nskp = 0;
	    do ilev=1,nlev
		if(skip_test(profflag(idat,ilev)))then
		    nskp = nskp + 1;
		    go to 99;
		endif ! Skip this level
		if(profdata(idat,ilev) > test_maximum_pres)then
		    testflag(idat,ilev,ifl) = BAD; nbad = nbad + 1;
		else
		    testflag(idat,ilev,ifl) = GOOD;; ngud = ngud + 1;
		endif
		99 continue ! skip_test
	    enddo
	endif 											! if ntot>0
	call evaluate_flags(sub,QCTEST,datname(idat),idat,ifl,beg,result); maxresult=max(maxresult,result);
    endif !  if(hasDATA										! if hasDATA
enddo												! idat=beg,end

! The result returned is the max flag of those evaluated
qc19_deepest_pressure = maxresult;
if(DBGIO) call dbg_leave_type(sub,parmname(itype),result);

return
end function qc19_deepest_pressure
! ===========================================================================================================
