! ===========================================================================================================
! integer function qca2_analysis.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release    of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  1 = Success, profile passed
! -1 = Failure, profile failed
!  0 = Unable to determine
!

integer function qca2_analysis(caller,info)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: info(3)

character(len=4), parameter :: sub = 'QCA2'
character(len=21) ,parameter :: QCTEST='Analysis Test'
integer :: result;

integer :: ifl, itype, beg, end;
integer :: ngood,nbad,nskip;

! ===========================================================================================================
! executable statements

ifl = analysis_ncol; 

result = NOTD; ! "can not be determined."

itype=info(1); beg=info(2); end=info(3); ! run characteristics

if(DBGIO) call dbg_enter_type(sub,parmname(itype))

write(msg,'(a,x,a," is not functional")')sub,QCTEST;
if(DEBUG)then; call wrtdbg(sub,msg); else; call wrttrc(sub,msg); endif
if(DBLEV/=24) return; if(DBLEV==24) return;

do idat=beg,end											! idat=beg,end
    if(hasDATA(idat))then									! if hasDATA
	if(proftot(idat) > 0)then								! if ntot>0

	    ngood = 0; nbad = 0; nskip = 0;
	    do ilev=1,nlev
		if(skip_test(profflag(idat,ilev)))then; nskip = nskip + 1;  go to 99; endif ! Skip this level
		if(profdata(idat,ilev) < FTST)then
		    testflag(idat,ilev,ifl) = MISSING
		elseif(profdata(idat,ilev) < FTST)then ! DUMMY TEST, insert the real one here
		    testflag(idat,ilev,ifl) = BAD;
		else
		    testflag(idat,ilev,ifl) = GOOD; ngood = ngood + 1;
		endif
		99 continue ! skip_test
	    enddo

	endif ! if(proftot(idat) == 0)								! if ntot>0
	! NOT NOW call evaluate_flags_obs(sub,QCTEST,datname(idat),idat,ifl,beg,result)	! evaluate the flags for this test
    endif											! if hasDATA
enddo												! idat=beg,end

! The result returned from evaluate_flags is for the PRIMARY profile (unadjusted field)
qca2_analysis = result;
if(DBGIO) call dbg_leave_type(sub,parmname(itype),result);

return
end function qca2_analysis
! ===========================================================================================================
