! i=========================================================================================================
! character function write_qcprof.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20190521 VH     Only write out the grey list fla line if the flag is > 1
! 20190521 VH     Write the variable name "JULD FROM" for that qc line
! 20190416 VH     Include the ICE_DETECTION_&_INTERPOLATION_COMMENT if there is an interpolated position.
! 20190311 VH     Write LOCATION_COMMENT even if no valid position, eliminate LOCATION FROM (posloc now in posn_cmnt)
! 20190114 JN     Fix "NUMBER OF COLUMNS: problem
! 20180724 VH     INTERPOLATION/ICE DETECTION: Write out qc lines if int. ice lines in phy file
! 20180619 VH     Write PROFILE_UPLOAD_TIME_QC if PROFILE_UPLOAD_TIME is written from header
! 20180426 VH     Eliminate extraneous check for open file
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! TEMP units DEGC DEG C deg... See notes in read_pqc.f90

! Function returns 'y' for successful file creation, "F" for failure
! ===========================================================================================================

character function write_qcprof(qcplus_version,qcprof)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: qcplus_version
character(*), intent( in ) :: qcprof

character(len=8)   :: testch
character(len=12)  :: testch12
character(len=40)  :: c40
integer :: status, ios, ic,  nback;
character(len=SZC) :: oline           ! generic output line
character(len=SZC) :: odata           ! data output line
character(len=SZC) :: oflag           ! flag output line (ie, testflag - one for each test)
character(len=SZC) :: xflag           ! extra flags for whole level (from profflag - grade for that level)
logical :: HDR = .false.; integer :: nch; character(len=SZC) :: h1, h2, h3, h4, h5, h6  ! Header lines, (if needed)

logical :: file_exist
character(len=11) :: suffix     ! Suffix of flag name = the <P/T/S/adj> + FLAG
character(len=72) :: c72, write_var_ival ! declare the function to left-justify var/ival pairs
character(len=24) :: ch24	! Use for writing o9ut the JULD

integer :: icolout, ncolout, icoltot 			! How many columns are in the "column" header
integer :: ncolwrt					! How many cols are we actually writing out?
logical :: ckcw = .true.				! Check the number of columns written out (only do once)
integer :: beg, end, inc, max,loc
character(len=5) :: parnam
character(len=1) :: tqc
character(len=9) :: char7
character(len=20) :: lefty
character(len=82), parameter :: QCSEPLINE = '=================================================================================='
                                          !1234567890123456789012345678901234567890123456789012345678901234
 
character(len=12), parameter :: sub = 'write_qcprof'

! ===========================================================================================================
! executable statements

! Which file is being written, and wat is the filmsg index?
filmsg(l:l) = 'R'; call wrtflm(sub,'Initialize to Running')

if(DEBUG) call dbg_enter(sub);

! First, determin which columns we have, and if we have enought room for them all
icolout = 0; ! keep track of the out columns

! moved up 13Mar14
do icol = 1,ncolinp ! First all of the input columns
    if(trim(inpcols(icol)) == "PRESSURE_ADJ (DBAR)")then
        if(DEBUG)then; msg='do NOT put old pres_adj col name into qcprof'; call wrtdbg(sub,msg); endif
    elseif(trim(inpcols(icol)) == "PSAL_ADJUSTED (PSU)")then
        if(DEBUG)then; msg='do NOT put old psal_adj col name into qcprof'; call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"PHASE") > 0)then
        if(DEBUG)then; write(msg,*)'do NOT put PHASE columns into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"TEMP_DOXY") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put TEMP_DOXY columninto qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"NITRATE") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put NITRATE columns into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"SIG") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put SIG columns into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"VOLTAGE") > 0)then
        if(DEBUG)then; write(msg,*)'do NOT put VOLTAGE columns into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"RESOLUTION OF") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put RESOLUTION columns into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"DOXYB") > 0)then
        if(DEBUG)then; write(msg,*)'do NOT put DOXYB column into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"DOXYB") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put DOXYC column into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"INTERP_STATUS_") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put INTERP_STATUS_T2/S2 into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"BATTERY ") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put BATTERY into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"FREQUENCY") > 0)then	
	if(DEBUG)then; write(msg,*)'do NOT put FREQUENCY into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"NUMBER OF VALUES") > 0)then
	if(DEBUG)then; write(msg,*)'do NOT put NUMBER OF VALUES into qcprof: ',trim(inpcols(icol)); call wrtdbg(sub,msg); endif
    elseif(index(inpcols(icol),"DISSOLVED OXYGEN") > 0)then
	if(index(inpcols(icol),"DISSOLVED OXYGEN (") > 0)then
	    icolout = icolout + 1; outcols(icolout) = trim(inpcols(icol));
	elseif(index(inpcols(icol),"DISSOLVED OXYGEN2 (") > 0)then
	    icolout = icolout + 1; outcols(icolout) = trim(inpcols(icol));
	elseif(index(inpcols(icol),"DISSOLVED OXYGEN2A (") > 0)then
	    icolout = icolout + 1; outcols(icolout) = trim(inpcols(icol));
	elseif(index(inpcols(icol),"DISSOLVED OXYGEN2B (") > 0)then
	    icolout = icolout + 1; outcols(icolout) = trim(inpcols(icol));
	elseif(index(inpcols(icol),"DISSOLVED OXYGEN2C (") > 0)then
	    icolout = icolout + 1; outcols(icolout) = trim(inpcols(icol));
	endif
    else
	icolout = icolout + 1; outcols(icolout) = trim(inpcols(icol));
    endif
enddo

! 28Jul2014 - two phy files were incorrect, and had NOT column defs - catch this problem
if(icolout == 0 .and. nlev>0)then
    write(errmsg(1),'("FATAL ERROR 901 QCPROF WRTFILE There are NO columns from the input phy file, but nlev > 0")'); ! ERR
    write(errmsg(2),'("FATAL ERROR 901 QCPROF WRTFILE For this problem, scream a bit, then exit abruptly")'); ! ERR
    write(errmsg(3),'("FATAL ERROR 901 QCPROF WRTFILE The phy file is probably in bad shape.")'); ! ERR
    call error_exit(sub,901,3,errmsg); ! ERR QCPROF WRTFIL
endif

if(do_pres_adj)then
    icolout = icolout + 1; outcols(icolout) = "PRESSURE_ADJ (DBAR)";
    if(hasDATA(p2r))then
	icolout = icolout + 1; outcols(icolout) = "PRESSURE2_ADJ (DBAR)"
	if(hasDATA(p3r))then
	    icolout = icolout + 1; outcols(icolout) = "PRESSURE3_ADJ (DBAR)"
	endif
    endif
endif

if(do_psal_adj)then
    icolout = icolout + 1; outcols(icolout) = "PSAL_ADJUSTED (PSU)";
    if(hasDATA(s2r))then
	icolout = icolout + 1; outcols(icolout) = "PSAL2_ADJUSTED (PSU)"
	if(hasDATA(s3r))then
	    icolout = icolout + 1; outcols(icolout) = "PSAL3_ADJUSTED (PSU)"
	endif
    endif
endif
! Next, for the level flage for P,T,S n_prof=1
if(hasDATA(p1r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_P FLAG"; endif
if(hasDATA(t1r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_T FLAG"; endif
if(hasDATA(s1r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_S FLAG"; endif
if(hasDOXY(1))then;      icolout = icolout + 1; outcols(icolout) = "LEV CHECK_O FLAG"; endif

! Next, for the level flage for P,T,S n_prof=2
if(hasDATA(p2r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_P2 FLAG"; endif
if(hasDATA(t2r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_T2 FLAG"; endif
if(hasDATA(s2r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_S2 FLAG"; endif

if(hasDOXY(2))then;     icolout = icolout + 1; outcols(icolout) = "LEV CHECK_O2 FLAG"; endif
if(hasDOXY(3))then;     icolout = icolout + 1; outcols(icolout) = "LEV CHECK_O2A FLAG"; endif
if(hasDOXY(4))then;     icolout = icolout + 1; outcols(icolout) = "LEV CHECK_O2B FLAG"; endif
if(hasDOXY(5))then;     icolout = icolout + 1; outcols(icolout) = "LEV CHECK_O2C FLAG"; endif

! Next, for the level flage for P,T,S n_prof=3
if(hasDATA(p3r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_P3 FLAG"; endif
if(hasDATA(t3r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_T3 FLAG"; endif
if(hasDATA(s3r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_S3 FLAG"; endif

if(do_pres_adj)then
    if(hasDATA(p1r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_P ADJ FLAG"; endif
    if(hasDATA(p2r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_P2 ADJ FLAG"; endif
    if(hasDATA(p3r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_P3 ADJ FLAG"; endif
endif
if(do_psal_adj)then
    if(hasDATA(s1r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_S ADJ FLAG"; endif
    if(hasDATA(s2r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_S2 ADJ FLAG"; endif
    if(hasDATA(s3r))then; icolout = icolout + 1; outcols(icolout) = "LEV CHECK_S3 ADJ FLAG"; endif
endif


! Now for all the flag fields
!======================= PRES  Create COLUMN names for pres flags
nqcout = 0
if(hasDATA(p1r))then
    nqcout=nqcout+1; qcout(nqcout) = P(2); ! begining element of P 
    do iflg=1,NFLAGPRES
	flag_name = trim(pres_flg_name(iflg)) // "_" // trim(datname(p1r)) // " FLAG"
	icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= TEMP  Create COLUMN names for temp flags
if(hasDATA(t1r))then
    nqcout=nqcout+1; qcout(nqcout) = T(2); ! begining element of T
    if(do_regional_test)then
	flag_name = trim(rgnl_flg_name) // "_" // trim(datname(t1r)) // " FLAG"
	icolout = icolout + 1; outcols(icolout) = flag_name
    endif
    do iflg=1,NFLAGTEMP
        flag_name = trim(temp_flg_name(iflg)) // "_" // trim(datname(t1r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= PSAL  Create COLUMN names for psal flags
if(hasDATA(s1r))then
    nqcout=nqcout+1; qcout(nqcout) = S(2); ! begining element of S
    if(do_regional_test)then
	flag_name = trim(rgnl_flg_name) // "_" // trim(datname(s1r)) // " FLAG"
	icolout = icolout + 1; outcols(icolout) = flag_name
    endif
    do iflg=1,NFLAGPSAL
        flag_name = trim(psal_flg_name(iflg)) // "_" // trim(datname(s1r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif

!======================= DOXY  Create COLUMN names for doxy1 flags
if(hasDOXY(1))then
    nqcout=nqcout+1; qcout(nqcout) = O(2); ! first element of O
    do iflg=1,NFLAGDOXY
        flag_name = trim(doxy_flg_name(iflg)) // "_" // trim(datname(do1)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= PRES2 Create COLUMN names for pres2 flags
if(hasDATA(p2r))then
    nqcout=nqcout+1; qcout(nqcout) = P(2)+2; ! begining element of P
    do iflg=1,NFLAGPRES
        flag_name = trim(pres_flg_name(iflg)) // "_" // trim(datname(p2r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= TEMP2 Create COLUMN names for temp2 flags
if(hasDATA(t2r))then
    nqcout=nqcout+1; qcout(nqcout) = T(2)+1; ! begining element of T
    if(do_regional_test)then
        flag_name = trim(rgnl_flg_name) // "_" // trim(datname(t2r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    endif
    do iflg=1,NFLAGTEMP
        flag_name = trim(temp_flg_name(iflg)) // "_" // trim(datname(t2r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= PSAL2 Create COLUMN names for psal2 flags
if(hasDATA(s2r))then
    nqcout=nqcout+1; qcout(nqcout) = S(2)+2; ! begining element of S
    if(do_regional_test)then
        flag_name = trim(rgnl_flg_name) // "_" // trim(datname(s2r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    endif
    do iflg=1,NFLAGPSAL
        flag_name = trim(psal_flg_name(iflg)) // "_" // trim(datname(s2r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= DOXY2 Create COLUMN names for doxy2 flags
if(hasDOXY(2))then
    nqcout=nqcout+1; qcout(nqcout) = O(2)+1; ! second element of O
    do iflg=1,NFLAGDOXY
        flag_name = trim(doxy_flg_name(iflg)) // "_" // trim(datname(do2)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= DOXYA Create COLUMN names for doxy2A flags
if(hasDOXY(3))then
    nqcout=nqcout+1; qcout(nqcout) = O(2)+1; ! second element of O
    do iflg=1,NFLAGDOXY
        flag_name = trim(doxy_flg_name(iflg)) // "_" // trim(datname(doa)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= DOXYB Create COLUMN names for doxy2B flags
if(hasDOXY(4))then
    nqcout=nqcout+1; qcout(nqcout) = O(2)+1; ! second element of O
    do iflg=1,NFLAGDOXY
        flag_name = trim(doxy_flg_name(iflg)) // "_" // trim(datname(dob)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= DOXYC Create COLUMN names for doxy2C flags
if(hasDOXY(5))then
    nqcout=nqcout+1; qcout(nqcout) = O(2)+1; ! second element of O
    do iflg=1,NFLAGDOXY
        flag_name = trim(doxy_flg_name(iflg)) // "_" // trim(datname(doc)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo

endif

!======================= PRES3 Create COLUMN names for pres3 flags
if(hasDATA(p3r))then
    nqcout=nqcout+1; qcout(nqcout) = P(2)+2; ! begining element of P
    do iflg=1,NFLAGPRES
        flag_name = trim(pres_flg_name(iflg)) // "_" // trim(datname(p3r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= TEMP3 Create COLUMN names for temp3 flags
if(hasDATA(t3r))then
    nqcout=nqcout+1; qcout(nqcout) = T(2)+1; ! begining element of T
    if(do_regional_test)then
        flag_name = trim(rgnl_flg_name) // "_" // trim(datname(t3r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    endif
    do iflg=1,NFLAGTEMP
        flag_name = trim(temp_flg_name(iflg)) // "_" // trim(datname(t3r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= PSAL3 Create COLUMN names for psal3 flags
if(hasDATA(s3r))then
    nqcout=nqcout+1; qcout(nqcout) = S(2)+2; ! begining element of S
    if(do_regional_test)then
        flag_name = trim(rgnl_flg_name) // "_" // trim(datname(s3r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    endif
    do iflg=1,NFLAGPSAL
        flag_name = trim(psal_flg_name(iflg)) // "_" // trim(datname(s3r)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
endif
!======================= PRESA Create COLUMN names for pres_adj flags
if(do_pres_adj)then
    nqcout=nqcout+1; qcout(nqcout) = P(2)+1; ! second element of P
    do iflg=1,NFLAGPRES		! Why on earth did I do this???? ,2
	flag_name = trim(pres_flg_name(iflg)) // "_" // trim(datname(p1a)) // " FLAG"
	icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
    if(hasDATA(p2r))then;
	nqcout=nqcout+1; qcout(nqcout) = P(2)+3; ! fourth element of P
	do iflg=1,NFLAGPRES
	        flag_name = trim(pres_flg_name(iflg)) // "_" // trim(datname(p2a)) // " FLAG"
	    icolout = icolout + 1; outcols(icolout) = flag_name
	enddo
    endif
    if(hasDATA(p3r))then;
        nqcout=nqcout+1; qcout(nqcout) = P(2)+5; ! sixth element of P
        do iflg=1,NFLAGPRES
                flag_name = trim(pres_flg_name(iflg)) // "_" // trim(datname(p3a)) // " FLAG"
            icolout = icolout + 1; outcols(icolout) = flag_name
        enddo
    endif
endif
!======================= PSALA Create COLUMN names for psal_adj flags
if(do_psal_adj)then
    nqcout=nqcout+1; qcout(nqcout) = S(2)+1; ! second element of S
    do iflg=1,NFLAGPSAL
        flag_name = trim(psal_flg_name(iflg)) // "_" // trim(datname(s1a)) // " FLAG"
        icolout = icolout + 1; outcols(icolout) = flag_name
    enddo
    if(hasDATA(s2r))then;
	nqcout=nqcout+1; qcout(nqcout) = S(2)+3; ! fourthe element of S
	do iflg=1,NFLAGPSAL
	    flag_name = trim(psal_flg_name(iflg)) // "_" // trim(datname(s2a)) // " FLAG"
	    icolout = icolout + 1; outcols(icolout) = flag_name
	enddo
    endif
    if(hasDATA(s3r))then;
	nqcout=nqcout+1; qcout(nqcout) = S(2)+5; ! fourthe element of S
	do iflg=1,NFLAGPSAL
	    flag_name = trim(psal_flg_name(iflg)) // "_" // trim(datname(s3a)) // " FLAG"
	    icolout = icolout + 1; outcols(icolout) = flag_name
	enddo
    endif
endif
!======================= NSFLAGS Cread COLUMN name for near_surface flag
! There is a single flag for the near_surface column - it applies to p,t,s in the near_surfae profile
! It will then be place at the far right (of ALL flags)
if(near_surface(0))then
    nqcout=nqcout+1; qcout(nqcout) = 0;		! ARE THESE EVER USED - FIX THIS
    write(flag_name,'("NEAR_SURFACE MIX AIR/WATER CHECK_P",i1," FLAG")')near_surface_profile
    icolout = icolout + 1; outcols(icolout) = flag_name
endif
!======================= end of defining column names for flags

ncolout = icolout	!
if(ncolout > MAXCOLS)then
    write(errmsg(1),'("FATAL ERROR 901 QCPROF WRTFILE ncolout (",i3,") > MAXCOLS (",i3,")")')ncolout,MAXCOLS; ! ERR
    write(errmsg(2),'("FATAL ERROR 901 QCPROF WRTFILE Recount the cols, or reset MAXCOLS")'); ! ERR
    call error_exit(sub,901,2,errmsg); ! ERR QCPROF WRTFIL
endif
! Next, open the output qcprof file

out_open=open_new_file(OUT,qcprof,"OUT","New QC file");
write(msg,'("write QCPROF: ",a)')trim(qcprof); call wrtstd(sub,msg); ! STD

call wrtflm(sub,'After opening outputfile');

! Did the file open correctly?
if(.not. out_open)then;
    write(msg,'(a,"Input file was NOT opened: ",a)')HEDR,trim(qcprof);
    if(filmsg(l:l) == 'R')then;
    endif
    return
else
    write(msg,'(a,"Input file was opened: ",a)')HEDR,trim(qcprof);
endif
call wrtflm(sub,'After opening outputfile');

! Did the file fail to auccessfully open? - either out_open=F, or filmsg(l:l) has changed from  'R'
if(.not. out_open)then; write(msg,'(a,"out_opet=F, FILMSG: ",a)')HEDF,filmsg(l:l); call wrttrc(sub,msg); return; endif
if(filmsg(l:l) /= 'R')then; write(msg,'(a,"out_opet=T, FILMSG no R: ",a)')HEDF,filmsg(l:l); call wrttrc(sub,msg); return; endif

write(msg,'(a,"output file opened: ",a)')WRTFILE,trim(qcprof); call wrtlog(sub,msg);

! Header informatin lines

do ihed=1,nhed
    if(index(hedline(ihed),"VERTICAL SAMPLING SCHEME") > 0)then; go to 10; endif
    write(oline,'(a)')trim(hedline(ihed)); write(OUT,'(a)')trim(oline)
    if(hedline(ihed)(1:32)=='TIME ASCENT END (YYYYMMDDHHMMSS)')then
	write(oline,'("TIME ASCENT END QC                      ",a1)')TIME_ASCENT_END_QC;  write(OUT,'(a)')trim(oline)
    elseif(hedline(ihed)(1:32)=='START OF TRANSMISSION FROM EPOCH')then
	! Do not write the qc value - it is not being tested at tis time
    elseif(hedline(ihed)(1:21)=='START OF TRANSMISSION')then
	write(oline,'("START OF TRANSMISSION QC                ",a1)')TIME_TRANS_START_QC;  write(OUT,'(a)')trim(oline)
    elseif(hedline(ihed)(1:19)=='PROFILE_UPLOAD_TIME')then
	write(oline,'("PROFILE_UPLOAD_TIME_QC                  "a1)')PROFILE_UPLOAD_TIME_QC;  write(OUT,'(a)')trim(oline)
    endif
    10 continue;	! skip VERTICAL SAMPLING SCHEME for now
enddo

if(sampling_found)then
    write(oline,'("VERTICAL SAMPLING SCHEME FROM           ",a)')trim(VERT_SAMPLING_SCHEME_FROM);
    write(OUT,'(a)')trim(oline)
    do i=1,sampling_entries
	write(oline,'("VERTICAL SAMPLING SCHEME(",i1,")             ",a)')i,trim(chr_VERT_SAMPLING_SCHEME(i));
	write(OUT,'(a)')trim(oline)
    enddo
endif

! See if any profiles are near-surface
if(near_surface(0) .and. nlev>0)then;	! NEAR SURF
    write(OUT,'(a)')trim(nsqcline)
endif					! NEAR SURF

! Write out the profile direction
write(oline,'("DIRECTION                               ",a)') chr_DIRECTION; write(OUT,'(a)')trim(oline)

! Write all the juld stiff
write(oline,'("DATE_TIME_PROFILE                       ",a)')prof_DATE_TIME_PROFILE; write(OUT,'(a)')trim(oline)
write(oline,'("JULD FROM                               ",a)')trim(juld_from); write(OUT,'(a)')trim(oline)
if(DEBUG) write(DBG,*)'In WRQC, oline: ',trim(oline);
if(DBGMCL)then; write(DBG,'("WRQC DBG MCL ",a20,"         JULD FROM ",a)')profile,trim(juld_from); endif
if(DBGMCL .and. DBLEVEXIT)then; write(DBG,'("TEST EXIT from wrote+qc[rpf")'); call exit(0); endif

if(prof_JULD==FILLDBLE)then
    ch24 = '999999.0';
else
    write(ch24,'(f18.12)')prof_JULD;
endif
write(oline,'("JULD                                    ",a24)')ch24; write(OUT,'(a)')trim(oline)
write(oline,'("JULD QC                                 ",a1)')prof_JULD_QC;  write(OUT,'(a)')trim(oline)

if(DBLEV== 2)then
    write(DBG,'("LEV02 TEST DATE qcwrite DATE_TIME_PROFILE: ",a)')prof_DATE_TIME_PROFILE; ! DBG LEV02
    write(DBG,'("LEV02 TEST DATE qcwrite JULD_QC: ",a1,"   JULD:   ",a)')prof_JULD_QC,ch24;     ! DBG LEV02
endif

! Now, write out the position information (new Dec13)
write(ch24,'(f10.5)')real_LATITUDE
write(oline,'("LATITUDE                                ",a24)')ch24; write(OUT,'(a)')trim(oline)
write(ch24,'(f10.5)')real_LONGITUDE
write(oline,'("LONGITUDE                               ",a24)')ch24; write(OUT,'(a)')trim(oline)
write(oline,'("POSITION_QC                             ",a1)')prof_POSITION_QC; write(OUT,'(a)')trim(oline)
write(oline,'("DATE_TIME_LOCATION                      ",a14)')prof_DATE_TIME_LOCATION; write(OUT,'(a)')trim(oline)
if(prof_JULD_LOCATION> 99990.0)then;
    write(ch24,'(f8.1)')prof_JULD_LOCATION
    write(oline,'("JULD_LOCATION                           999999.0")');            write(OUT,'(a)')trim(oline)
    write(oline,'("LOCATION_COMMENT                        ",a)')trim(posn_cmnt); write(OUT,'(a)')trim(oline)
else
    write(ch24,'(f18.12)')prof_JULD_LOCATION
    write(oline,'("JULD_LOCATION                           ",a24)')ch24; write(OUT,'(a)')trim(oline)
    write(oline,'("LOCATION_COMMENT                        ",a)')trim(posn_cmnt); write(OUT,'(a)')trim(oline)
endif

! July 2018 - ICE INT Include interpolation/ice detection status information IF THERE ARE ICE LINES IN THE PHY FILE
if(ice_int_status /= 0)then
    write(oline,'("ICE_DETECTION_&_INTERPOLATION_COMMENT   ",a)')trim(ice_int_cmnt); write(OUT,'(a)')trim(oline)
endif

! Write out some more informaitn

write(testch12,'(f12.2)')maximum_prof_pres;
write(OUT,'(a40,a)')"MAXIMUM_EXPECTED_PROFILE_PRES           ",adjustl(testch12);
write(OUT,'(a40,a)')"MAXIMUM_EXPECTED_PROFILE_PRES FROM      ",trim(maximum_prof_pres_from);

! Did we find surface pres?

if(found_surf_pres .or. try_pres_adj)then
    write(OUT,'("SURFACE PRESSURE COMMENT                ",a)')trim(surf_pres_comment);
    write(msg,'("SURFACE PRESSURE COMMENT                ",a)')trim(surf_pres_comment);
endif

! Always write out the MOST RECENT VALID SURFACE PRES, even if it is FILL
write(testch,'(f8.2)')out_mrvsp;
write(OUT,'(a40,a)')"MOST RECENT VALID SURFACE PRES (DBAR)   ",adjustl(testch);
write(testch,'(i5.3)')out_mrcyc
write(OUT,'(a40,a)')"MOST RECENT VALID SURFACE PRES CYCLE    ",adjustl(testch);
write(OUT,'(a40,a)')"MOST RECENT VALID SURFACE PRES COMMENT  ",trim(out_mrvsp_comment);
    if(do_pres_adj) then;
	if(pres_offset >= 10.0)then
	    write(cpres_offset,'(f5.2)') pres_offset
	elseif(pres_offset >= 0.0)then
	    write(cpres_offset,'(f4.2)') pres_offset
	elseif(pres_offset > -10.0)then
	    write(cpres_offset,'(f5.2)') pres_offset
	else
	    write(cpres_offset,'(f6.2)') pres_offset
	endif
	write(OUT,'("PRESSURE ADJUSTMENT OFFSET              ",a)')trim(cpres_offset);
    endif; ! for  do_pres_adj
    write(OUT,'("PRESSURE ADJUSTMENT COMMENT             ",a)')trim(pres_adj_comment);

if(do_psal_adj) write(OUT,'("SALINITY ADJUSTMENT OFFSET              ",f10.7)')psal_offset;
                            !1234567890123456789012345678901234567890

! Write out the various NON-FLAG indicators

if(prof_on_land) write(OUT,'("POSITION ON LAND INDICATOR              1")')

if(hasMC)then;
    write(OUT,'("MEASUREMENT CODES IN PHY FILE           1")')
else
    write(OUT,'("MEASUREMENT CODES IN PHY FILE           0")')
endif

! How many levels do we have?

write(testch,'(i8)')nlev
write(oline,'(a40,a)')'PROFILE LEVELS                          ',adjustl(testch); write(OUT,'(a)')trim(oline)

if(nlev == 0) go to 20			! Skip adding column names - no levels == no columns

! Column number and nmes
write(testch,'(i3)')ncolout
write(oline,'(a40,a)')'NUMBER OF COLUMNS                       ',adjustl(testch); write(OUT,'(a)')trim(oline)
icolout = 0

do icol = 1,ncolout ! ALL the columns
    write(variable,'(i2,". COLUMN")')icol;
    write(oline,'(a40,a)') variable,trim(outcols(icol)); write(OUT,'(a)')trim(oline)
enddo

! The COLUMN names have been written to the qc file, not write out the qcplus versin number,
! overall flags, then the max flags and frozen indicators

20 continue				! We skipped to here if nlev == 0

write(testch,'(i8)')int_CONFIG_MISSION_NUMBER;
write(oline,'(a40,a)')"CONFIG_MISSION_NUMBER                   ",adjustl(testch);
write(OUT,'(a)')trim(oline); call wrtlog(sub,oline); call wrttrc(sub,oline);
write(oline,'(a40,a)')"CONFIG_MISSION_NUMBER TYPE              ",config_mission_type;
write(OUT,'(a)')trim(oline); call wrtlog(sub,oline); call wrttrc(sub,oline);
write(oline,'(a40,a)')"CONFIG_MISSION_COMMENT                  ",trim(config_mission_comment)
write(OUT,'(a)')trim(oline); call wrtlog(sub,oline); call wrttrc(sub,oline);

write(oline,'(a40,a)')"QCPLUS_VERSION                          ",qcplus_version;
write(OUT,'(a)')trim(oline); call wrtlog(sub,oline); call wrttrc(sub,oline);

! If there are no levels, skip writing out the flag information
if(nlev == 0) go to 25;
oline=write_var_ival("NUMBER_OF_PROFILES",nprf); write(OUT,'(a)')trim(oline);
write(oline,'("MAXIMUM_PROFILE_FLAG                    ",i1,3i2)')(maxprfflag(iprf),iprf=1,nprf); write(OUT,'(a)')trim(oline);
if(hasPRES(0))then;
    oline=write_var_ival('OVERALL FLAG FOR P',overflag(P(2)));write(OUT,'(a)')trim(oline);
    write(oline,'("PROFILE PRES QC                         ",a1,3a2)')(prof_pres_qc(iprf),iprf=1,nprf); write(OUT,'(a)')trim(oline);
endif
if(hasDATA(p1r))then
    if(greyflag(1) > 1)then
	oline=write_var_ival('GREY LIST FLAG FOR P',greyflag(1));write(OUT,'(a)')trim(oline);
	call wrtlog(sub,oline); call wrttrc(sub,oline);
    endif

    do iflg=1,NFLAGPRES
	flag_name = "MAXIMUM " // trim(pres_flg_name(iflg)) // "_" // trim(datname(p1r)) // " FLAG"
	oline=write_var_ival(flag_name,max_flag(p1r,iflg));write(OUT,'(a)')trim(oline);
    enddo
endif
if(do_pres_adj)then
    do iflg=1,NFLAGPRES,2           ! Do not need to do the MAX line for pres increasing for ADJ
	flag_name = "MAXIMUM " // trim(pres_flg_name(iflg)) // "_" // trim(datname(p1a)) // " FLAG"
	oline=write_var_ival(flag_name,max_flag(p1a,iflg));write(OUT,'(a)')trim(oline)
    enddo   ! iflg
endif
if(do_regional_test)then
  if(sea=="MED")then
	 write(oline,'("REGION USED                             MEDITERRAREAN SEA")');
    elseif(sea=="RED")then 
	write(oline,'("REGION USED                             RED SEA")');
    else
	write(oline,'("REGION USED                             UNKNOWN")');
	write(errmsg(1),'("NON-FATAL ERROR 246 in ",a," profile: ",a)')trim(sub),trim(profile)
	write(errmsg(2),'("NON-FATAL ERROR 246 Profile  is identified as REGIONAL, but the SEA is not defined correctly")');
	write(errmsg(3),'("NON-FATAL ERROR 246 The value defined as sea is: ",a)')trim(sea);
	write(errmsg(4),'("NON-FATAL ERROR 246 This should NOT happe!")');
	call error_report(sub,246,4,errmsg); ! write_qcprof REGIONAL profile, wront SEA
    endif
    write(OUT,'(a)')trim(oline);
endif

if(hasTEMP(0))then;
    oline=write_var_ival('OVERALL FLAG FOR T',overflag(T(2)));write(OUT,'(a)')trim(oline);
    write(oline,'("PROFILE TEMP QC                         ",a1,3a2)')(prof_temp_qc(iprf),iprf=1,nprf); write(OUT,'(a)')trim(oline);
endif
if(hasDATA(t1r))then;
    if(greyflag(T(1)) > 1)then
	oline=write_var_ival('GREY LIST FLAG FOR T',greyflag(2));write(OUT,'(a)')trim(oline);
	call wrtlog(sub,oline); call wrttrc(sub,oline);
    endif

    if(do_regional_test)then
	flag_name = "MAXIMUM " // trim(rgnl_flg_name) // "_" // trim(datname(t1r)) // " FLAG"
	oline=write_var_ival(flag_name,max_flag(t1r,regional_ncol));write(OUT,'(a)')trim(oline);
    endif
    do iflg=1,NFLAGTEMP
        flag_name = "MAXIMUM " // trim(temp_flg_name(iflg)) // "_" // trim(datname(t1r)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(t1r,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg
    ! Add the ! DENSITY INVERSION - stored in PSAL flags - report also for TEMP
    flag_name = "MAXIMUM " // trim(psal_flg_name(dengrad_ncol)) // "_" // trim(datname(t1r)) // " FLAG"
    oline=write_var_ival(flag_name,max_flag(s1r,dengrad_ncol));write(OUT,'(a)')trim(oline);		! Use FLAG from PSAL for TEMP

    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_T FLAG',STUCK_T);write(OUT,'(a)')trim(oline);
    oline=write_var_ival('MAXIMUM JUMP CHECK_T FLAG',          JUMPC_T);write(OUT,'(a)')trim(oline);
    oline=write_var_ival('FROZEN PROFILE INDICATOR FOR T',FROZN_T);write(OUT,'(a)')trim(oline);
endif

if(hasPSAL(0))then;
    oline=write_var_ival('OVERALL FLAG FOR S',overflag(S(2)));write(OUT,'(a)')trim(oline);
    write(oline,'("PROFILE PSAL QC                         ",a1,3a2)')(prof_psal_qc(iprf),iprf=1,nprf); write(OUT,'(a)')trim(oline);
endif
if(hasDATA(s1r))then;
    if(greyflag(S(1)) > 1)then
	oline=write_var_ival('GREY LIST FLAG FOR S',greyflag(3));write(OUT,'(a)')trim(oline);
	call wrtlog(sub,oline); call wrttrc(sub,oline);
    endif

    if(do_regional_test)then
        flag_name = "MAXIMUM " // trim(rgnl_flg_name) // "_" // trim(datname(s1r)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(s1r,regional_ncol));write(OUT,'(a)')trim(oline);
    endif
    do iflg=1,NFLAGPSAL
        flag_name = "MAXIMUM " // trim(psal_flg_name(iflg)) // "_" // trim(datname(s1r)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(s1r,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg

    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_S FLAG',STUCK_S);write(OUT,'(a)')trim(oline);
    oline=write_var_ival('MAXIMUM JUMP CHECK_S FLAG',          JUMPC_S);write(OUT,'(a)')trim(oline);
    oline=write_var_ival('FROZEN PROFILE INDICATOR FOR S',FROZN_S);write(OUT,'(a)')trim(oline);
endif

if(do_psal_adj)then
    do iflg=1,NFLAGPSAL
	flag_name = "MAXIMUM " // trim(psal_flg_name(iflg)) // "_" // trim(datname(s1a)) // " FLAG"
	oline=write_var_ival(flag_name,max_flag(s1a,iflg));write(OUT,'(a)')trim(oline)
    enddo   ! iflg
endif
if(hasDOXY(0))then;
    oline=write_var_ival('OVERALL FLAG FOR O',overflag(O(2)));write(OUT,'(a)')trim(oline);
    write(OUT,'(a)')trim(oline);
    write(oline,'("PROFILE DOXY QC                         ",a1,3a2)')(prof_doxy_qc(iprf),iprf=1,nprf);

    if(hasDOXYB)then
	write(oline,'("PROFILE DOXY QC                         ",a1,3a2)')(prof_doxy_qc(iprf),iprf=1,nprf);
	
	if(hasDOXYC)then
	    write(oline,'("PROFILE DOXY QC                         ",a1,3a2)')(prof_doxy_qc(iprf),iprf=1,nprf);
	    
	endif
    endif
endif
if(hasDOXY(1))then;
    do iflg=1,NFLAGDOXY
        flag_name = "MAXIMUM " // trim(doxy_flg_name(iflg)) // "_" // trim(datname(do1)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(do1,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg
    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_O FLAG',STUCK_O);write(OUT,'(a)')trim(oline);
endif;

if(hasDOXY(2))then;
    oline=write_var_ival('OVERALL FLAG FOR O2',overflag((O(2)+1)));write(OUT,'(a)')trim(oline); ! O(2)+1 is start for doxy2
    tqc = prof_qc(4,1); if(tqc == ' ') tqc = 'N';
    write(oline,'("PROFILE DOXY2 QC                        ",a1)')tqc; write(OUT,'(a)')trim(oline);

    do iflg=1,NFLAGDOXY
        flag_name = "MAXIMUM " // trim(doxy_flg_name(iflg)) // "_" // trim(datname(do2)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(do2,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg
    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_O2 FLAG',STUCK_O);write(OUT,'(a)')trim(oline);
endif;

if(hasDOXY(3))then;
    oline=write_var_ival('OVERALL FLAG FOR O2A',overflag((O(2)+2)));write(OUT,'(a)')trim(oline); ! O(2)+2 is start for doxy2a
    tqc = prof_qc(4,1); if(tqc == ' ') tqc = 'N';
    write(oline,'("PROFILE DOXY2A QC                       ",a1)')tqc; write(OUT,'(a)')trim(oline);

    do iflg=1,NFLAGDOXY
        flag_name = "MAXIMUM " // trim(doxy_flg_name(iflg)) // "_" // trim(datname(doa)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(doa,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg
    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_O2 FLAG',STUCK_O);write(OUT,'(a)')trim(oline);
endif;
if(hasDOXY(4))then;
    oline=write_var_ival('OVERALL FLAG FOR O2B',overflag((O(2)+3)));write(OUT,'(a)')trim(oline); ! O(2)+3 is start for doxy2b
    tqc = prof_qc(4,1); if(tqc == ' ') tqc = 'N';
    write(oline,'("PROFILE DOXY2B QC                       ",a1)')tqc; write(OUT,'(a)')trim(oline);

    do iflg=1,NFLAGDOXY
        flag_name = "MAXIMUM " // trim(doxy_flg_name(iflg)) // "_" // trim(datname(dob)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(dob,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg
    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_O2 FLAG',STUCK_O);write(OUT,'(a)')trim(oline);
endif;
if(hasDOXY(5))then;
    oline=write_var_ival('OVERALL FLAG FOR O2C',overflag((O(2)+4)));write(OUT,'(a)')trim(oline); ! O(2)+4 is start for doxy2c
    tqc = prof_qc(4,1); if(tqc == ' ') tqc = 'N';
    write(oline,'("PROFILE DOXY2C QC                       ",a1)')tqc; write(OUT,'(a)')trim(oline);

    do iflg=1,NFLAGDOXY
        flag_name = "MAXIMUM " // trim(doxy_flg_name(iflg)) // "_" // trim(datname(doc)) // " FLAG"
        oline=write_var_ival(flag_name,max_flag(doc,iflg));write(OUT,'(a)')trim(oline);
    enddo       ! iflg
    oline=write_var_ival('MAXIMUM CONSTANT VALUE CHECK_O2 FLAG',STUCK_O);write(OUT,'(a)')trim(oline);
endif;


25 continue ! Skip adding columns and flag analysis if no data

! Separator between header lines  and position block
write(OUT,'(a)')QCSEPLINE

! Now for the POS - position block
! Labels for position columns
write(OUT,'(a)')trim(poslabs)

do ipos=1,npos ! for each position
    if(len(trim(posline(ipos))) == 57)then
	write(oline,'(a,6x,a1,5x,a1,5x,a1,5x,a1)')trim(posline(ipos)),pos_posnqc(ipos),pos_dateqc(ipos), &
		pos_onland(ipos),pos_speedc(ipos);
    else
	write(oline,'(a,5x,a1,5x,a1,5x,a1,5x,a1)')trim(posline(ipos)),pos_posnqc(ipos),pos_dateqc(ipos), &
		pos_onland(ipos),pos_speedc(ipos);
    endif
    if(this_mcode_present) write(oline,'(a,a)')trim(oline),trim(posadnl(ipos));
    write(OUT,'(a)')trim(oline);
    IF(DBLEV== 5)then; write(msg,'(a)')trim(oline); call wrtdbg(sub,msg); endif ! qc test - qc05_impossible_speed
enddo

do ipos=1,npos_traj
    if(DBGPOS)then
	write(DBG,*)'====================================================================='	! DBG
	write(DBG,*)'pos_posnqc_traj(ipos): ',pos_posnqc_traj(ipos)				! DBG
	write(DBG,*)'pos_dateqc_traj(ipos): ',pos_dateqc_traj(ipos)				! DBG
	write(DBG,*)'pos_onland_traj(ipos): ',pos_onland_traj(ipos)				! DBG
	write(DBG,*)'pos_speedc_traj(ipos): ',pos_speedc_traj(ipos)				! DBG
	write(DBG,*)'   posadnl_traj(ipos): ',trim(posadnl_traj(ipos))				! DBG
	write(DBG,*)'====================================================================='	! DBG
    endif

    write(oline,'(x,a,6x,a1,5x,a1,5x,a1,5x,a1,a)')trim(posline_traj(ipos)),pos_posnqc_traj(ipos),pos_dateqc_traj(ipos), &
	pos_onland_traj(ipos),pos_speedc_traj(ipos),trim(posadnl_traj(ipos));
    write(OUT,'(a)')trim(oline);



enddo
    
! Separator between position block and data lines
write(OUT,'(a)')QCSEPLINE

if(nlev == 0)then;
    go to 999;  ! nlev = 0, so skip to THE_END
endif

! Reformat the output line (eliminate double spaces)
! beg new ===========================================================

if(profile_reversed)then
    beg = nlev;
    end = 1;
    inc = -1;
else
    beg = 1;
    end = nlev;
    inc = 1;
endif
odata=' '; oflag=' ';
if(OUTPUT_LEVS)then; 	! Do you want to output a set of levels to the log file
    if(beglev==0 .or. endlev==0)then; beglev=1; endlev=nlev; endif	! Zeroes mean all levels
    write(msg,'(a," OUTPUT_LEVS: ",3i5)')sub,beglev,endlev,inc; call wrtdbg(sub,msg);
endif

if(DBLEV==99 .or. DBLEV>5)then
    HDR = .true.; nch = 0; h1=" ";h2=" ";h3=" ";h4=" ";h5=" ";h6=" "; ic = 0;                          ! LEV99
    write(msg,*)'          p p p p p p p p t t t t s s s s s s s s d d d d d';   call wrtdbg(sub,msg); ! LEV99
    write(msg,*)'hasDATA: ',hasDATA;                                             call wrtdbg(sub,msg); ! LEV99
    write(msg,*)'therest: ',hasPRES,hasTEMP,hasPSAL,hasDOXY,hasCNDC;             call wrtdbg(sub,msg); ! LEV99
    write(msg,*)'          p 1 2 3 4 t 1 2 3 4 s 1 2 3 4 d 1 2 3 4 5 c 1 2 3 4'; call wrtdbg(sub,msg); ! LEV99
endif

ic = 0;
do ilev=beg,end,inc
    odata=' '; oflag=' '; xflag=' ';
    do iprf=1,MAXPRF

	if(hasPRES(iprf))then
	    loc = regPRES(iprf); if(HDR) write(parnam,'("pres",i1)')iprf; ! pres1,pres2,pres...
	    write(odata,'(a,f9.2)') trim(odata),profdata(regPRES(iprf),ilev);
	    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(regPRES(iprf),ilev,i),i=1,NFLAG((regPRES(iprf))));
	    write(xflag,'(a,i2)') trim(xflag),profflag(regPRES(iprf),ilev);

	    if(HDR)then
		    ic=ic+5;	! 1 dat + 1 lev + 3 flags
		if(iprf==1)then
		    h1=trim(h1)//"    -----";  h3="  "//trim(h3)//" -"; h5=trim(h5)//"    "//parnam		! pres
		    h2=trim(h2)//"    "//parnam;h4="  "//trim(h4)//" P"; h6=trim(h6)//"    G I D";		! pres
		else
		    h1=trim(h1)//"    -----";  h3=trim(h3)//" -";       h5=trim(h5)//"  "//parnam		! pres
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" P";       h6=trim(h6)//"  G I D";		! pres
		endif
	    endif
	endif
	! -----------------------------------------------------------------------------------------
	if(hasTEMP(iprf))then
	    loc = regTEMP(iprf); if(HDR) write(parnam,'("temp",i1)')iprf; ! temp1,temp2,temp...
	    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev);
	    if(do_regional_test)then
		write(oflag,'(a,x,i2)') trim(oflag), testflag(loc,ilev,regional_ncol);
		write(oflag,'(a,9i2)') trim(oflag), (testflag(loc,ilev,i),i=1,NFLAG((loc)));
		if(HDR)then;
		    ic=ic+8;	! 1 dat + 1 lev + 1rgn + 5 flags
		    h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"        "//parnam;			! temp
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" T";h6=trim(h6)//"  R G S V A C";			! temp
		endif
	    else
		write(oflag,'(a,x,9i2)') trim(oflag), (testflag(loc,ilev,i),i=1,NFLAG((loc)));
		if(HDR)then;
		    ic=ic+7;	! 1 dat + 1 lev + 5 flag
		    h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"      "//parnam;			! temp
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" T";h6=trim(h6)//"  G S V A C";			! temp
		endif
	    endif
	    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev);
	endif
	! -----------------------------------------------------------------------------------------
	if(hasCNDC(iprf))then
	    loc = iprf;          if(HDR) write(parnam,'("cndc",i1)')iprf; ! cndc1,cndc2,cndc...
            write(odata,'(a,x,f12.5)')trim(odata),profcndc(iprf,ilev);
            if(HDR)then
                    ic= ic + 1          ! 1 data no flags
                    h1=trim(h1)//"        -----";		! Wider than the other variables 		! cndc
                    h2=trim(h2)//"        "//parnam;		! Wider than the other variables		! cndc
            endif
	endif
	! -----------------------------------------------------------------------------------------
	if(hasPSAL(iprf))then

	    loc = regPSAL(iprf); if(HDR) write(parnam,'("psal",i1)')iprf; ! psal1,psal2,psal...
	    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev);
	    if(do_regional_test)then
	        write(oflag,'(a,x,i2)')trim(oflag),testflag(loc,ilev,regional_ncol)
		write(oflag,'(a, 9i2)') trim(oflag), (testflag(loc,ilev,i),i=1,NFLAG((loc)))
		if(HDR)then;
		    ic=ic+9;	! 1 dat + 1 lev + 1 rgnl + 5 flag + 1 DG
		    h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"          "//parnam;		! psal
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" S";h6=trim(h6)//"  R G S V A C D";		! psal
		endif
	    else
		write(oflag,'(a,x,9i2)') trim(oflag), (testflag(loc,ilev,i),i=1,NFLAG((loc)))
		if(HDR)then;
		    ic=ic+8;    ! 1 dat + 1 lev + 5 flag + 1 DG
		    h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"        "//parnam;			! psal
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" S";h6=trim(h6)//"  G S V A C D";			! psal
		endif
	    endif
	    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
	endif

	! -----------------------------------------------------------------------------------------
	if(hasDOXY(0))then
	    if(iprf ==3) go to 30;		! We have no floats with DOXY in prof3
	    if(iprf ==4) go to 30;		! We have no floats with DOXY in prof4
	    if(iprf == 1)then
		if(hasDOXY(1))then
		    loc = do1; if(HDR) parnam = "doxy1";
                    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev)
                    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(loc,ilev,i),i=1,NFLAG((loc)))
                    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
                    if(HDR)then;
                        ic=ic+5; ! 1 data 1 lev 3 flags
                        h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"  "//parnam;         
                        h2=trim(h2)//"    "//parnam;h4=trim(h4)//" O";h6=trim(h6)//"  G S V";   
		    endif
		endif
	    elseif(iprf == 2) then
		if(hasDOXY(2))then
		    loc = do2; if(HDR) parnam = "doxy2";
                    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev)
                    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(loc,ilev,i),i=1,NFLAG((loc)))
                    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
                    if(HDR)then;
                        ic=ic+5; ! 1 data 1 lev 3 flags
                        h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"  "//parnam;         
                        h2=trim(h2)//"    "//parnam;h4=trim(h4)//" O";h6=trim(h6)//"  G S V";   
		    endif
		endif
		if(hasDOXY(3))then
		    loc = doa; if(HDR) parnam = "doxya";
                    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev)
                    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(loc,ilev,i),i=1,NFLAG((loc)))
                    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
                    if(HDR)then;
                        ic=ic+5; ! 1 data 1 lev 3 flags
                        h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"  "//parnam;         
                        h2=trim(h2)//"    "//parnam;h4=trim(h4)//" O";h6=trim(h6)//"  G S V";   
		    endif
		endif
		if(hasDOXY(4))then
		    loc = dob; if(HDR) parnam = "doxyb";
                    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev)
                    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(loc,ilev,i),i=1,NFLAG((loc)))
                    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
                    if(HDR)then;
                        ic=ic+5; ! 1 data 1 lev 3 flags
                        h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"  "//parnam;         
                        h2=trim(h2)//"    "//parnam;h4=trim(h4)//" O";h6=trim(h6)//"  G S V";   
		    endif
		endif
		if(hasDOXY(5))then
		    loc = doc; if(HDR) parnam = "doxyc";
                    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev)
                    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(loc,ilev,i),i=1,NFLAG((loc)))
                    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
                    if(HDR)then;
                        ic=ic+5; ! 1 data 1 lev 3 flags
                        h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"  "//parnam;         
                        h2=trim(h2)//"    "//parnam;h4=trim(h4)//" O";h6=trim(h6)//"  G S V";   
		    endif
		endif
	    endif
	endif
	30 continue
	! -----------------------------------------------------------------------------------------
	if(hasCKSM(iprf))then;
	    write(odata,'(a,i2)') trim(odata),ifix(cksm(iprf,ilev))
	    if(HDR)then;
		    ic=ic+1; ! 1 data no flags
		    h1=trim(h1)//" -";										! cksm
		    h2=trim(h2)//" C";										! cksm
	    endif
	endif
	! -----------------------------------------------------------------------------------------
    enddo
    ! now for the adjusted fields - pres first ====================================================
    do iprf=1,MAXPRF
	! -----------------------------------------------------------------------------------------
	if(hasPRES(iprf) .and. do_pres_adj)then
	    loc = adjPRES(iprf); if(HDR) write(parnam,'("prs",i1,"a")')iprf; ! adj pres1,pres2,pres...
	    write(odata,'(a,f9.2)') trim(odata),profdata(adjPRES(iprf),ilev);
	    write(oflag,'(a,x,9i2)') trim(oflag),(testflag(adjPRES(iprf),ilev,i),i=1,NFLAG((adjPRES(iprf))));
	    write(xflag,'(a,i2)') trim(xflag),profflag(adjPRES(iprf),ilev);

	    if(HDR)then
		ic=ic+5;    ! 1 dat + 1 lev + 3 flags
		h1=trim(h1)//"    -----";  h3=trim(h3)//" -";       h5=trim(h5)//"  "//parnam		! pres
		h2=trim(h2)//"    "//parnam;h4=trim(h4)//" P";       h6=trim(h6)//"  G I D";		! pres
	    endif
	endif
	! -----------------------------------------------------------------------------------------
    enddo
    ! now psal ====================================================================================
    do iprf=1,MAXPRF
	! -----------------------------------------------------------------------------------------
	if(hasPSAL(iprf) .and. do_psal_adj)then
	    loc = adjPSAL(iprf); if(HDR) write(parnam,'("psl",i1,"a")')iprf; ! adj psal1,psal2,psal...
	    write(odata,'(a,f9.3)') trim(odata),profdata(loc,ilev);
	    if(do_regional_test)then
		write(oflag,'(a,x,i2)')trim(oflag),testflag(loc,ilev,regional_ncol)
		write(oflag,'(a, 9i2)') trim(oflag), (testflag(loc,ilev,i),i=1,NFLAG((loc)))
		if(HDR)then;
		    ic=ic+9;    ! 1 dat + 1 lev + 1 rgnl + 5 flag + 1 DG
		    h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"          "//parnam;		! psal
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" S";h6=trim(h6)//"  R G S V A C D";		! psal
		endif
	    else
		write(oflag,'(a,x,9i2)') trim(oflag), (testflag(loc,ilev,i),i=1,NFLAG((loc)))
		if(HDR)then;
		    ic=ic+8;    ! 1 dat + 1 lev + 5 flag + 1 DG
		    h1=trim(h1)//"    -----";h3=trim(h3)//" -";h5=trim(h5)//"        "//parnam;			! psal
		    h2=trim(h2)//"    "//parnam;h4=trim(h4)//" S";h6=trim(h6)//"  G S V A C D";			! psal
		endif
	    endif
	    write(xflag,'(a,i2)') trim(xflag),profflag(loc,ilev)
	endif
	! -----------------------------------------------------------------------------------------
    enddo
    ! ---------------------------------------------------------------------------------------------
    ! If we have an NST ro an STS near_surface profile, add the sngle column at the end of the line
    if(near_surface(0))then
	write(oflag,'(a,i3)') trim(oflag),nsflag(ilev);
	h5 = trim(h5) // "  -"; h6 = trim(h6) // "  N"
    endif
    ! ---------------------------------------------------------------------------------------------

    if(HDR)then	 ! Do we need to write out the header?
	HDR = .false.;	! Only write the header the first time
        write(OUT,'(a,a,a)')trim(h1),trim(h3),trim(h5)
        write(OUT,'(a,a,a)')trim(h2),trim(h4),trim(h6)
    endif
    if(DBLEV==99 .or. DBLEV>5)then							! LEV99 frst/last lines only
	if(ilev <= 5)then
	    write(OUT,'(a,2x,a,2x,a," frst")')trim(odata),trim(xflag),trim(oflag);	! LEV99 frst of frst/last lines only
	elseif(ilev>=(nlev-4))then
	    write(OUT,'(a,2x,a,2x,a," last")')trim(odata),trim(xflag),trim(oflag);	! LEV99 last of frst/last lines only
	endif
    else
	write(OUT,'(a,2x,a,2x,a)')trim(odata),trim(xflag),trim(oflag)
    endif
enddo
if(OUTPUT_LEVS)then
    write(msg,'(" ")'); call wrtdbg(sub,msg);
    write(msg,'(a," OUTPUT_LEVS: ",3i5)')sub,beg,end,inc; call wrtdbg(sub,msg);
    write(msg,'(a)')msg; call wrtdbg(sub,msg); call wrtdbg(sub,msg);
endif

999 continue  ! Here is THE_END

out_open=close_file(OUT,"OUT","New QC file");

! See if an error occurred in an array module
if(filmsg(l:l) == 'R')then;
    filmsg(l:l) = 'y'; call wrtflm(sub,'Normal exit, return y for SUCCESS');
elseif((filmsg(l:l) == 'f') .or. (filmsg(l:l) == 'F'))then
    if(DEBUG) write(msg,'(a,a1,x,a8,x,a)')HEDF,'R',filmsg,'ERROR exit, return F or f'; call wrtdbg(sub,msg);
else
    write(msg,'(a,a1,x,a8,x,a)')HEDF,filmsg(l:l),'W Non-standard file result'; call wrtlog(sub,msg);
endif

write(msg,'(a," output file closed: ",a)')WRTFILE,trim(qcprof); call wrtlog(sub,msg);

write_qcprof = filmsg(l:l);

return          ! Normal return
! ===========================================================================================================
end function write_qcprof
! ===========================================================================================================

character(*) function write_var_ival(var,ival)

use communication                       ! module

implicit none

character(*), intent( in )     :: var
integer,      intent( in )     :: ival

character(len=40) :: c40
character(len=32) :: c32
character(len=72) :: c72

character(len=12), parameter :: sub = 'wrt_var_ival'

write(c40,'(a)')var;
write(c32,'(i1)')ival;
if(c32(1:1) == "*") write(c32,'(i2)')ival;
if(c32(1:1) == "*")then;
    write(errmsg(1),'("FATAL ERROR 903 in write_qcprof")')
    write(errmsg(2),'("FATAL ERROR 903 attempt to write a large integer into i2 specification")')
    write(errmsg(3),'("FATAL ERROR 903 the variable  is: ",a)') trim(var)
    write(errmsg(4),'("FATAL ERROR 903 the int value is: ",i10)') ival
    call error_exit(sub,903,4,errmsg); ! write_qcprof - DRASTIC = ouput string contains * (ie, bad write satement)
    write(c32,'(i10)')ival;
endif
write(c72,'(a40,a32)')adjustl(c40),adjustl(c32)
write_var_ival = trim(c72)

end function write_var_ival
! =========================================================================================================== 
